@@ -24,8 +24,8 @@ namespace osl
 	static CArray<int, 18> table;
 	static int index(Piece piece)
 	{
-	  return ((piece.owner() == BLACK) ? (piece.position().y() - 1) :
-		  (9 - piece.position().y())) + (piece.isPromoted() ? 9 : 0);
+	  return ((piece.owner() == BLACK) ? (piece.square().y() - 1) :
+		  (9 - piece.square().y())) + (piece.isPromoted() ? 9 : 0);
 	}
       public:
 	enum { DIM = 18 };
@@ -37,7 +37,7 @@ namespace osl
 	       i < PtypeTraits<MajorBasic>::indexLimit;
 	       ++i)
 	  {
-	    const Piece piece = state.getPieceOf(i);
+	    const Piece piece = state.pieceOf(i);
 	    if (piece.isOnBoard())
 	      {
 		if (piece.owner() == BLACK)
@@ -92,15 +92,15 @@ namespace osl
 	static int index(const Piece rook, const int pawn_y)
 	{
 	  const int rook_y =
-	    (rook.owner() == BLACK ? rook.position().y() : 10 - rook.position().y());
+	    (rook.owner() == BLACK ? rook.square().y() : 10 - rook.square().y());
 	  return (rook_y - 1) * 10 + pawn_y + (rook.isPromoted() ? 90 : 0);
 	}
-	static int indexY(const Position king,
+	static int indexY(const Square king,
 			  const Piece rook, int pawn_y)
 	{
-	  const int x_diff = std::abs(rook.position().x() - king.x());
+	  const int x_diff = std::abs(rook.square().x() - king.x());
 	  const int rook_y =
-	    (rook.owner() == BLACK ? rook.position().y() : 10 - rook.position().y());
+	    (rook.owner() == BLACK ? rook.square().y() : 10 - rook.square().y());
 	  return x_diff * 10 * 9 + (rook_y - 1) * 10 + pawn_y + (rook.isPromoted() ? 810 : 0);
 	}
 	static CArray<MultiInt, 180> table;
@@ -163,13 +163,13 @@ namespace osl
       protected:
 	template<Player P>
 	static MultiInt evalOne(const NumEffectState& state,
-				 Position rook,
-				 Position myKing,
-				 Position opKing,
-				 Position up,
-				 Position dp,
-				 Position rp,
-				 Position lp,
+				 Square rook,
+				 Square myKing,
+				 Square opKing,
+				 Square up,
+				 Square dp,
+				 Square rp,
+				 Square lp,
 				 bool isP);
 	/**
 	 * (abs_x_diff, y_diff) - 玉を原点とした時の空マスの相対位置
@@ -204,7 +204,7 @@ namespace osl
 	 *          (BLACK,PTYPE_EDGE)もあり得る
 	 * isP - 竜の場合
 	 */
-	static int index1(Position king,Position from,PtypeO ptypeO,bool isP)
+	static int index1(Square king,Square from,PtypeO ptypeO,bool isP)
 	{
 	  int y_diff=from.y()-king.y();
 	  int x_diff=from.x()-king.x();
@@ -233,7 +233,7 @@ namespace osl
 	 * from - 飛車(竜)の位置
 	 * isP - 竜の場合
 	 */
-        static int index2(Position king,Position from,bool isP)
+        static int index2(Square king,Square from,bool isP)
 	{
 	  int y_diff=from.y()-king.y();
 	  int x_diff=from.x()-king.x();
@@ -314,13 +314,13 @@ namespace osl
       protected:
 	template<Player P>
 	static MultiInt evalOne(const NumEffectState& state,
-				 Position bishop,
-				 Position myKing,
-				 Position opKing,
-				 Position ulp,
-				 Position urp,
-				 Position dlp,
-				 Position drp,
+				 Square bishop,
+				 Square myKing,
+				 Square opKing,
+				 Square ulp,
+				 Square urp,
+				 Square dlp,
+				 Square drp,
 				 bool isP);
 	static int index(int x_diff, int y_diff, bool ur, bool promoted)
 	{
@@ -344,7 +344,7 @@ namespace osl
 	 *          (BLACK,PTYPE_EDGE)もあり得る
 	 * isP - 馬の場合
 	 */
-	static int index1(Position king,Position from,PtypeO ptypeO,bool isP)
+	static int index1(Square king,Square from,PtypeO ptypeO,bool isP)
 	{
 	  int y_diff=from.y()-king.y();
 	  int x_diff=from.x()-king.x();
@@ -373,7 +373,7 @@ namespace osl
 	 * from - 角(馬)の位置
 	 * isP - 馬の場合
 	 */
-        static int index2(Position king,Position from,bool isP)
+        static int index2(Square king,Square from,bool isP)
 	{
 	  int y_diff=from.y()-king.y();
 	  int x_diff=from.x()-king.x();
@@ -487,12 +487,12 @@ namespace osl
       private:
 	static CArray<MultiInt, 374544> table;
 	template <Player King>
-	static int index(const Position king, const Piece rook, const Piece bishop)
+	static int index(const Square king, const Piece rook, const Piece bishop)
 	{
-	  const int rook_x = std::abs(king.x() - rook.position().x());
-	  const int bishop_x = std::abs(king.x() - bishop.position().x());
-	  const int rook_y = (King == BLACK ? rook.position().y() - king.y() : king.y() - rook.position().y());
-	  const int bishop_y = (King == BLACK ? bishop.position().y() - king.y() : king.y() - bishop.position().y());
+	  const int rook_x = std::abs(king.x() - rook.square().x());
+	  const int bishop_x = std::abs(king.x() - bishop.square().x());
+	  const int rook_y = (King == BLACK ? rook.square().y() - king.y() : king.y() - rook.square().y());
+	  const int bishop_y = (King == BLACK ? bishop.square().y() - king.y() : king.y() - bishop.square().y());
 	  return bishop_y + 8 + 17 * (bishop_x + 9 * (rook_y + 8 + 17 * (rook_x + 9 * ((bishop.owner() == King ? 1 : 0) + 2 * ((rook.owner() == King ? 1 : 0) + 2 * (2 * (bishop.isPromoted() ? 1 : 0) + (rook.isPromoted() ? 1 : 0)))))));
 	}
       };
@@ -506,7 +506,7 @@ namespace osl
 	static MultiInt eval(const NumEffectState &state);
       private:
 	static void countBetween(const NumEffectState &state,
-				 Position king, Piece bishop,
+				 Square king, Piece bishop,
 				 int &self_count, int &opp_count,
 				 int &total_count);
 	static CArray<MultiInt, 9> self_table;
@@ -557,8 +557,8 @@ namespace osl
 	template <bool SamePlayer, Player P>
 	static int index(Piece rook1, Piece rook2)
 	{
-	  const int y1 = (rook1.isOnBoard() ? rook1.position().y() : 0);
-	  const int y2 = (rook2.isOnBoard() ? rook2.position().y() : 0);
+	  const int y1 = (rook1.isOnBoard() ? rook1.square().y() : 0);
+	  const int y2 = (rook2.isOnBoard() ? rook2.square().y() : 0);
 	  if (SamePlayer)
 	  {
 	    if (P == BLACK)
@@ -631,6 +631,25 @@ namespace osl
       private:
 	static CArray<MultiInt, 32> table;
       };
+
+      class MajorCheckWithCapture
+      {
+      public:
+	enum {
+	  ONE_DIM = PTYPE_SIZE * 2/*bishop or rook*/ * 2 /*promotable*/,
+	  DIM = ONE_DIM * EvalStages
+	};
+	static void setUp(const Weights &weights);
+	static MultiInt eval(const NumEffectState &state);
+      private:
+	static CArray<MultiInt, ONE_DIM> table;	
+	template <Player Owner>
+	static MultiInt addOne(const NumEffectState &state);
+	static size_t index(Ptype ptype, bool is_rook, bool can_promote) 
+	{
+	  return ptype * 4 + is_rook * 2 + can_promote;
+	}
+      };
     }
   }
 }
