@@ -57,17 +57,17 @@ namespace osl
 	{
 	  assert(Owner == king.owner());
 	  const int king_y = (Owner == BLACK ?
-			      king.position().y() : 10 - king.position().y());
-	  return std::abs(x - king.position().x()) * 9 + king_y - 1;
+			      king.square().y() : 10 - king.square().y());
+	  return std::abs(x - king.square().x()) * 9 + king_y - 1;
 	}
-	static int index(const Position king, int x)
+	static int index(const Square king, int x)
 	{
 	  return std::abs(x - king.x());
 	}
 	template <bool Attack>
 	static int indexX(const Piece king, int x)
 	{
-	  const int king_x = king.position().x();
+	  const int king_x = king.square().x();
 	  const int target_x = (king_x > 5 ? 10 - king_x : king_x);
 	  if (king_x >= 6 || (king.owner() == WHITE && king_x == 5))
 	    x = 10 - x;
@@ -117,10 +117,10 @@ namespace osl
 	  }
 
 	  MultiInt result(last_value);
-	  const CArray<Position, 2> king_bw = {{ state.getKingPosition<BLACK>(), state.getKingPosition<WHITE>() }};
-	  const CArray<Position, 2> kings = {{ king_bw[playerToIndex(P)], king_bw[playerToIndex(alt(P))] }};
-	  const CArray<Piece, 2> king_piece = {{ state.getKingPiece(P),
-						 state.getKingPiece(alt(P)) }};
+	  const CArray<Square, 2> king_bw = {{ state.kingSquare<BLACK>(), state.kingSquare<WHITE>() }};
+	  const CArray<Square, 2> kings = {{ king_bw[playerToIndex(P)], king_bw[playerToIndex(alt(P))] }};
+	  const CArray<Piece, 2> king_piece = {{ state.kingPiece(P),
+						 state.kingPiece(alt(P)) }};
 	  if (moved.oldPtype() == PAWN) 
 	  {
 	    if (moved.isDrop())
@@ -223,7 +223,7 @@ namespace osl
 		}
 	      }
 	    }
-	    if (moved.isPromote())
+	    if (moved.isPromotion())
 	    {
 	      const int attack_index = index(kings[1], moved.to().x());
 	      const int defense_index = index(kings[0], moved.to().x());
@@ -362,7 +362,7 @@ namespace osl
 				     defense_index_y, attack_index_x, defense_index_x);
 	      }
 	    }
-	    if (!(moved.ptype() == PPAWN && moved.isPromote())) // promote is already handled above
+	    if (!(moved.ptype() == PPAWN && moved.isPromotion())) // promote is already handled above
 	    {
 	      const int index_x =
 		(moved.to().x() > 5 ? 10 - moved.to().x() : moved.to().x());
@@ -501,19 +501,19 @@ namespace osl
 
       struct PawnAdvanceUtil
       {
-	static int index(Player P, Position pos)
+	static int index(Player P, Square pos)
 	{
 	  return (P == BLACK ? (pos.y() - 1) : (9 - pos.y()));
 	}
 	static bool cantAdvance(const NumEffectState &state, const Piece pawn)
 	{
-	  return cantAdvance(state, pawn.ptypeO(), pawn.position());
+	  return cantAdvance(state, pawn.ptypeO(), pawn.square());
 	}
 	static bool cantAdvance(const NumEffectState &state,
-				const PtypeO ptypeO, const Position position)
+				const PtypeO ptypeO, const Square position)
 	{
 	  assert(getPtype(ptypeO) == PAWN);
-	  return state.getPieceAt(Board_Table.nextPosition(getOwner(ptypeO),
+	  return state.pieceAt(Board_Table.nextSquare(getOwner(ptypeO),
 							   position,
 							   U)).isOnBoardByOwner(getOwner(ptypeO));
 	}
@@ -533,7 +533,7 @@ namespace osl
 	enum { DIM = 9 };
       private:
 	static CArray<MultiInt, 9> table;
-	friend class PawnAdvanceAll;
+	friend struct PawnAdvanceAll;
       public:
 	static void setUp(const Weights &weights,int stage);
 	static MultiInt eval(const NumEffectState &state);
@@ -545,7 +545,7 @@ namespace osl
 	static MultiInt eval(const NumEffectState &state);
       protected:
 	template <Player P>
-	static int indexRetreat(Position pos)
+	static int indexRetreat(Square pos)
 	{
 	  return (P == BLACK ? (pos.y() - 1) : (9 - pos.y()));
 	}
@@ -555,27 +555,27 @@ namespace osl
 	template <Player P>
 	static MultiInt evalOne(const NumEffectState &state,
 				const Piece silver,
-				const CArray<Position, 2> &kings)
+				const CArray<Square, 2> &kings)
 	{
 	  MultiInt result;
 	  if (!canRetreat<P>(state,silver))
 	  {
-	    result += retreat_table[indexRetreat<P>(silver.position())];
+	    result += retreat_table[indexRetreat<P>(silver.square())];
 	  }
-	  const Position up =
-	    silver.position() + DirectionPlayerTraits<U, P>::offset();
+	  const Square up =
+	    silver.square() + DirectionPlayerTraits<U, P>::offset();
 	  if (up.isOnBoard())
 	  {
-	    const Piece up_piece = state.getPieceAt(up);
+	    const Piece up_piece = state.pieceAt(up);
 	    if (up_piece.isEmpty() &&
-		(state.hasEffectByPtype<PAWN>(PlayerTraits<P>::opponent, up) ||
-		 !state.isPawnMaskSet<PlayerTraits<P>::opponent>(silver.position().x())))
+		(state.hasEffectByPtype<PAWN>(alt(P), up) ||
+		 !state.isPawnMaskSet<PlayerTraits<P>::opponent>(silver.square().x())))
 	    {
 	      const int x_diff =
-		std::abs(kings[P].x() - silver.position().x());
+		std::abs(kings[P].x() - silver.square().x());
 	      const int y_diff = (P == BLACK ?
-				  silver.position().y() - kings[P].y() :
-				  kings[P].y() - silver.position().y());
+				  silver.square().y() - kings[P].y() :
+				  kings[P].y() - silver.square().y());
 	      result += head_table[x_diff + 9 * (y_diff + 8)];
 	    }
 	  }
@@ -605,16 +605,16 @@ namespace osl
 	static MultiInt eval(const NumEffectState &state);
       protected:
 	template <Player P>
-	static int indexRetreat(Position pos)
+	static int indexRetreat(Square pos)
 	{
 	  return (P == BLACK ? (pos.y() - 1) : (9 - pos.y()));
 	}
-	static int indexSideX(Position pos)
+	static int indexSideX(Square pos)
 	{
 	  return (pos.x() > 5 ? 9 - pos.x() : pos.x() - 1);
 	}
 	template <Player P>
-	static int indexSideY(Position pos)
+	static int indexSideY(Square pos)
 	{
 	  return (P == BLACK ? (pos.y() - 1) : (9 - pos.y())) + 5;
 	}
@@ -625,15 +625,15 @@ namespace osl
 	static bool canMoveToSide(const NumEffectState &state,
 				  const Piece gold)
 	{
-	  Position r = gold.position() + DirectionPlayerTraits<R, P>::offset();
-	  Position l = gold.position() + DirectionPlayerTraits<L, P>::offset();
+	  Square r = gold.square() + DirectionPlayerTraits<R, P>::offset();
+	  Square l = gold.square() + DirectionPlayerTraits<L, P>::offset();
 	  // check effect is from lesser pieces?
 	  if ((!r.isOnBoard() ||
-	       state.getPieceAt(r).isOnBoardByOwner(gold.owner()) ||
-	       state.hasEffectBy(alt(gold.owner()), r)) &&
+	       state.pieceAt(r).isOnBoardByOwner(gold.owner()) ||
+	       state.hasEffectAt(alt(gold.owner()), r)) &&
 	      (!l.isOnBoard() ||
-	       state.getPieceAt(l).isOnBoardByOwner(gold.owner()) ||
-	       state.hasEffectBy(alt(gold.owner()), l)))
+	       state.pieceAt(l).isOnBoardByOwner(gold.owner()) ||
+	       state.hasEffectAt(alt(gold.owner()), l)))
 	  {
 	    return false;
 	  }
@@ -642,33 +642,30 @@ namespace osl
 	template <Player P>
 	static MultiInt evalOne(const NumEffectState &state,
 				const Piece gold,
-				const CArray<Position, 2> &kings)
+				const CArray<Square, 2> &kings)
 	{
+	  assert(P==gold.owner());
 	  MultiInt result;
 	  if (!canRetreat<P>(state, gold))
 	  {
-	    result += retreat_table[indexRetreat<P>(gold.position())];
+	    result += retreat_table[indexRetreat<P>(gold.square())];
 	  }
 	  if (!canMoveToSide<P>(state, gold))
 	  {
-	    result += side_table[indexSideX(gold.position())] +
-	      side_table[indexSideY<P>(gold.position())];
+	    result += side_table[indexSideX(gold.square())] +
+	      side_table[indexSideY<P>(gold.square())];
 	  }
-	  const Position uur =
-	    Board_Table.nextPosition(gold.owner(), gold.position(), UUR);
-	  const Position uul =
-	    Board_Table.nextPosition(gold.owner(), gold.position(), UUL);
-	  if ((uul.isOnBoard() && state.getPieceAt(uul).isEmpty() &&
-	       !state.hasEffectBy(gold.owner(), uul)) ||
-	      (uur.isOnBoard() && state.getPieceAt(uur).isEmpty() &&
-	       !state.hasEffectBy(gold.owner(), uur)))
+	  const Square uur = gold.square().neighbor<P,UUR>();
+	  const Square uul = gold.square().neighbor<P,UUL>();
+	  if ((state.pieceAt(uul).isEmpty() && !state.hasEffectAt(P, uul))
+	      || (state.pieceAt(uur).isEmpty() && !state.hasEffectAt(P, uur)))
 	  {
-	    assert(state.getKingPosition(gold.owner()) == kings[gold.owner()]);
-	    const Position king = kings[gold.owner()];
-	    const int x_diff = std::abs(king.x() - gold.position().x());
-	    const int y_diff = (gold.owner() == BLACK ?
-				gold.position().y() - king.y() :
-				king.y() - gold.position().y());
+	    assert(state.kingSquare(gold.owner()) == kings[gold.owner()]);
+	    const Square king = kings[P];
+	    const int x_diff = std::abs(king.x() - gold.square().x());
+	    const int y_diff = (P == BLACK ?
+				gold.square().y() - king.y() :
+				king.y() - gold.square().y());
 	    result += knight_table[x_diff + 9 * (y_diff + 8)];
 	  }
 	  return result;
@@ -705,7 +702,7 @@ namespace osl
 	enum { DIM = 9 };
       private:
 	static CArray<MultiInt, 9> table;
-	static int index(Player P, Position pos)
+	static int index(Player P, Square pos)
 	{
 	  return (P == BLACK ? (pos.y() - 1) : (9 - pos.y()));
 	}
@@ -748,9 +745,9 @@ namespace osl
 	static CArray<MultiInt, 144> table;
 	static int index(const Piece piece)
 	{
-	  return index(piece.owner(), piece.ptype(), piece.position());
+	  return index(piece.owner(), piece.ptype(), piece.square());
 	}
-	static int index(const Player player, const Ptype ptype, const Position pos)
+	static int index(const Player player, const Ptype ptype, const Square pos)
 	{
 	  const int y = (player == BLACK ? pos.y() : 10 - pos.y()) - 1;
 	  return ptype * 9 + y;
@@ -770,9 +767,9 @@ namespace osl
 	static CArray<MultiInt, 80> table;
 	static int index(const Piece piece)
 	{
-	  return index(piece.owner(), piece.ptype(), piece.position());
+	  return index(piece.owner(), piece.ptype(), piece.square());
 	}
-	static int index(const Player, const Ptype ptype, const Position pos)
+	static int index(const Player, const Ptype ptype, const Square pos)
 	{
 	  const int x = (pos.x() > 5 ? 10 - pos.x() : pos.x()) - 1;
 	  return ptype * 5 + x;
@@ -789,16 +786,16 @@ namespace osl
 	template <Player Defense>
 	static bool canCheck(const NumEffectState &state)
 	{
-	  const Position king = state.getKingPosition<Defense>();
+	  const Square king = state.kingSquare<Defense>();
 	  const Player offense = PlayerTraits<Defense>::opponent;
-	  const Position ul =
+	  const Square ul =
 	    king + DirectionPlayerTraits<UUL, Defense>::offset();
-	  const Position ur =
+	  const Square ur =
 	    king + DirectionPlayerTraits<UUR, Defense>::offset();
 	  if (ul.isOnBoard())
 	  {
-	    const Piece p = state.getPieceAt(ul);
-	    if (!state.hasEffectBy<Defense>(ul) &&
+	    const Piece p = state.pieceAt(ul);
+	    if (!state.hasEffectAt<Defense>(ul) &&
 		((p.isEmpty() && state.hasPieceOnStand<KNIGHT>(offense)) ||
 		 (!p.isOnBoardByOwner<offense>() &&
 		  state.hasEffectByPtypeStrict<KNIGHT>(offense, ul))))
@@ -806,8 +803,8 @@ namespace osl
 	  }
 	  if (ur.isOnBoard())
 	  {
-	    const Piece p = state.getPieceAt(ur);
-	    if (!state.hasEffectBy<Defense>(ur) &&
+	    const Piece p = state.pieceAt(ur);
+	    if (!state.hasEffectAt<Defense>(ur) &&
 		((p.isEmpty() && state.hasPieceOnStand<KNIGHT>(offense)) ||
 		 (!p.isOnBoardByOwner<offense>() &&
 		  state.hasEffectByPtypeStrict<KNIGHT>(offense, ur))))
@@ -904,7 +901,7 @@ namespace osl
 			    const PieceMask promoted,
 			    MultiInt &result);
 	template <bool attack, Player owner>
-	static int indexY(const Position king, int x_diff)
+	static int indexY(const Square king, int x_diff)
 	{
 	  const int y = (owner == BLACK ? king.y() : 10 - king.y());
 	  return x_diff + (y - 1) * 9 + (attack ? 0 : 81);
@@ -938,8 +935,8 @@ namespace osl
 	    (has_support ? 0 : PTYPE_SIZE * 2);
 	}
 	template <bool Attack>
-	static int indexK(Position king, bool same_turn, bool has_support,
-			  Position position, Player owner, Ptype ptype)
+	static int indexK(Square king, bool same_turn, bool has_support,
+			  Square position, Player owner, Ptype ptype)
 	{
 	  const int x_diff = std::abs(position.x() - king.x());
 	  const int y_diff = (owner == BLACK ?
@@ -950,11 +947,11 @@ namespace osl
 	    y_diff + 8 + (Attack ? 0 : 9792);
 	}
 	template <bool Attack>
-	static int indexK(Position king, bool same_turn, bool has_support,
+	static int indexK(Square king, bool same_turn, bool has_support,
 			  Piece piece)
 	{
 	  return indexK<Attack>(king, same_turn, has_support,
-				piece.position(), piece.owner(),
+				piece.square(), piece.owner(),
 				piece.ptype());
 	}
 
@@ -966,20 +963,20 @@ namespace osl
 	  int moved_piece_number,
 	  MultiIntPair &result)
 	{
-	  CArray<Position, 2> kings = {{ state.getKingPosition<BLACK>(),
-					 state.getKingPosition<WHITE>() }};
+	  CArray<Square, 2> kings = {{ state.kingSquare<BLACK>(),
+					 state.kingSquare<WHITE>() }};
 	  // old without, new with
-	  PieceMask black_old = (~effected_mask[PlayerTraits<Attacked>::opponent]) & new_mask[PlayerTraits<Attacked>::opponent] & state.getOnBoardMask(Attacked);
+	  PieceMask black_old = (~effected_mask[alt(Attacked)]) & new_mask[alt(Attacked)] & state.piecesOnBoard(Attacked);
 	  black_old.reset(moved_piece_number);
 	  while (black_old.any())
 	  {
-	    const Piece piece = state.getPieceOf(black_old.takeOneBit());
+	    const Piece piece = state.pieceOf(black_old.takeOneBit());
 	    const bool has_support =
 	      new_mask[Attacked].test(piece.number());
 	    const int index_king_black_turn_attack =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == BLACK, has_support, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == BLACK, has_support, piece);
 	    const int index_king_white_turn_attack =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == WHITE, has_support, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == WHITE, has_support, piece);
 	    const int index_king_black_turn_defense =
 	      indexK<false>(kings[Attacked], Attacked == BLACK, has_support, piece);
 	    const int index_king_white_turn_defense =
@@ -991,17 +988,17 @@ namespace osl
 	  }
 
 	  // old with, new without
-	  PieceMask black_new = effected_mask[PlayerTraits<Attacked>::opponent] & (~new_mask[PlayerTraits<Attacked>::opponent]) & state.getOnBoardMask(Attacked);
+	  PieceMask black_new = effected_mask[alt(Attacked)] & (~new_mask[alt(Attacked)]) & state.piecesOnBoard(Attacked);
 	  black_new.reset(moved_piece_number);
 	  while (black_new.any())
 	  {
-	    const Piece piece = state.getPieceOf(black_new.takeOneBit());
+	    const Piece piece = state.pieceOf(black_new.takeOneBit());
 	    const bool has_support =
-	      effected_mask[PlayerTraits<Attacked>::index].test(piece.number());
+	      effected_mask[Attacked].test(piece.number());
 	    const int index_king_black_turn_attack =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == BLACK, has_support, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == BLACK, has_support, piece);
 	    const int index_king_white_turn_attack =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == WHITE, has_support, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == WHITE, has_support, piece);
 	    const int index_king_black_turn_defense =
 	      indexK<false>(kings[Attacked], Attacked == BLACK, has_support, piece);
 	    const int index_king_white_turn_defense =
@@ -1012,24 +1009,24 @@ namespace osl
 	      result);
 	  }
 	  // old with, new with, self with, self without
-	  PieceMask black_self_old = effected_mask[PlayerTraits<Attacked>::opponent] & new_mask[PlayerTraits<Attacked>::opponent] &
-	    effected_mask[Attacked] & (~new_mask[Attacked]) & state.getOnBoardMask(Attacked);
+	  PieceMask black_self_old = effected_mask[alt(Attacked)] & new_mask[alt(Attacked)] &
+	    effected_mask[Attacked] & (~new_mask[Attacked]) & state.piecesOnBoard(Attacked);
 	  black_self_old.reset(moved_piece_number);
 	  while (black_self_old.any())
 	  {
-	    const Piece piece = state.getPieceOf(black_self_old.takeOneBit());
+	    const Piece piece = state.pieceOf(black_self_old.takeOneBit());
 	    const int index_king_black_turn_attack =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == BLACK, false, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == BLACK, false, piece);
 	    const int index_king_white_turn_attack =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == WHITE, false, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == WHITE, false, piece);
 	    const int index_king_black_turn_defense =
 	      indexK<false>(kings[Attacked], Attacked == BLACK, false, piece);
 	    const int index_king_white_turn_defense =
 	      indexK<false>(kings[Attacked], Attacked == WHITE, false, piece);
 	    const int index_king_black_turn_attack_old =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == BLACK, true, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == BLACK, true, piece);
 	    const int index_king_white_turn_attack_old =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == WHITE, true, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == WHITE, true, piece);
 	    const int index_king_black_turn_defense_old =
 	      indexK<false>(kings[Attacked], Attacked == BLACK, true, piece);
 	    const int index_king_white_turn_defense_old =
@@ -1044,24 +1041,24 @@ namespace osl
 	      result);
 	  }
 	  // old with, new with, self without, self with
-	  PieceMask black_self_new = effected_mask[PlayerTraits<Attacked>::opponent] & new_mask[PlayerTraits<Attacked>::opponent] &
-	    (~effected_mask[Attacked]) & new_mask[Attacked] & state.getOnBoardMask(Attacked);
+	  PieceMask black_self_new = effected_mask[alt(Attacked)] & new_mask[alt(Attacked)] &
+	    (~effected_mask[Attacked]) & new_mask[Attacked] & state.piecesOnBoard(Attacked);
 	  black_self_new.reset(moved_piece_number);
 	  while (black_self_new.any())
 	  {
-	    const Piece piece = state.getPieceOf(black_self_new.takeOneBit());
+	    const Piece piece = state.pieceOf(black_self_new.takeOneBit());
 	    const int index_king_black_turn_attack =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == BLACK, true, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == BLACK, true, piece);
 	    const int index_king_white_turn_attack =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == WHITE, true, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == WHITE, true, piece);
 	    const int index_king_black_turn_defense =
 	      indexK<false>(kings[Attacked], Attacked == BLACK, true, piece);
 	    const int index_king_white_turn_defense =
 	      indexK<false>(kings[Attacked], Attacked == WHITE, true, piece);
 	    const int index_king_black_turn_attack_old =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == BLACK, false, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == BLACK, false, piece);
 	    const int index_king_white_turn_attack_old =
-	      indexK<true>(kings[PlayerTraits<Attacked>::opponent], Attacked == WHITE, false, piece);
+	      indexK<true>(kings[alt(Attacked)], Attacked == WHITE, false, piece);
 	    const int index_king_black_turn_defense_old =
 	      indexK<false>(kings[Attacked], Attacked == BLACK, false, piece);
 	    const int index_king_white_turn_defense_old =
@@ -1163,34 +1160,34 @@ namespace osl
 				  MultiIntPair &result)
 	{
 	  // old without, new with
-	  PieceMask old = (~effected_mask[PlayerTraits<P>::opponent]) & new_mask[PlayerTraits<P>::opponent] & state.getOnBoardMask(P);
+	  PieceMask old = (~effected_mask[alt(P)]) & new_mask[alt(P)] & state.piecesOnBoard(P);
 	  old.reset(p.number());
 	  while (old.any())
 	  {
-	    const Piece piece = state.getPieceOf(old.takeOneBit());
+	    const Piece piece = state.pieceOf(old.takeOneBit());
 	    const bool has_support =
-	      new_mask[PlayerTraits<P>::index].test(piece.number());
+	      new_mask[P].test(piece.number());
 	    PieceMask attacking =
-	      state.getEffect(piece.position()) &
-	      state.getOnBoardMask(PlayerTraits<P>::opponent);
+	      state.effectSetAt(piece.square()) &
+	      state.piecesOnBoard(alt(P));
 	    attacked_mask[piece.number()] = attacking;
 	    while (attacking.any())
 	    {
-	      const Piece attack = state.getPieceOf(attacking.takeOneBit());
+	      const Piece attack = state.pieceOf(attacking.takeOneBit());
 	      evalOnePiece<P == BLACK>(P, piece.ptype(), attack.ptype(),
 				       has_support, result);
 	    }
 	  }
 	  // old with, new without
-	  PieceMask new_without = effected_mask[PlayerTraits<P>::opponent] & (~new_mask[PlayerTraits<P>::opponent]) & state.getOnBoardMask(P);
+	  PieceMask new_without = effected_mask[alt(P)] & (~new_mask[alt(P)]) & state.piecesOnBoard(P);
 	  new_without.reset(p.number());
 	  while (new_without.any())
 	  {
-	    const Piece piece = state.getPieceOf(new_without.takeOneBit());
+	    const Piece piece = state.pieceOf(new_without.takeOneBit());
 	    const bool has_support =
-	      effected_mask[PlayerTraits<P>::index].test(piece.number());
+	      effected_mask[P].test(piece.number());
 	    PieceMask attacking = attacked_mask[piece.number()];
-	    if (moved.isPromote() && attacking.test(p.number()))
+	    if (moved.isPromotion() && attacking.test(p.number()))
 	    {
 	      evalOnePiece<P != BLACK>(P, piece.ptype(), moved.oldPtype(),
 				       has_support, result);
@@ -1204,21 +1201,21 @@ namespace osl
 	    }
 	    while (attacking.any())
 	    {
-	      const Piece attack = state.getPieceOf(attacking.takeOneBit());
+	      const Piece attack = state.pieceOf(attacking.takeOneBit());
 	      evalOnePiece<P != BLACK>(P, piece.ptype(), attack.ptype(),
 				       has_support, result);
 	    }
 	  }
 	  // old with, new with, self with, self without
-	  PieceMask self_old = effected_mask[PlayerTraits<P>::opponent] &
-	    new_mask[PlayerTraits<P>::opponent] &
-	    effected_mask[P] & (~new_mask[P]) & state.getOnBoardMask(P);
+	  PieceMask self_old = effected_mask[alt(P)] &
+	    new_mask[alt(P)] &
+	    effected_mask[P] & (~new_mask[P]) & state.piecesOnBoard(P);
 	  self_old.reset(p.number());
 	  while (self_old.any())
 	  {
-	    const Piece piece = state.getPieceOf(self_old.takeOneBit());
+	    const Piece piece = state.pieceOf(self_old.takeOneBit());
 	    PieceMask old_attacking = attacked_mask[piece.number()];
-	    if (moved.isPromote() && old_attacking.test(p.number()))
+	    if (moved.isPromotion() && old_attacking.test(p.number()))
 	    {
 	      evalOnePiece<P != BLACK>(P, piece.ptype(), moved.oldPtype(),
 				       true, result);
@@ -1232,30 +1229,30 @@ namespace osl
 	    }
 	    while (old_attacking.any())
 	    {
-	      const Piece attack = state.getPieceOf(old_attacking.takeOneBit());
+	      const Piece attack = state.pieceOf(old_attacking.takeOneBit());
 	      evalOnePiece<P != BLACK>(P, piece.ptype(), attack.ptype(),
 				       true, result);
 	    }
-	    PieceMask new_attacking = state.getEffect(piece.position())
-	      & state.getOnBoardMask(PlayerTraits<P>::opponent);
+	    PieceMask new_attacking = state.effectSetAt(piece.square())
+	      & state.piecesOnBoard(alt(P));
 	    attacked_mask[piece.number()] = new_attacking;
 	    while (new_attacking.any())
 	    {
-	      const Piece attack = state.getPieceOf(new_attacking.takeOneBit());
+	      const Piece attack = state.pieceOf(new_attacking.takeOneBit());
 	      evalOnePiece<P == BLACK>(P, piece.ptype(), attack.ptype(),
 				       false, result);
 	    }
 	  }
 	  // old with, new with, self without, self with
-	  PieceMask self_new_with = effected_mask[PlayerTraits<P>::opponent] &
-	    new_mask[PlayerTraits<P>::opponent] &
-	    (~effected_mask[P]) & new_mask[P] & state.getOnBoardMask(P);
+	  PieceMask self_new_with = effected_mask[alt(P)] &
+	    new_mask[alt(P)] &
+	    (~effected_mask[P]) & new_mask[P] & state.piecesOnBoard(P);
 	  self_new_with.reset(p.number());
 	  while (self_new_with.any())
 	  {
-	    const Piece piece = state.getPieceOf(self_new_with.takeOneBit());
+	    const Piece piece = state.pieceOf(self_new_with.takeOneBit());
 	    PieceMask old_attacking = attacked_mask[piece.number()];
-	    if (moved.isPromote() && old_attacking.test(p.number()))
+	    if (moved.isPromotion() && old_attacking.test(p.number()))
 	    {
 	      evalOnePiece<P != BLACK>(P, piece.ptype(), moved.oldPtype(),
 				       false, result);
@@ -1269,16 +1266,16 @@ namespace osl
 	    }
 	    while (old_attacking.any())
 	    {
-	      const Piece attack = state.getPieceOf(old_attacking.takeOneBit());
+	      const Piece attack = state.pieceOf(old_attacking.takeOneBit());
 	      evalOnePiece<P != BLACK>(P, piece.ptype(), attack.ptype(),
 				       false, result);
 	    }
-	    PieceMask new_attacking = state.getEffect(piece.position())
-	      & state.getOnBoardMask(PlayerTraits<P>::opponent);
+	    PieceMask new_attacking = state.effectSetAt(piece.square())
+	      & state.piecesOnBoard(alt(P));
 	    attacked_mask[piece.number()] = new_attacking;
 	    while (new_attacking.any())
 	    {
-	      const Piece attack = state.getPieceOf(new_attacking.takeOneBit());
+	      const Piece attack = state.pieceOf(new_attacking.takeOneBit());
 	      evalOnePiece<P == BLACK>(P, piece.ptype(), attack.ptype(),
 				       true, result);
 	    }
@@ -1287,16 +1284,16 @@ namespace osl
 	  PieceMask effected = effected_mask[P];
 	  effected ^= new_mask[P];
 	  effected = ~effected;
-	  PieceMask attack_changed = effected_mask[PlayerTraits<P>::opponent] &
-	    new_mask[PlayerTraits<P>::opponent] &
-	    effected & state.getOnBoardMask(P) &
-	    state.effectedChanged(PlayerTraits<P>::opponent);
+	  PieceMask attack_changed = effected_mask[alt(P)] &
+	    new_mask[alt(P)] &
+	    effected & state.piecesOnBoard(P) &
+	    state.effectedChanged(alt(P));
 	  attack_changed.reset(p.number());
 	  while (attack_changed.any())
 	  {
-	    const Piece attacked = state.getPieceOf(attack_changed.takeOneBit());
+	    const Piece attacked = state.pieceOf(attack_changed.takeOneBit());
 	    PieceMask attack_old_mask = attacked_mask[attacked.number()];
-	    PieceMask attack_new_mask = state.getEffect(attacked.position()) & state.getOnBoardMask(PlayerTraits<P>::opponent);
+	    PieceMask attack_new_mask = state.effectSetAt(attacked.square()) & state.piecesOnBoard(alt(P));
 	    if (captured_number != -1 &&
 		attack_old_mask.test(captured_number))
 	    {
@@ -1306,7 +1303,7 @@ namespace osl
 				       result);
 	      attack_old_mask.reset(captured_number);
 	    }
-	    if (moved.isPromote() &&
+	    if (moved.isPromotion() &&
 		attack_old_mask.test(p.number()))
 	    {
 	      evalOnePiece<P != BLACK>(P, attacked.ptype(),
@@ -1315,7 +1312,7 @@ namespace osl
 				       result);
 	      attack_old_mask.reset(p.number());
 	    }
-	    if (moved.isPromote() &&
+	    if (moved.isPromotion() &&
 		attack_new_mask.test(p.number()))
 	    {
 	      evalOnePiece<P == BLACK>(P, attacked.ptype(),
@@ -1327,7 +1324,7 @@ namespace osl
 	    PieceMask gone = attack_old_mask & (~attack_new_mask);
 	    while (gone.any())
 	    {
-	      const Piece attacking = state.getPieceOf(gone.takeOneBit());
+	      const Piece attacking = state.pieceOf(gone.takeOneBit());
 	      evalOnePiece<P != BLACK>(P, attacked.ptype(),
 				       attacking.ptype(),
 				       effected_mask[P].test(attacked.number()),
@@ -1336,14 +1333,14 @@ namespace osl
 	    PieceMask added = (~attack_old_mask) & attack_new_mask;
 	    while (added.any())
 	    {
-	      const Piece attacking = state.getPieceOf(added.takeOneBit());
+	      const Piece attacking = state.pieceOf(added.takeOneBit());
 	      evalOnePiece<P == BLACK>(P, attacked.ptype(),
 				       attacking.ptype(),
 				       new_mask[P].test(attacked.number()),
 				       result);
 	    }
 
-	    attacked_mask[attacked.number()] = state.getEffect(attacked.position()) & state.getOnBoardMask(PlayerTraits<P>::opponent);
+	    attacked_mask[attacked.number()] = state.effectSetAt(attacked.square()) & state.piecesOnBoard(alt(P));
 	  }
 	}
 
@@ -1364,9 +1361,9 @@ namespace osl
 	static int index1(const NumEffectState &state, Piece piece)
 	{
 	  const Ptype attack_ptype
-	    = state.findCheapThreat(alt(piece.owner()), piece.position()).ptype();
-	  const bool has_support = state.hasEffectBy(piece.owner(),
-						     piece.position());
+	    = state.findCheapAttack(alt(piece.owner()), piece.square()).ptype();
+	  const bool has_support = state.hasEffectAt(piece.owner(),
+						     piece.square());
 	  return (piece.ptype() + 
 		  (has_support ? 0 : PTYPE_SIZE)) * PTYPE_SIZE + attack_ptype;
 	}
@@ -1526,7 +1523,7 @@ namespace osl
 	static void setUp(const Weights &weights);
 	static MultiInt eval(const NumEffectState &state);
       private:
-	static int index(Player p, Position pos, Position king,
+	static int index(Player p, Square pos, Square king,
 			 PtypeO ptypeO, bool attack)
 	{
 	  const int y_diff = (p == BLACK ? king.y() - pos.y() : pos.y() - king.y());
@@ -1576,7 +1573,7 @@ namespace osl
 			      const CArray2d<int, 2, 3> &gs_count);
       private:
 	template <Player Defense>
-	static int index(const Position king, int distance0, int count)
+	static int index(const Square king, int distance0, int count)
 	{
 	  int king_x = (king.x() > 5 ? 10 - king.x() : king.x());
 	  int king_y = (Defense == WHITE ? 10 - king.y() : king.y());
@@ -1586,7 +1583,7 @@ namespace osl
 	static MultiInt evalOne(const NumEffectState &state,
 				 const CArray2d<int, 2, 3> &gs_count);
 	template <Player Defense>
-	static int indexCombination(const Position king, int count0,
+	static int indexCombination(const Square king, int count0,
 			     int count1, int count2)
 	{
 	  int king_x = (king.x() > 5 ? 10 - king.x() : king.x());
@@ -1630,15 +1627,15 @@ namespace osl
       {
 	static std::pair<int,int> matchRook(const NumEffectState& state, Piece rook,
 					    const CArray<bool,2>& has_silver,
-					    Position& silver_drop);
+					    Square& silver_drop);
 	static std::pair<int,int> matchGold(const NumEffectState& state, Piece gold, 
 					    const CArray<bool,2>& has_silver,
-					    Position& silver_drop);
+					    Square& silver_drop);
       public:
 	enum { ONE_DIM = 5*2, DIM = ONE_DIM * EvalStages };
 	static void setUp(const Weights &weights);
 	static MultiIntPair eval(const NumEffectState& state,
-				 CArray<std::pair<Position,int>,2>& silver_drop);
+				 CArray<std::pair<Square,int>,2>& silver_drop);
 	static CArray<MultiInt, ONE_DIM> table;
       };
       class BishopRookFork
@@ -1648,26 +1645,26 @@ namespace osl
 	  DROP_DIM = PTYPE_SIZE*PTYPE_SIZE, 
 	  ONE_DIM = 2*DROP_DIM*2, DIM = ONE_DIM * EvalStages 
 	};
-	static const Position isBishopForkPosition(const NumEffectState& state, Player defense, const Position a, const Position b);
-	static const Position isRookForkPosition(const NumEffectState& state, Player defense, const Position a, const Position b);
+	static const Square isBishopForkSquare(const NumEffectState& state, Player defense, const Square a, const Square b, bool maybe_empty=false);
+	static const Square isRookForkSquare(const NumEffectState& state, Player defense, const Square a, const Square b);
 	static int bishopIndex(Ptype a, Ptype b) { return a * PTYPE_SIZE + b; }
 	static int rookIndex(Ptype a, Ptype b) { return bishopIndex(a,b) + DROP_DIM; }
 	static void setUp(const Weights &weights);
 	template <Player Defense>
 	static MultiIntPair evalOne(const NumEffectState& state, const PieceVector& target,
-				    std::pair<Position,int>& bishop_drop,
-				    std::pair<Position,int>& rook_drop);
+				    std::pair<Square,int>& bishop_drop,
+				    std::pair<Square,int>& rook_drop);
 	static MultiIntPair eval(const NumEffectState& state,
-				 CArray<std::pair<Position,int>,2>& bishop_drop,
-				 CArray<std::pair<Position,int>,2>& rook_drop);
+				 CArray<std::pair<Square,int>,2>& bishop_drop,
+				 CArray<std::pair<Square,int>,2>& rook_drop);
 	static CArray<MultiInt, ONE_DIM> table;
       private:
-	static const Position findDropInLine
+	static const Square findDropInLine
 	(const NumEffectState& state, Player defense, 
-	 const Position a, const Position b, Piece king);
+	 const Square a, const Square b, Piece king);
 	static bool testCenter(const NumEffectState& state, Player defense, 
-			       const Position a, const Position b, Piece king,
-			       Position center);
+			       const Square a, const Square b, Piece king,
+			       Square center, bool maybe_empty=false);
       };
 
       class KnightFork
@@ -1680,18 +1677,50 @@ namespace osl
 	static void setUp(const Weights &weights);
 	template <Player Defense>
 	static MultiIntPair evalOne(const NumEffectState& state,
-				    const CArray<PieceVector,10>& pieces, bool has_knight,
-				    std::pair<Position,int>& knight_drop);
+				    bool has_knight,
+				    BoardMask& knight_fork_squares, 
+				    std::pair<Square,int>& knight_drop);
 	static MultiIntPair eval(const NumEffectState& state, 
-				 CArray<std::pair<Position,int>,2>& knight_drop);
+				 CArray<BoardMask,2>& knight_fork_squares,
+				 CArray<std::pair<Square,int>,2>& knight_drop);
+	template <Player P>
+	static MultiIntPair evalWithUpdate(const NumEffectState& state, 
+					   Move moved,
+					   CArray<BoardMask,2>& knight_fork_squares,
+					   CArray<std::pair<Square,int>,2>& knight_drop);
 	static CArray<MultiInt, ONE_DIM> table;
 
-	static bool isForkPosition(const NumEffectState& state, Player defense, 
+	static bool isForkSquare(const NumEffectState& state, Player defense, 
 				   int y, int x0, int x1);
 	static int index(Ptype a, Ptype b)
 	{
 	  return a * PTYPE_SIZE + b;
 	}
+	static bool isTarget(Ptype ptype) 
+	{
+	  ptype = unpromote(ptype);
+	  return ptype != PAWN && ptype != LANCE && ptype != KNIGHT;
+	}
+      private:
+	template <Player P, Player Defense>
+	static void updateSquares
+	(const NumEffectState& state, Move moved,
+	 BoardMask& knight_fork_squares);
+	template <osl::Player Defense>
+	static MultiIntPair accumulate
+	(const NumEffectState& state,
+	 bool has_knight, const BoardMask& knight_fork_squares,
+	 std::pair<Square,int>& knight_drop);
+      };
+
+      class SilverAdvance26
+      {
+      public:
+	enum { ONE_DIM = 1, DIM = ONE_DIM*EvalStages };
+	static void setUp(const Weights &weights);
+	static MultiInt eval(const NumEffectState &state);
+      private:
+	static CArray<MultiInt, ONE_DIM> table;
       };
 
     }
