@@ -8,6 +8,7 @@
 #include "osl/eval/ml/pieceEval.h"
 #include "osl/eval/ml/evalStagePair.h"
 #include "osl/eval/evalTraits.h"
+#include "osl/eval/pieceEval.h"
 #include "osl/state/numEffectState.h"
 #include "osl/progress/progress32.h"
 #include "osl/progress/effect5x3.h"
@@ -25,6 +26,11 @@ namespace osl
     namespace ml
     {
       using namespace osl::progress::ml;
+      class OpenMidEndingPtypeTable : public PtypeEvalTable
+      {
+      public:
+	OpenMidEndingPtypeTable();
+      };
 
       struct OpenMidEndingEvalDebugInfo
       {
@@ -61,55 +67,61 @@ namespace osl
       {
       private:
 	enum { INVALID=EvalTraits<BLACK>::MAX_VALUE+1 };
+	enum {
+	  /** one should attack king after when he captured almost all pieces */
+	  ProgressIndependentValueLimit = 4000
+	};
 	enum LoadStatus { Zero=0, Loaded, Random };
 	static volatile LoadStatus initialized_flag;
 	static Weights piece_pair_weights;
-	mutable int cache;
 #ifdef USE_TEST_PROGRESS
-	osl::progress::ml::NewProgress progress;
+	typedef osl::progress::ml::NewProgress progress_t;
 #else
-	osl::progress::Effect5x3 progress;
+	typedef osl::progress::Effect5x3 progress_t;
 #endif
-	CArray<PieceMask, 2> pin_mask;
-	CArray<PieceMask, 2> effect25; // index: owner of king
-	CArray<PieceMask, 2> effect25_supported; // index: owner of king
-	CArray<int, 5>  black_vertical, white_vertical,
-	  black_king_vertical, white_king_vertical;
-	// flat
-	int progress_independent_value, // should be renamed to piece
-	  recalculated_value, piece_pair_value;
-	CArray<int,2> piece_pair_king_value;
+	progress_t progress;
+	MultiIntPair kingx_blocked, king25_effect_each;
+	MultiIntPair king25_both_side,king_rook_bishop;
+	MultiIntPair piece_stand_turn, non_pawn_attacked,
+	  non_pawn_attacked_ptype, piece_fork_turn;
 	MultiInt ptypey, ptypex, king_table_value;
 	MultiInt piece_stand_value, recalculated_stage_value, pawn_advance;
 	MultiInt rook_mobility, bishop_mobility, lance_mobility;
-	MultiIntPair kingx_blocked, king25_effect_each;
-	MultiIntPair king25_both_side,king_rook_bishop;
-	int black_pawn_count;
-	int black_major_count, black_gold_count;
-	CArray<int, 2> non_pawn_stand_count;
-	CArray2d<int, 2, 3> gs_near_king_count;
-	int black_attack_effect, black_attack_piece,
-	  white_attack_effect, white_attack_piece,
-	  black_attack_supported_piece, white_attack_supported_piece;
-	int black_defense_effect, black_defense_piece,
-	  white_defense_effect, white_defense_piece;
 	MultiInt knight_advance, pawn_drop, promoted_minor_piece, rook_pawn,
 	  rook_effect, bishop_effect, bishop_head, nosupport, ptype_yy, king3pieces;
 	MultiInt rook_promote_defense;
 	MultiInt piece_stand_combination, piece_stand_y, knight_check,
 	  knight_head, pawn_ptypeo, ptype_count_value, lance_effect_piece,
 	  ptype_y_pawn_y, bishop_and_king;
-	MultiIntPair piece_stand_turn, non_pawn_attacked,
-	  non_pawn_attacked_ptype, piece_fork_turn;
+	CArray<BoardMask, 2> knight_fork_squares;
+	CArray<PieceMask, 2> effect25; // index: owner of king
+	CArray<PieceMask, 2> effect25_supported; // index: owner of king
 	CArray<PieceMask, 2> effected_mask;
 	CArray<PieceMask, 2> effected_mask_for_attacked;
 	CArray<PieceMask, 40> attacked_mask;
+	CArray<int, 5>  black_vertical, white_vertical,
+	  black_king_vertical, white_king_vertical;
+	// flat
+	CArray<int,2> piece_pair_king_value;
+	CArray<int, 2> non_pawn_stand_count;
+	CArray2d<int, 2, 3> gs_near_king_count;
 	CArray2d<int, 2, PTYPE_SIZE> ptype_count, ptype_board_count;
-	CArray<std::pair<Position,int>, 2> knight_drop, silver_drop, bishop_drop, rook_drop;
+	CArray<std::pair<Square,int>, 2> knight_drop, silver_drop, bishop_drop, rook_drop;
 	CArray2d<int, 2, 9> pawns;
+	int progress_independent_value, // should be renamed to piece
+	  recalculated_value, piece_pair_value;
+	int black_pawn_count;
+	int black_major_count, black_gold_count;
+	int black_attack_effect, black_attack_piece,
+	  white_attack_effect, white_attack_piece,
+	  black_attack_supported_piece, white_attack_supported_piece;
+	int black_defense_effect, black_defense_piece,
+	  white_defense_effect, white_defense_piece;
+	mutable int cache;
 	Player turn;
 	unsigned int ptypeo_mask;
 	CArray<bool, 2> can_check; // king is defense
+	bool use_progress_independent_value_limit;
 	static const int ROUND_UP = 2;
 	static int roundUp(int v)
 	{
@@ -117,18 +129,18 @@ namespace osl
 	}
 	void updateGoldSilverNearKing(const NumEffectState &state)
 	{
-	  const CArray<Position,2> kings = {{ 
-	      state.getKingPosition(BLACK),
-	      state.getKingPosition(WHITE),
+	  const CArray<Square,2> kings = {{ 
+	      state.kingSquare(BLACK),
+	      state.kingSquare(WHITE),
 	    }};
 	  gs_near_king_count.fill(0);
 	  for (int i = PtypeTraits<GOLD>::indexMin;
 	       i < PtypeTraits<GOLD>::indexLimit; ++i)
 	  {
-	    const Piece p = state.getPieceOf(i);
+	    const Piece p = state.pieceOf(i);
 	    if (p.isOnBoard())
 	    {
-	      const Position pos = p.position();
+	      const Square pos = p.square();
 	      const int y_diff = std::abs(pos.y() - kings[p.owner()].y());
 	      const int x_diff = std::abs(pos.x() - kings[p.owner()].x());
 	      if (y_diff <= 2 && x_diff <= 3)
@@ -140,10 +152,10 @@ namespace osl
 	  for (int i = PtypeTraits<SILVER>::indexMin;
 	       i < PtypeTraits<SILVER>::indexLimit; ++i)
 	  {
-	    const Piece p = state.getPieceOf(i);
+	    const Piece p = state.pieceOf(i);
 	    if (p.isOnBoard())
 	    {
-	      const Position pos = p.position();
+	      const Square pos = p.square();
 	      const int y_diff = std::abs(pos.y() - kings[p.owner()].y());
 	      const int x_diff = std::abs(pos.x() - kings[p.owner()].x());
 	      if (y_diff <= 2 && x_diff <= 3)
@@ -168,6 +180,22 @@ namespace osl
 	    + piece_pair_king_value[BLACK] + piece_pair_king_value[WHITE];
 	}
 	void debug() const;
+	MultiInt stageValue() const 
+	{
+	  return king_table_value + piece_stand_value +
+	    king25_effect_each[BLACK] + king25_effect_each[WHITE] +
+	    ptypex + ptypey + rook_mobility + bishop_mobility + lance_mobility +
+	    rook_effect + bishop_effect +
+	    piece_stand_combination + piece_stand_turn[turn] +
+	    rook_pawn + pawn_drop + piece_stand_y + knight_check +
+	    pawn_advance + pawn_ptypeo + promoted_minor_piece +
+	    nosupport +
+	    non_pawn_attacked[turn] + non_pawn_attacked_ptype[turn] +
+	    ptype_yy + king3pieces + bishop_head + knight_head
+	    + rook_promote_defense +
+	    ptype_count_value + lance_effect_piece + ptype_y_pawn_y +
+	    bishop_and_king + piece_fork_turn[turn] + recalculated_stage_value;
+	}
 	int openingValue() const 
 	{
 	  return (king_table_value[0] +
@@ -313,11 +341,30 @@ namespace osl
 		  recalculated_stage_value[EndgameIndex]);
 	}
 	void invalidateCache() { cache=INVALID; }
+	static int progressIndependentValueAdjusted(int value, int progress,
+						    int progress_max)
+	{
+	  if (value > ProgressIndependentValueLimit) {
+	    int diff = value - ProgressIndependentValueLimit;
+	    value = ProgressIndependentValueLimit
+	      + diff * progress/progress_max;
+	  }
+	  else if (value < -ProgressIndependentValueLimit) {
+	    int diff = value + ProgressIndependentValueLimit;
+	    value = -ProgressIndependentValueLimit
+	      + diff * progress/progress_max;
+	  }
+	  return value;
+	}
 	int composeOpenMidEndgame() const
 	{
 	  const int progress_max = NewProgress::maxProgress(), c = progress_max/2;
 	  const int progress = this->progress.progress();
-	  int sum = progressIndependentValue() * progress_max;
+	  int progress_independent = use_progress_independent_value_limit
+	    ? progressIndependentValueAdjusted
+	    (progressIndependentValue(), progress, progress_max)
+	    : progressIndependentValue();
+	  int sum = progress_independent * progress_max;
 	  if (progress < c) 
 	  {
 	    sum += openingValue() * 2*(c - progress);
@@ -340,21 +387,24 @@ namespace osl
 	  const int w2 = progress_max - c1;
 #endif
 	  assert(c0 == w2);
-	  int sum = progressIndependentValue() * c0;
+	  int progress_independent = progressIndependentValueAdjusted
+	    (progressIndependentValue(), progress, progress_max);
+	  int sum = progress_independent * c0;
+	  const MultiInt stage_sum = stageValue();
 	  if (progress < c0) 
 	  {
-	    sum += openingValue() * (c0 - progress);
-	    sum += midgameValue() * progress;
+	    sum += stage_sum[0] * (c0 - progress);
+	    sum += stage_sum[1] * progress;
 	  }
 	  else if (progress < c1) 
 	  {
-	    sum += midgameValue()  * (c1 - progress);
-	    sum += midgame2Value() * (progress-c0);
+	    sum += stage_sum[1] * (c1 - progress);
+	    sum += stage_sum[2] * (progress-c0);
 	  }
 	  else 
 	  {
-	    sum += midgame2Value() * (progress_max - progress);
-	    sum += endgameValue()  * (progress - c1);
+	    sum += stage_sum[2] * (progress_max - progress);
+	    sum += stage_sum[3] * (progress - c1);
 	  }
 	  return sum;
 	}
@@ -374,8 +424,8 @@ namespace osl
 #    error "not supported"
 #  else
 	    cache = roundUp(progressIndependentValue() * 16 +
-			 openingValue() * (16 - progress.progress16().value()) + 
-			 endgameValue() * progress.progress16().value());
+			    openingValue() * (16 - progress.progress16().value()) + 
+			    endgameValue() * progress.progress16().value());
 #  endif
 #endif
 	  }
@@ -383,7 +433,7 @@ namespace osl
 	}
 	const Move suggestMove(const NumEffectState& state) const
 	{
-	  assert(turn == state.getTurn());
+	  assert(turn == state.turn());
 	  Move suggest;
 	  int best_value = 0;
 	  if (! rook_drop[turn].first.isPieceStand()) {
@@ -422,10 +472,6 @@ namespace osl
 			    + progress.progress16(WHITE).value()); 
 	}
 	const Progress16 progress16() const { return progress.progress16(); }
-	const PieceMask pins(Player player) const
-	{
-	  return pin_mask[playerToIndex(player)];
-	}
       public:
 	static int infty()
 	{
@@ -460,6 +506,7 @@ namespace osl
 
 	OpenMidEndingEvalDebugInfo debugInfo(const NumEffectState &state);
 	static void setRandom();
+	static const OpenMidEndingPtypeTable Piece_Value;
       };
     }
   }
