@@ -20,11 +20,11 @@ namespace osl
     namespace psn
     {
       const Move strToMove(const std::string&, const state::SimpleState&);
-      const Position strToPos(const std::string&);
+      const Square strToPos(const std::string&);
       Ptype charToPtype(char);
 
       const std::string show(Move);
-      const std::string show(Position);
+      const std::string show(Square);
       char show(Ptype);
 
       /** decorate capture by 'x', promote by '+', and unpromote by '=' */
