@@ -18,7 +18,6 @@
 #include "osl/move_classifier/check_.h"
 #include "osl/move_classifier/moveAdaptor.h"
 #include "osl/move_classifier/pawnDropCheckmate.h"
-#include "osl/apply_move/applyMoveWithPath.h"
 #include "osl/record/csa.h"
 #include "osl/container/moveVector.h"
 #ifdef USE_TBB_HASH
@@ -253,9 +252,9 @@ namespace osl
     int attackProofCost(Player attacker, const NumEffectState& state, Move move)
     {
       int proof = 0;
-      if (move.capturePtype() == PTYPE_EMPTY)
+      if (! move.isCapture())
       {
-	const Position from=move.from(), to=move.to();
+	const Square from=move.from(), to=move.to();
 	const int a = (state.countEffect(attacker,to) 
 		       + (from.isPieceStand() ? 1 : 0));
 	int d = state.countEffect(alt(attacker),to);
@@ -420,7 +419,7 @@ struct osl::checkmate::Dfpn::Tree
   {
     return state.inCheck(P);
   }
-  const Piece king(Player P) const { return state.getKingPiece(P); }
+  const Piece king(Player P) const { return state.kingPiece(P); }
   void newVisit(Player P, Move move, const HashKey& next_hash)
   {
     assert(P == move.player());
@@ -756,6 +755,7 @@ List::findProofOracle(const HashKey& key, PieceStand white_stand, Move last_move
       continue;
     if (attack_stand.isSuperiorOrEqualTo(record.proofPieces())) {
       result.setFrom(record);
+      ++record.node_count;
       if (record.last_move == last_move)
 	break;
     }
@@ -857,7 +857,7 @@ DfpnTable::find(const HashKey& key, int subindex)
     return 0;
   return &it->second;
 #else
-  Table::const_iterator p = table[subindex].find(key.boardKey());
+  Table::iterator p = table[subindex].find(key.boardKey());
   if (p == table[subindex].end())
     return 0;
   return &p->second;
@@ -1100,7 +1100,7 @@ struct osl::checkmate::Dfpn::CallAttack
   CallAttack(Dfpn *s) : search(s)
   {
   }
-  void operator()(Position) const 
+  void operator()(Square) const 
   {
     search->attack<P>();
   }
@@ -1113,7 +1113,7 @@ struct osl::checkmate::Dfpn::CallDefense
   CallDefense(Dfpn *s) : search(s)
   {
   }
-  void operator()(Position) const 
+  void operator()(Square) const 
   {
     search->defense<P>();
   }
@@ -1149,7 +1149,7 @@ void osl::checkmate::Dfpn::setIllegal(const HashKey& key, PieceStand white_stand
   // こちらが重要
   DfpnRecord result(key.blackStand(), white_stand);
   result.proof_disproof = ProofDisproof::NoCheckmate();
-  result.setDisproofPieces(key.isPlayerOfTurn(BLACK) ? key.blackStand() : white_stand);
+  result.setDisproofPieces((table->attack() == WHITE) ? key.blackStand() : white_stand);
   table->store(key, result);
 }
 
@@ -1179,7 +1179,7 @@ Dfpn::hasCheckmateMove(const NumEffectState& state, const HashKey& key,
 
   Node& root = tree->node[0];
   try {
-    tree->state = state;
+    tree->state.copyFrom(state);
     tree->depth = 0;
     root.hash_key = key;
     root.path = path;
@@ -1187,7 +1187,7 @@ Dfpn::hasCheckmateMove(const NumEffectState& state, const HashKey& key,
     root.threshold = ProofDisproof(ROOT_PROOF_TOL, ROOT_DISPROOF_TOL);
     root.white_stand = PieceStand(WHITE, state);
     root.moved = last_move;
-    if (state.getTurn() == BLACK)
+    if (state.turn() == BLACK)
       attack<BLACK>();
     else
       attack<WHITE>();
@@ -1248,7 +1248,7 @@ Dfpn::tryProofMain(const NumEffectState& state, const HashKey& key,
     return ProofDisproof();
   path_table->clear();
   
-  tree->state = state;
+  tree->state.copyFrom(state);
   node_count = tree->depth = 0;
   node_count_limit = root_proof_simulation_limit;
 
@@ -1260,7 +1260,7 @@ Dfpn::tryProofMain(const NumEffectState& state, const HashKey& key,
   root.white_stand = PieceStand(WHITE, state);
   root.moved = last_move;
 
-  root.record = (state.getTurn() == BLACK) 
+  root.record = (state.turn() == BLACK) 
     ? table->probe<BLACK>(root.hash_key, root.white_stand)
     : table->probe<WHITE>(root.hash_key, root.white_stand);
   if (root.record.proof_disproof.isFinal() || root.record.tried_oracle > oracle_id) {
@@ -1269,7 +1269,7 @@ Dfpn::tryProofMain(const NumEffectState& state, const HashKey& key,
   }
 
   try {
-    if (state.getTurn() == BLACK)
+    if (state.turn() == BLACK)
       proofOracleAttack<BLACK,UseTable>(oracle);
     else
       proofOracleAttack<WHITE,UseTable>(oracle);
@@ -1300,7 +1300,7 @@ Dfpn::hasEscapeMove(const NumEffectState& state,
 		    size_t limit, Move last_move)
 {
   assert(table);
-  if (! state.hasEffectBy(alt(state.getTurn()), state.getKingPosition(state.getTurn())))
+  if (! state.hasEffectAt(alt(state.turn()), state.kingSquare(state.turn())))
     return ProofDisproof::NoCheckmate();
   if (! table)
     return ProofDisproof();
@@ -1310,7 +1310,7 @@ Dfpn::hasEscapeMove(const NumEffectState& state,
 
   Node& root = tree->node[0];  
   try {
-    tree->state = state;
+    tree->state.copyFrom(state);
     tree->depth = 0;
     root.hash_key = key;
     root.path = path;
@@ -1318,14 +1318,14 @@ Dfpn::hasEscapeMove(const NumEffectState& state,
     root.clear();
     root.threshold = ProofDisproof(ROOT_PROOF_TOL, ROOT_DISPROOF_TOL);
     root.white_stand = PieceStand(WHITE, state);
-    if (state.getTurn() == BLACK)
+    if (state.turn() == BLACK)
       defense<WHITE>();
     else
       defense<BLACK>();
 
     if (root.record.need_full_width) {
       root.clear();
-      if (state.getTurn() == BLACK)
+      if (state.turn() == BLACK)
 	defense<WHITE>();
       else
 	defense<BLACK>();
@@ -1356,7 +1356,7 @@ namespace osl
       const NumEffectState *state;
       move_compare(const NumEffectState& s) : state(&s)
       {
-	assert(Turn == state->getTurn());
+	assert(Turn == state->turn());
       }
       tuple_t convert(Move m) const
       {
@@ -1364,8 +1364,12 @@ namespace osl
 	const int d = state->countEffect(alt(Turn), m.to());
 	const int to_y = playerToMul(Turn)*m.to().y();
 	const int to_x = (5 - abs(5-m.to().x()))*2 + (m.to().x() > 5);
-	const int from_to = (to_y*16+to_x)*256+m.from().index();
-	return boost::make_tuple(a > d, from_to, m.isPromote());
+	int from_to = (to_y*16+to_x)*256;
+	if (m.isDrop())
+          from_to += m.ptype();
+        else
+          from_to += m.from().index();
+	return boost::make_tuple(a > d, from_to, m.isPromotion());
       }
       bool operator()(Move l, Move r) const
       {
@@ -1411,22 +1415,19 @@ Dfpn::generateCheck(const NumEffectState& state, DfpnMoveVector& moves, bool &ha
   }
   else
   {
-    {
-      move_action::Store store(moves);
-      move_generator::AddEffectWithEffect<move_action::Store>::template generate<P,true>
-	(state,state.getKingPiece(alt(P)).position(),store,has_pawn_checkmate);
+    move_action::Store store(moves);
+    move_generator::AddEffectWithEffect<move_action::Store>::template generate<P,true>
+      (state,state.kingPiece(alt(P)).square(),store,has_pawn_checkmate);
+  }
+  BOOST_FOREACH(Move move, moves)
+  {
+    if(move.hasIgnoredUnpromote<P>()){
+      if(Ptype_Table.getEffect(unpromote(move.ptypeO()),move.to(),
+			       state.kingSquare(alt(P))).hasEffect()
+	 || (state.pinOrOpen(alt(P)).test
+	     (state.pieceAt(move.from()).number())))
+	moves.push_back(move.unpromote());
     }
-    BOOST_FOREACH(Move move, moves)
-    {
-      if(move.hasIgnoredUnpromote<P>()){
-	if(Ptype_Table.getEffect(unpromote(move.ptypeO()),move.to(),
-				 state.getKingPosition(alt(P))).hasEffect()
-	   || (state.pinOrOpen(alt(P)).test
-	       (state.getPieceAt(move.from()).number())))
-	  moves.push_back(move.unpromote());
-      }
-    }
-    
   }
   sort<P>(state, moves);
 }
@@ -1526,8 +1527,8 @@ Dfpn::attack()
 #endif
 #ifdef CHECKMATE_A3_GOLD
       || (record.proof_disproof == ProofDisproof(1,1) && tree->state.hasPieceOnStand<GOLD>(P)
-	  && (tree->king(alt(P)).position().x() <= 3 || tree->king(alt(P)).position().x() >= 7
-	      || tree->king(alt(P)).position().template positionForBlack<P>().y() <= 3))
+	  && (tree->king(alt(P)).square().x() <= 3 || tree->king(alt(P)).square().x() >= 7
+	      || tree->king(alt(P)).square().template squareForBlack<P>().y() <= 3))
 #endif
     )
   {
@@ -1591,7 +1592,7 @@ Dfpn::attack()
   {
     node.allocate(node.moves.size());
     const King8Info info_modified 
-      = Edge_Table.resetEdgeFromLiberty(alt(P), tree->king(alt(P)).position(), King8Info(tree->state.Iking8Info(alt(P))));
+      = Edge_Table.resetEdgeFromLiberty(alt(P), tree->king(alt(P)).square(), King8Info(tree->state.Iking8Info(alt(P))));
     for (size_t i=0; i<node.moves.size(); ++i) {
 #ifdef MEMORIZE_SOLVED_IN_BITSET
       if (record.solved & (1ull << i))
@@ -1710,12 +1711,12 @@ Dfpn::attack()
 #endif
 	    if (node.moves[i].isDrop()
 		|| (isMajor(node.moves[i].ptype())
-		    && node.moves[i].capturePtype() == PTYPE_EMPTY
-		    && ! node.moves[i].isPromote() && isPromoted(node.moves[i].ptype())
-		    && ! tree->state.hasEffectBy(alt(P), node.moves[i].to()))) {
+		    && ! node.moves[i].isCapture()
+		    && ! node.moves[i].isPromotion() && isPromoted(node.moves[i].ptype())
+		    && ! tree->state.hasEffectAt(alt(P), node.moves[i].to()))) {
 	      const EffectContent e
 		= Ptype_Table.getEffect(node.moves[i].ptypeO(),
-					Offset32(tree->king(alt(P)).position(), node.moves[i].to()));
+					Offset32(tree->king(alt(P)).square(), node.moves[i].to()));
 	      if (! e.hasUnblockableEffect()) {
 		size_t *target = &max_drop_disproof_lance;
 		if (unpromote(node.moves[i].ptype()) == ROOK)
@@ -1786,7 +1787,8 @@ Dfpn::attack()
 				   - (sum_disproof - node.children[next_i].disproof()));
     CallDefense<P> helper(this);
     tree->depth += 1;
-    ApplyMoveWithPath<P>::doUndoMove(tree->state, next.path, next.moved, helper);
+    next.path.pushMove(next.moved);
+    tree->state.makeUnmakeMove(Player2Type<P>(), next.moved, helper);
     tree->depth -= 1;
     node.children[next_i] = next.record;
     node.children_path[next_i] = next.path_record;
@@ -1835,15 +1837,15 @@ Dfpn::attack()
 template <osl::Player P> 
 void osl::checkmate::
 Dfpn::generateEscape(const NumEffectState& state, bool need_full_width, 
-		     Position last_to, DfpnMoveVector& moves)
+		     Square last_to, DfpnMoveVector& moves)
 {
   assert(moves.empty());
   const Player AltP=PlayerTraits<P>::opponent;
 #ifdef GRAND_PARENT_DELAY
-  const bool delay_node = last_to != Position() 
-      && state.hasEffectBy(alt(P), last_to)
-      && (state.hasEffectNotBy(alt(P), state.getKingPiece(alt(P)), last_to)
-	  || ! state.hasEffectBy(P, last_to));
+  const bool delay_node = last_to != Square() 
+      && state.hasEffectAt(alt(P), last_to)
+      && (state.hasEffectNotBy(alt(P), state.kingPiece(alt(P)), last_to)
+	  || ! state.hasEffectAt(P, last_to));
   if (delay_node)
   {
     DfpnMoveVector all;
@@ -1901,7 +1903,7 @@ Dfpn::grandParentSimulationSuitable() const
     const Move alm = node.moved; // attacker's last move
     const Move dlm = parent.moved; // defense's last move
     const Move alm2 = gparent.moved; // attacker's second-last move
-    if (dlm.isNormal() && alm.to() == dlm.to() && dlm.capturePtype() == PTYPE_EMPTY
+    if (dlm.isNormal() && alm.to() == dlm.to() && ! dlm.isCapture()
 	&& alm2.isNormal() && alm2.to() == alm.from()) {
       return true;
     }
@@ -1958,10 +1960,10 @@ Dfpn::defense()
   if (record.proof_disproof.isFinal())
     return;
   const bool grand_parent_simulation = grandParentSimulationSuitable();
-  if (record.last_to == Position())
-    record.last_to = grand_parent_simulation ? node.moved.to() : tree->king(alt(P)).position();
-  const Position grand_parent_delay_last_to
-    = (record.last_to != tree->king(alt(P)).position()) ? record.last_to : Position();
+  if (record.last_to == Square())
+    record.last_to = grand_parent_simulation ? node.moved.to() : tree->king(alt(P)).square();
+  const Square grand_parent_delay_last_to
+    = (record.last_to != tree->king(alt(P)).square()) ? record.last_to : Square();
 
   generateEscape<P>(tree->state, record.need_full_width, grand_parent_delay_last_to, node.moves);
   if (node.moves.empty() && ! record.need_full_width) {
@@ -2172,7 +2174,7 @@ Dfpn::defense()
 	    }
 	    else
 #endif
-	    if (node.moves[i].isDrop() && !tree->state.hasEffectBy(alt(P), node.moves[i].to())) {
+	    if (node.moves[i].isDrop() && !tree->state.hasEffectAt(alt(P), node.moves[i].to())) {
 	      max_drop_proof = std::max(max_drop_proof, proof);
 	      proof = SacrificeBlockCount;
 	    }
@@ -2194,7 +2196,7 @@ Dfpn::defense()
       if (false_branch_candidate && ! node.children[i].proof_disproof.isFinal()
 	  && (node.children[i].node_count == 0
 	      || ! node.children[i].best_move.isNormal()
-	      || ! (node.moves[i].ptype() == KING && node.moves[i].capturePtype() == PTYPE_EMPTY)))
+	      || ! (node.moves[i].ptype() == KING && ! node.moves[i].isCapture())))
 	false_branch_candidate = false;
       max_proof = std::max(max_proof, proof);
 #endif
@@ -2278,7 +2280,8 @@ Dfpn::defense()
 					    (size_t)node.threshold.disproof()));
     CallAttack<P> helper(this);
     tree->depth += 1;
-    ApplyMoveWithPath<PlayerTraits<P>::opponent>::doUndoMove(tree->state, next.path, node.moves[next_i], helper);
+    next.path.pushMove(node.moves[next_i]);
+    tree->state.makeUnmakeMove(Player2Type<PlayerTraits<P>::opponent>(), node.moves[next_i], helper);
     tree->depth -= 1;
     if (parallel_shared && parallel_shared->data[thread_id].restart) {
       if (tree->depth == 0)
@@ -2323,7 +2326,7 @@ Dfpn::defense()
   }
 }
 
-#ifndef MINIMAL
+#if (!defined MINIMAL) || (defined DFPNSTATONE)
 void osl::checkmate::
 Dfpn::analyze(const PathEncoding& path_src, 
 	      const NumEffectState& src, const vector<Move>& moves) const
@@ -2334,7 +2337,7 @@ Dfpn::analyze(const PathEncoding& path_src,
   for (size_t i=0; i<moves.size(); ++i) {
     if (! state.isAlmostValidMove(moves[i]))
       break;
-    ApplyMoveOfTurn::doMove(state, moves[i]);
+    state.makeMove(moves[i]);
     key = key.newMakeMove(moves[i]);
     path.pushMove(moves[i]);
     DfpnRecord record = table->probe(key, PieceStand(WHITE, state));
@@ -2351,17 +2354,17 @@ Dfpn::analyze(const PathEncoding& path_src,
     }
     std::cerr << "\n";
     DfpnMoveVector moves;
-    if (state.getTurn() == table->attack()) {
+    if (state.turn() == table->attack()) {
       bool has_pawn_checkmate=false;
-      if (state.getTurn() == BLACK)
+      if (state.turn() == BLACK)
 	generateCheck<BLACK>(state, moves, has_pawn_checkmate);
       else
 	generateCheck<WHITE>(state, moves, has_pawn_checkmate);
     }    
     else {
-      const Position grand_parent_delay_last_to
-	= (record.last_to != state.getKingPosition(state.getTurn())) ? record.last_to : Position();
-      if (state.getTurn() == BLACK)
+      const Square grand_parent_delay_last_to
+	= (record.last_to != state.kingSquare(state.turn())) ? record.last_to : Square();
+      if (state.turn() == BLACK)
 	generateEscape<WHITE>(state, true, grand_parent_delay_last_to, moves);
       else
 	generateEscape<BLACK>(state, true, grand_parent_delay_last_to, moves);
@@ -2393,7 +2396,7 @@ struct osl::checkmate::Dfpn::CallProofOracleAttack
   CallProofOracleAttack(Dfpn *s, const ProofOracle& o) : search(s), oracle(o)
   {
   }
-  void operator()(Position) const 
+  void operator()(Square) const 
   {
     search->proofOracleAttack<P,UseTable>(oracle);
   }
@@ -2407,7 +2410,7 @@ struct osl::checkmate::Dfpn::CallProofOracleDefense
   CallProofOracleDefense(Dfpn *s, const ProofOracle& o) : search(s), oracle(o)
   {
   }
-  void operator()(Position) const 
+  void operator()(Square) const 
   {
     search->proofOracleDefense<P,UseTable>(oracle);
   }
@@ -2512,7 +2515,8 @@ Dfpn::proofOracleAttack(const ProofOracle& key)
 
     CallProofOracleDefense<P,UseTable> helper(this, key.newOracle(P, check_move));
     tree->depth += 1;
-    ApplyMoveWithPath<P>::doUndoMove(tree->state, next.path, next.moved, helper);
+    next.path.pushMove(next.moved);
+    tree->state.makeUnmakeMove(Player2Type<P>(), next.moved, helper);
     tree->depth -= 1;
     node.children[0] = next.record;
     node.children_path[0] = next.path_record;
@@ -2568,10 +2572,10 @@ Dfpn::proofOracleDefense(const ProofOracle& key)
     return;
   // move generation
   const bool grand_parent_simulation = grandParentSimulationSuitable();
-  if (record.last_to == Position())
-    record.last_to = grand_parent_simulation ? node.moved.to() : tree->king(alt(P)).position();
-  const Position grand_parent_delay_last_to
-    = (record.last_to != tree->king(alt(P)).position()) ? record.last_to : Position();
+  if (record.last_to == Square())
+    record.last_to = grand_parent_simulation ? node.moved.to() : tree->king(alt(P)).square();
+  const Square grand_parent_delay_last_to
+    = (record.last_to != tree->king(alt(P)).square()) ? record.last_to : Square();
   generateEscape<P>(tree->state, true, grand_parent_delay_last_to, node.moves);
   if (node.moves.empty()) {
     record.setProofPieces(ProofPieces::leaf(tree->state, P, record.stands[P]));
@@ -2600,7 +2604,6 @@ Dfpn::proofOracleDefense(const ProofOracle& key)
 	FixedDepthSearcher fixed_searcher(tree->state);
 	PieceStand proof_pieces;
 	Move check_move;
-	DfpnRecord old_child = node.children[i];
 	node.children[i].proof_disproof
 	  = fixed_searcher.hasEscapeByMove<P>(node.moves[i], 0, check_move, proof_pieces);
 	if (node.children[i].proof_disproof.isCheckmateSuccess()) {	  
@@ -2651,9 +2654,9 @@ Dfpn::proofOracleDefense(const ProofOracle& key)
 	break;
       continue;
     }
-    const Position next_to = node.moves[next_i].to();
-    if (sum_proof && tree->state.hasEffectBy(P, next_to)
-	&& (! tree->state.hasEffectBy(alt(P), next_to)
+    const Square next_to = node.moves[next_i].to();
+    if (sum_proof && tree->state.hasEffectAt(P, next_to)
+	&& (! tree->state.hasEffectAt(alt(P), next_to)
 	    || (tree->state.countEffect(alt(P), next_to) == 1
 		&& ! node.moves[next_i].isDrop())))
       continue;
@@ -2666,7 +2669,8 @@ Dfpn::proofOracleDefense(const ProofOracle& key)
 
     CallProofOracleAttack<P,UseTable> helper(this, key.newOracle(alt(P), node.moves[next_i]));
     tree->depth += 1;
-    ApplyMoveWithPath<PlayerTraits<P>::opponent>::doUndoMove(tree->state, next.path, node.moves[next_i], helper);
+    next.path.pushMove(node.moves[next_i]);
+    tree->state.makeUnmakeMove(Player2Type<PlayerTraits<P>::opponent>(), node.moves[next_i], helper);
     tree->depth -= 1;
 
     node.children[next_i] = next.record;
@@ -2709,7 +2713,7 @@ Dfpn::blockingSimulation(int oracle_i, const ProofOracle& oracle)
   Node& node = tree->node[tree->depth];
   Node& next = tree->node[tree->depth+1];
   const Move oracle_move = node.moves[oracle_i];
-  const Position to = oracle_move.to();
+  const Square to = oracle_move.to();
   assert((node.record.solved & (1ull << oracle_i))
 	 || node.children[oracle_i].proof_disproof.isCheckmateSuccess());
   for (size_t i=0; i<node.moves.size(); ++i) {
@@ -2730,7 +2734,8 @@ Dfpn::blockingSimulation(int oracle_i, const ProofOracle& oracle)
     CallProofOracleAttack<P,true> helper(this, oracle);
 
     tree->depth += 1;
-    ApplyMoveWithPath<PlayerTraits<P>::opponent>::doUndoMove(tree->state, next.path, node.moves[i], helper);
+    next.path.pushMove(node.moves[i]);
+    tree->state.makeUnmakeMove(Player2Type<PlayerTraits<P>::opponent>(), node.moves[i], helper);
     tree->depth -= 1;
 
     node.children[i] = next.record;
@@ -2769,7 +2774,8 @@ Dfpn::grandParentSimulation(int cur_i, const Node& gparent, int gp_i)
   CallProofOracleAttack<P,true> helper(this, oracle);
 
   tree->depth += 1;
-  ApplyMoveWithPath<PlayerTraits<P>::opponent>::doUndoMove(tree->state, next.path, move, helper);
+  next.path.pushMove(move);
+  tree->state.makeUnmakeMove(Player2Type<PlayerTraits<P>::opponent>(), move, helper);
   tree->depth -= 1;
 
   node.children[cur_i] = next.record;
