@@ -36,9 +36,9 @@ int osl::eval::ml::RookPawn<Opening>::eval(const NumEffectState &state)
        i < PtypeTraits<ROOK>::indexLimit;
        ++i)
   {
-    const Piece piece = state.getPieceOf(i);
-    if (piece.isOnBoard() && !piece.position().canPromote(piece.owner()) &&
-	!state.isPawnMaskSet(piece.owner(), piece.position().x()))
+    const Piece piece = state.pieceOf(i);
+    if (piece.isOnBoard() && !piece.square().canPromote(piece.owner()) &&
+	!state.isPawnMaskSet(piece.owner(), piece.square().x()))
     {
       if (piece.owner() == BLACK)
 	result += weight;
@@ -85,21 +85,21 @@ RookPawnY::eval(const NumEffectState &state,
 		const CArray2d<int, 2, 9> &pawns)
 {
   MultiInt result;
-  const CArray<Position,2> kings = {{
-    state.getKingPosition<BLACK>(),
-    state.getKingPosition<WHITE>(),
+  const CArray<Square,2> kings = {{
+    state.kingSquare<BLACK>(),
+    state.kingSquare<WHITE>(),
   }};
   for (int i = PtypeTraits<ROOK>::indexMin;
        i < PtypeTraits<ROOK>::indexLimit;
        ++i)
   {
-    const Piece piece = state.getPieceOf(i);
+    const Piece piece = state.pieceOf(i);
     if (piece.isOnBoard())
     {
       if (piece.owner() == BLACK)
       {
 	const int pawn_y =
-	  pawns[PlayerTraits<BLACK>::index][piece.position().x() - 1];
+	  pawns[BLACK][piece.square().x() - 1];
 	result +=
 	  table[index(piece, pawn_y)] +
 	  y_attack_table[indexY(kings[WHITE], piece, pawn_y)] +
@@ -107,7 +107,7 @@ RookPawnY::eval(const NumEffectState &state,
       }
       else
       {
-	int y = pawns[PlayerTraits<WHITE>::index][piece.position().x() - 1];
+	int y = pawns[WHITE][piece.square().x() - 1];
 	if (y != 0)
 	  y = 10 - y;
 	result -=
@@ -148,7 +148,7 @@ int osl::eval::ml::
 MajorGoldSilverAttacked<Opening>::index(
   const NumEffectState &state, Piece piece)
 {
-  return piece.ptype() + (state.getTurn() == piece.owner() ? 0 : PTYPE_SIZE);
+  return piece.ptype() + (state.turn() == piece.owner() ? 0 : PTYPE_SIZE);
 }
 
 template <bool Opening>
@@ -161,9 +161,9 @@ MajorGoldSilverAttacked<Opening>::evalOne(const NumEffectState &state)
        i < PtypeTraits<PTYPE>::indexLimit;
        ++i)
   {
-    const Piece piece = state.getPieceOf(i);
+    const Piece piece = state.pieceOf(i);
     if (piece.isOnBoard() &&
-	state.hasEffectBy(alt(piece.owner()), piece.position()))
+	state.hasEffectAt(alt(piece.owner()), piece.square()))
     {
       const int weight = table[index(state, piece)];
       if (piece.owner() == BLACK)
@@ -208,20 +208,20 @@ template<osl::Player P>
 inline
 MultiInt osl::eval::ml::RookEffectBase::evalOne(
  const NumEffectState& state,
- Position rook,
- Position myKing,
- Position opKing,
- Position up,
- Position dp,
- Position rp,
- Position lp,
+ Square rook,
+ Square myKing,
+ Square opKing,
+ Square up,
+ Square dp,
+ Square rp,
+ Square lp,
  bool isP)
 {
   MultiInt result;
-  PtypeO uPtypeO=state.getPieceAt(up).ptypeO();
-  PtypeO dPtypeO=state.getPieceAt(dp).ptypeO();
-  PtypeO rPtypeO=state.getPieceAt(rp).ptypeO();
-  PtypeO lPtypeO=state.getPieceAt(lp).ptypeO();
+  PtypeO uPtypeO=state.pieceAt(up).ptypeO();
+  PtypeO dPtypeO=state.pieceAt(dp).ptypeO();
+  PtypeO rPtypeO=state.pieceAt(rp).ptypeO();
+  PtypeO lPtypeO=state.pieceAt(lp).ptypeO();
   if(P==WHITE){
     uPtypeO=(PtypeO)(static_cast<int>(uPtypeO)^(~15));
     dPtypeO=(PtypeO)(static_cast<int>(dPtypeO)^(~15));
@@ -253,21 +253,21 @@ MultiInt osl::eval::ml::RookEffectBase::evalOne(
 MultiInt osl::eval::ml::
 RookEffectBase::eval(const NumEffectState &state)
 {
-  const CArray<Position,2> kings = {{ 
-    state.getKingPosition(BLACK), 
-    state.getKingPosition(WHITE), 
+  const CArray<Square,2> kings = {{ 
+    state.kingSquare(BLACK), 
+    state.kingSquare(WHITE), 
   }};
   MultiInt result;
   for (int i = PtypeTraits<ROOK>::indexMin; i < PtypeTraits<ROOK>::indexLimit;
        ++i)
   {
-    const Piece p = state.getPieceOf(i);
+    const Piece p = state.pieceOf(i);
     if (! p.isOnBoard()) continue;
-    const Position pos=p.position();
-    Position up=state.getMobility(U,i);
-    Position dp=state.getMobility(D,i);
-    Position lp=state.getMobility(L,i);
-    Position rp=state.getMobility(R,i);
+    const Square pos=p.square();
+    Square up=state.mobilityOf(U,i);
+    Square dp=state.mobilityOf(D,i);
+    Square lp=state.mobilityOf(L,i);
+    Square rp=state.mobilityOf(R,i);
     const bool isP=p.isPromoted();
     if(p.owner()==BLACK)
       result+=evalOne<BLACK>(state,pos,kings[0],kings[1],up,dp,rp,lp,isP);
@@ -401,22 +401,22 @@ RookPromoteDefense::eval(const NumEffectState &state)
   for (int i = PtypeTraits<ROOK>::indexMin;
        i < PtypeTraits<ROOK>::indexLimit;
        ++i){
-    const Piece rook = state.getPieceOf(i);
+    const Piece rook = state.pieceOf(i);
     if(rook.isOnBoardNotPromoted()){
       if(rook.owner()==BLACK){
-	Position rookPos=rook.position();
+	Square rookPos=rook.square();
 	if(rookPos.y()>=4){
-	  Position pos=state.getMobility(U,i);
-	  const Piece attacked = state.getPieceAt(pos);
+	  Square pos=state.mobilityOf(U,i);
+	  const Piece attacked = state.pieceAt(pos);
 	  if (attacked.canMoveOn<BLACK>()){
-	    const NumBitmapEffect effect = state.getEffect(pos);
+	    const NumBitmapEffect effect = state.effectSetAt(pos);
 	    if (effect.countEffect(WHITE) == 1){
-	      PieceMask mask = effect & state.getOnBoardMask(WHITE);
-	      const Piece effect_piece = state.getPieceOf(mask.takeOneBit());
+	      PieceMask mask = effect & state.piecesOnBoard(WHITE);
+	      const Piece effect_piece = state.pieceOf(mask.takeOneBit());
 	      const int index = attacked.ptype() * 16 + effect_piece.ptype();
 	      result += promote_defense_table[index];
 	      if (effect_piece.ptype() == ROOK &&
-		  effect_piece.position().x() != rookPos.x())
+		  effect_piece.square().x() != rookPos.x())
 	      {
 		result +=
 		  promote_defense_rook_table[
@@ -429,19 +429,19 @@ RookPromoteDefense::eval(const NumEffectState &state)
 	}
       }
       else{
-	Position rookPos=rook.position();
+	Square rookPos=rook.square();
 	if(rookPos.y()<=6){
-	  Position pos=state.getMobility(D,i);
-	  const Piece attacked = state.getPieceAt(pos);
+	  Square pos=state.mobilityOf(D,i);
+	  const Piece attacked = state.pieceAt(pos);
 	  if (attacked.canMoveOn<WHITE>()){
-	    const NumBitmapEffect effect = state.getEffect(pos);
+	    const NumBitmapEffect effect = state.effectSetAt(pos);
 	    if (effect.countEffect(BLACK) == 1){
-	      PieceMask mask = effect & state.getOnBoardMask(BLACK);
-	      const Piece effect_piece = state.getPieceOf(mask.takeOneBit());
+	      PieceMask mask = effect & state.piecesOnBoard(BLACK);
+	      const Piece effect_piece = state.pieceOf(mask.takeOneBit());
 	      const int index = attacked.ptype() * 16 + effect_piece.ptype();
 	      result -= promote_defense_table[index];
 	      if (effect_piece.ptype() == ROOK &&
-		  effect_piece.position().x() != rookPos.x())
+		  effect_piece.square().x() != rookPos.x())
 	      {
 		result -=
 		  promote_defense_rook_table[
@@ -479,20 +479,20 @@ template<osl::Player P>
 inline
 MultiInt osl::eval::ml::BishopEffectBase::evalOne(
  const NumEffectState& state,
- Position bishop,
- Position myKing,
- Position opKing,
- Position ulp,
- Position urp,
- Position dlp,
- Position drp,
+ Square bishop,
+ Square myKing,
+ Square opKing,
+ Square ulp,
+ Square urp,
+ Square dlp,
+ Square drp,
  bool isP)
 {
   MultiInt result;
-  PtypeO ulPtypeO=state.getPieceAt(ulp).ptypeO();
-  PtypeO urPtypeO=state.getPieceAt(urp).ptypeO();
-  PtypeO dlPtypeO=state.getPieceAt(dlp).ptypeO();
-  PtypeO drPtypeO=state.getPieceAt(drp).ptypeO();
+  PtypeO ulPtypeO=state.pieceAt(ulp).ptypeO();
+  PtypeO urPtypeO=state.pieceAt(urp).ptypeO();
+  PtypeO dlPtypeO=state.pieceAt(dlp).ptypeO();
+  PtypeO drPtypeO=state.pieceAt(drp).ptypeO();
   if(P==WHITE){
     ulPtypeO=(PtypeO)(static_cast<int>(ulPtypeO)^(~15));
     urPtypeO=(PtypeO)(static_cast<int>(urPtypeO)^(~15));
@@ -522,22 +522,22 @@ MultiInt osl::eval::ml::BishopEffectBase::evalOne(
 MultiInt osl::eval::ml::
 BishopEffectBase::eval(const NumEffectState &state)
 {
-  const CArray<Position,2> kings = {{ 
-    state.getKingPosition(BLACK), 
-    state.getKingPosition(WHITE), 
+  const CArray<Square,2> kings = {{ 
+    state.kingSquare(BLACK), 
+    state.kingSquare(WHITE), 
   }};
 
   MultiInt result;
   for (int i = PtypeTraits<BISHOP>::indexMin; i < PtypeTraits<BISHOP>::indexLimit;
        ++i)
     {
-      const Piece p = state.getPieceOf(i);
+      const Piece p = state.pieceOf(i);
       if (! p.isOnBoard()) continue;
-      const Position pos=p.position();
-      Position ulp=state.getMobility(UL,i);
-      Position urp=state.getMobility(UR,i);
-      Position dlp=state.getMobility(DL,i);
-      Position drp=state.getMobility(DR,i);
+      const Square pos=p.square();
+      Square ulp=state.mobilityOf(UL,i);
+      Square urp=state.mobilityOf(UR,i);
+      Square dlp=state.mobilityOf(DL,i);
+      Square drp=state.mobilityOf(DR,i);
       const bool isP=p.isPromoted();
       if(p.owner()==BLACK)
 	result+=evalOne<BLACK>(state,pos,kings[0],kings[1],ulp,urp,dlp,drp,isP);
@@ -710,15 +710,15 @@ BishopHead::eval(const NumEffectState &state)
   for (int i = PtypeTraits<BISHOP>::indexMin;
        i < PtypeTraits<BISHOP>::indexLimit;
        ++i){
-    const Piece p = state.getPieceOf(i);
+    const Piece p = state.pieceOf(i);
     if (p.isOnBoardNotPromoted()){
-      const Position pos=p.position();
+      const Square pos=p.square();
       if (p.owner() == BLACK){
 	if(pos.y()>=2){
-	  const Position up = pos+DirectionPlayerTraits<U,BLACK>::offset();
-	  if (!state.hasEffectBy(BLACK, up)){
-	    const Position king = state.getKingPosition(BLACK);
-	    const PtypeO ptypeo = state.getPieceAt(up).ptypeO();
+	  const Square up = pos+DirectionPlayerTraits<U,BLACK>::offset();
+	  if (!state.hasEffectAt(BLACK, up)){
+	    const Square king = state.kingSquare(BLACK);
+	    const PtypeO ptypeo = state.pieceAt(up).ptypeO();
 	    const int index_k = indexK(BLACK, ptypeo,
 				       std::abs(pos.x() - king.x()),
 				       pos.y() - king.y());
@@ -728,10 +728,10 @@ BishopHead::eval(const NumEffectState &state)
 	}
       }
       else if(pos.y()<=8) {
-	const Position up = pos+DirectionPlayerTraits<U,WHITE>::offset();
-	if (!state.hasEffectBy(WHITE, up)){
-	  const Position king = state.getKingPosition(WHITE);
-	  const PtypeO ptypeo = state.getPieceAt(up).ptypeO();
+	const Square up = pos+DirectionPlayerTraits<U,WHITE>::offset();
+	if (!state.hasEffectAt(WHITE, up)){
+	  const Square king = state.kingSquare(WHITE);
+	  const PtypeO ptypeo = state.pieceAt(up).ptypeO();
 	  const int index_k = indexK(WHITE, ptypeo,
 				     std::abs(pos.x() - king.x()),
 				     pos.y() - king.y());
@@ -760,13 +760,13 @@ template<osl::Player P>
 MultiInt osl::eval::ml::
 KingRookBishop::evalOne(const NumEffectState &state)
 {
-  const Position king=state.getKingPosition(P);
+  const Square king=state.kingSquare(P);
   MultiInt result;
   for (int i = PtypeTraits<ROOK>::indexMin;
        i < PtypeTraits<ROOK>::indexLimit;
        ++i)
   {
-    const Piece rook = state.getPieceOf(i);
+    const Piece rook = state.pieceOf(i);
     if (!rook.isOnBoard())
     {
       continue;
@@ -775,7 +775,7 @@ KingRookBishop::evalOne(const NumEffectState &state)
 	 j < PtypeTraits<BISHOP>::indexLimit;
 	 ++j)
     {
-      const Piece bishop = state.getPieceOf(j);
+      const Piece bishop = state.pieceOf(j);
       if (!bishop.isOnBoard())
       {
 	continue;
@@ -838,14 +838,14 @@ NumPiecesBetweenBishopAndKing::eval(const NumEffectState &state)
        i < PtypeTraits<BISHOP>::indexLimit;
        ++i)
   {
-    const Piece bishop = state.getPieceOf(i);
+    const Piece bishop = state.pieceOf(i);
     if (!bishop.isOnBoard())
     {
       continue;
     }
     int self, opp, all;
     countBetween(state,
-		 state.getKingPosition(alt(bishop.owner())),
+		 state.kingSquare(alt(bishop.owner())),
 		 bishop, self, opp, all);
     if (bishop.owner() == BLACK)
     {
@@ -861,22 +861,22 @@ NumPiecesBetweenBishopAndKing::eval(const NumEffectState &state)
 
 void osl::eval::ml::
 NumPiecesBetweenBishopAndKing::countBetween(
-  const NumEffectState &state, Position king, Piece bishop,
+  const NumEffectState &state, Square king, Piece bishop,
   int &self_count, int &opp_count, int &total_count)
 {
   assert(bishop.isOnBoard());
-  if ((king.x() + king.y() != bishop.position().x() + bishop.position().y()) &&
-	(king.x() - king.y() != bishop.position().x() - bishop.position().y()))
+  if ((king.x() + king.y() != bishop.square().x() + bishop.square().y()) &&
+	(king.x() - king.y() != bishop.square().x() - bishop.square().y()))
   {
     self_count = opp_count = total_count = 8;
     return;
   }
   Direction dir;
-  assert(king.x() != bishop.position().x());
-  assert(king.y() != bishop.position().y());
-  if (king.x() < bishop.position().x())
+  assert(king.x() != bishop.square().x());
+  assert(king.y() != bishop.square().y());
+  if (king.x() < bishop.square().x())
   {
-    if (king.y() < bishop.position().y())
+    if (king.y() < bishop.square().y())
     {
 	dir = UR;
     }
@@ -887,7 +887,7 @@ NumPiecesBetweenBishopAndKing::countBetween(
   }
   else
   {
-    if (king.y() < bishop.position().y())
+    if (king.y() < bishop.square().y())
     {
 	dir = UL;
     }
@@ -899,11 +899,11 @@ NumPiecesBetweenBishopAndKing::countBetween(
   const Player player = bishop.owner();
   const Direction move_dir = (player == BLACK ? dir : inverse(dir));
   self_count = opp_count = total_count = 0;
-  for (Position pos = state.getMobility(dir, bishop.number());
-	 pos != king; pos = Board_Table.nextPosition(player, pos, move_dir))
+  for (Square pos = state.mobilityOf(dir, bishop.number());
+	 pos != king; pos = Board_Table.nextSquare(player, pos, move_dir))
   {
     assert(pos.isOnBoard());
-    const Piece piece = state.getPieceAt(pos);
+    const Piece piece = state.pieceAt(pos);
     if (!piece.isEmpty())
     {
       ++total_count;
@@ -933,24 +933,24 @@ osl::MultiInt osl::eval::ml::
 BishopBishopPiece::eval(const NumEffectState &state)
 {
   MultiInt result;
-  const Piece bishop1 = state.getPieceOf(PtypeTraits<BISHOP>::indexMin);
-  const Piece bishop2 = state.getPieceOf(PtypeTraits<BISHOP>::indexMin + 1);
+  const Piece bishop1 = state.pieceOf(PtypeTraits<BISHOP>::indexMin);
+  const Piece bishop2 = state.pieceOf(PtypeTraits<BISHOP>::indexMin + 1);
   if (!bishop1.isOnBoard() || !bishop2.isOnBoard() ||
       bishop1.owner() == bishop2.owner())
     return result;
-  if (bishop1.position().x() + bishop1.position().y() !=
-      bishop2.position().x() + bishop2.position().y() &&
-      bishop1.position().x() - bishop1.position().y() !=
-      bishop2.position().x() - bishop2.position().y())
+  if (bishop1.square().x() + bishop1.square().y() !=
+      bishop2.square().x() + bishop2.square().y() &&
+      bishop1.square().x() - bishop1.square().y() !=
+      bishop2.square().x() - bishop2.square().y())
     return result;
 
-  if (state.hasEffectByPtype<BISHOP>(bishop2.owner(), bishop1.position()))
+  if (state.hasEffectByPtype<BISHOP>(bishop2.owner(), bishop1.square()))
     return result;
 
   Direction dir;
-  if (bishop1.position().x() < bishop2.position().x())
+  if (bishop1.square().x() < bishop2.square().x())
   {
-    if (bishop1.position().y() < bishop2.position().y())
+    if (bishop1.square().y() < bishop2.square().y())
     {
 	dir = UR;
     }
@@ -961,7 +961,7 @@ BishopBishopPiece::eval(const NumEffectState &state)
   }
   else
   {
-    if (bishop1.position().y() < bishop2.position().y())
+    if (bishop1.square().y() < bishop2.square().y())
     {
 	dir = UL;
     }
@@ -970,17 +970,17 @@ BishopBishopPiece::eval(const NumEffectState &state)
 	dir = DL;
     }
   }
-  Position p1 = state.getMobility(inverse(dir), bishop1.number());
-  Position p2 = state.getMobility(dir, bishop2.number());
+  Square p1 = state.mobilityOf(inverse(dir), bishop1.number());
+  Square p2 = state.mobilityOf(dir, bishop2.number());
   if (p1 == p2)
   {
-    const Piece p = state.getPieceAt(p1);
+    const Piece p = state.pieceAt(p1);
     const bool black_with_support =
-      state.hasEffectBy<BLACK>(bishop1.owner() == BLACK ?
-			       bishop1.position() : bishop2.position());
+      state.hasEffectAt<BLACK>(bishop1.owner() == BLACK ?
+			       bishop1.square() : bishop2.square());
     const bool white_with_support =
-      state.hasEffectBy<WHITE>(bishop1.owner() == WHITE ?
-			       bishop1.position() : bishop2.position());
+      state.hasEffectAt<WHITE>(bishop1.owner() == WHITE ?
+			       bishop1.square() : bishop2.square());
     if (p.owner() == BLACK)
     {
       result += table[index(p.ptype(), black_with_support,
@@ -1062,8 +1062,8 @@ osl::MultiInt osl::eval::ml::
 RookRook::eval(const NumEffectState &state)
 {
   MultiInt result;
-  Piece rook1 = state.getPieceOf(PtypeTraits<ROOK>::indexMin);
-  Piece rook2 = state.getPieceOf(PtypeTraits<ROOK>::indexMin + 1);
+  Piece rook1 = state.pieceOf(PtypeTraits<ROOK>::indexMin);
+  Piece rook2 = state.pieceOf(PtypeTraits<ROOK>::indexMin + 1);
   if (rook1.owner() == rook2.owner())
   {
     if (rook1.owner() == BLACK)
@@ -1104,21 +1104,21 @@ osl::MultiInt osl::eval::ml::
 RookRookPiece::eval(const NumEffectState &state)
 {
   MultiInt result;
-  const Piece rook1 = state.getPieceOf(PtypeTraits<ROOK>::indexMin);
-  const Piece rook2 = state.getPieceOf(PtypeTraits<ROOK>::indexMin + 1);
+  const Piece rook1 = state.pieceOf(PtypeTraits<ROOK>::indexMin);
+  const Piece rook2 = state.pieceOf(PtypeTraits<ROOK>::indexMin + 1);
   if (!rook1.isOnBoard() || !rook2.isOnBoard() ||
       rook1.owner() == rook2.owner())
     return result;
 
-  if (state.hasEffectByPtype<ROOK>(rook2.owner(), rook1.position()))
+  if (state.hasEffectByPtype<ROOK>(rook2.owner(), rook1.square()))
     return result;
 
   Direction dir;
   bool vertical = false;
-  if (rook1.position().x() == rook2.position().x())
+  if (rook1.square().x() == rook2.square().x())
   {
     vertical = true;
-    if (rook1.position().y() < rook2.position().y())
+    if (rook1.square().y() < rook2.square().y())
     {
 	dir = D;
     }
@@ -1127,9 +1127,9 @@ RookRookPiece::eval(const NumEffectState &state)
 	dir = U;
     }
   }
-  else if (rook1.position().y() == rook2.position().y())
+  else if (rook1.square().y() == rook2.square().y())
   {
-    if (rook1.position().x() < rook2.position().x())
+    if (rook1.square().x() < rook2.square().x())
     {
 	dir = L;
     }
@@ -1143,18 +1143,18 @@ RookRookPiece::eval(const NumEffectState &state)
     return result;
   }
 
-  Position p1 = state.getMobility(dir, rook1.number());
-  Position p2 = state.getMobility(inverse(dir), rook2.number());
+  Square p1 = state.mobilityOf(dir, rook1.number());
+  Square p2 = state.mobilityOf(inverse(dir), rook2.number());
   assert(p1.isOnBoard() && p2.isOnBoard());
   if (p1 == p2)
   {
-    const Piece p = state.getPieceAt(p1);
+    const Piece p = state.pieceAt(p1);
     const bool black_with_support =
-      state.hasEffectBy<BLACK>(rook1.owner() == BLACK ?
-			       rook1.position() : rook2.position());
+      state.hasEffectAt<BLACK>(rook1.owner() == BLACK ?
+			       rook1.square() : rook2.square());
     const bool white_with_support =
-      state.hasEffectBy<WHITE>(rook1.owner() == WHITE ?
-			       rook1.position() : rook2.position());
+      state.hasEffectAt<WHITE>(rook1.owner() == WHITE ?
+			       rook1.square() : rook2.square());
     if (p.owner() == BLACK)
     {
       result += table[index(p.ptype(), black_with_support,
@@ -1189,17 +1189,68 @@ BishopStandFile5::eval(const NumEffectState &state)
   MultiInt result;
   if (state.hasPieceOnStand<BISHOP>(BLACK))
   {
-    result += table[ptypeOIndex(state.getPieceAt(Position(5, 3)).ptypeO())];
+    result += table[ptypeOIndex(state.pieceAt(Square(5, 3)).ptypeO())];
   }
   if (state.hasPieceOnStand<BISHOP>(WHITE))
   {
-    PtypeO ptypeO = state.getPieceAt(Position(5, 7)).ptypeO();
+    PtypeO ptypeO = state.pieceAt(Square(5, 7)).ptypeO();
     ptypeO = altIfPiece(ptypeO);
     result -= table[ptypeOIndex(ptypeO)];
   }
   return result;
 }
 
+
+
+osl::misc::CArray<MultiInt, osl::eval::ml::MajorCheckWithCapture::ONE_DIM>
+osl::eval::ml::MajorCheckWithCapture::table;
+
+void osl::eval::ml::
+MajorCheckWithCapture::setUp(const Weights &weights)
+{
+  for (size_t i = 0; i < ONE_DIM; ++i)
+  {
+    for (int s=0; s<NStages; ++s)
+      table[i][s] = weights.value(i + ONE_DIM*s);
+  }
+}
+
+template <osl::Player Owner>
+osl::MultiInt osl::eval::ml::
+MajorCheckWithCapture::addOne(const NumEffectState &state)
+{
+  const Square king = state.kingSquare(Owner);
+  PieceMask pieces = state.effectedMask(alt(Owner));
+  pieces &= state.piecesOnBoard(Owner);
+  pieces &= ~state.effectedMask(Owner);
+  MultiInt sum;
+  while (pieces.any()) {
+    const Piece p = state.pieceOf(pieces.takeOneBit());
+    const Square sq = p.square();
+    if (state.hasLongEffectAt<ROOK>(alt(Owner), sq)
+	&& state.hasEffectIf(newPtypeO(BLACK,ROOK), sq, king)) {
+      if (Owner == BLACK)
+	sum += table[index(p.ptype(), true, sq.canPromote(alt(Owner)))];
+      else
+	sum -= table[index(p.ptype(), true, sq.canPromote(alt(Owner)))];
+    }
+    if (state.hasLongEffectAt<BISHOP>(alt(Owner), sq)
+	&& state.hasEffectIf(newPtypeO(BLACK,BISHOP), sq, king)) {
+      if (Owner == BLACK)
+	sum += table[index(p.ptype(), false, sq.canPromote(alt(Owner)))];
+      else
+	sum -= table[index(p.ptype(), false, sq.canPromote(alt(Owner)))];
+    }
+  }
+  return sum;
+}
+
+osl::MultiInt osl::eval::ml::
+MajorCheckWithCapture::eval(const NumEffectState &state)
+{
+  return addOne<BLACK>(state) + addOne<WHITE>(state);
+}
+
 
 namespace osl
 {
