@@ -11,6 +11,7 @@
 #include "osl/eval/ml/progress.h"
 #include "osl/effect_util/pin.h"
 #include "osl/misc/random.h"
+#include "osl/misc/binaryIO.h"
 #include "osl/oslConfig.h"
 #include "osl/pieceStand.h"
 
@@ -18,6 +19,18 @@
 #include <boost/thread/mutex.hpp>
 #include <fstream>
 
+osl::eval::ml::OpenMidEndingPtypeTable::
+OpenMidEndingPtypeTable()
+{
+  const CArray<int, PTYPE_SIZE> values = {{
+      0, 0,
+      583, 493, 491, 467, 1279, 1351,
+      PtypeEvalTraits<KING>::val, 585,
+      128, 318, 361, 540, 959, 1059
+    }};
+  reset(values);
+}
+
 volatile osl::eval::ml::OpenMidEndingEval::LoadStatus
 osl::eval::ml::OpenMidEndingEval::initialized_flag = osl::eval::ml::OpenMidEndingEval::Zero;
 static boost::mutex initialize_mutex;
@@ -47,37 +60,36 @@ namespace
     Eval::setUp(weights, stage);
   }
 #endif
+  typedef osl::misc::BinaryElementReader<int> reader_t;
   template <class Eval>
-  static int setUpOne(std::istream &is)
+  static int setUpOne(reader_t& p)
   {
-    int read_count = 0;
     osl::eval::ml::Weights weights(Eval::DIM);
     // std::cerr << typeid(Eval).name() << " " << Eval::DIM << "\n";
     for (size_t i = 0; i < weights.dimension(); ++i)
     {
-      int val;
-      is >> val;
+      if (! p.hasNext())
+	break;
+      int val = p.read();
       weights.setValue(i, val);
-      ++read_count;
     }
     Eval::setUp(weights);
-    return read_count;
+    return weights.dimension();
   }
   template <class Eval>
-  static int setUpOne(std::istream &is,int stage)
+  static int setUpOne(reader_t& p, int stage)
   {
-    int read_count = 0;
     osl::eval::ml::Weights weights(Eval::DIM);
     // std::cerr << typeid(Eval).name() << " " << Eval::DIM << "\n";
     for (size_t i = 0; i < weights.dimension(); ++i)
     {
-      int val;
-      is >> val;
+      if (!p.hasNext())
+	break;
+      int val = p.read();
       weights.setValue(i, val);
-      ++read_count;
     }
     Eval::setUp(weights,stage);
-    return read_count;
+    return weights.dimension();
   }
 }
 
@@ -87,14 +99,20 @@ bool osl::eval::ml::OpenMidEndingEval::setUp(const char *filename)
     return true;
   boost::mutex::scoped_lock lk(initialize_mutex);
   std::ifstream is(filename);
+  reader_t reader(is);
+  if (! reader.hasNext()) {
+    initialized_flag = Zero;
+    return false;
+  }
   Weights weights(PTYPE_SIZE);
-  int read_count = 0;
+  size_t read_count = 0;
 
   // flat
   for (int i = 0; i < PTYPE_SIZE; ++i)
   {
-    int val;
-    is >> val;
+    if (! reader.hasNext())
+      break;
+    int val = reader.read();
     weights.setValue(i, val);
     ++read_count;
   }
@@ -104,230 +122,236 @@ bool osl::eval::ml::OpenMidEndingEval::setUp(const char *filename)
   piece_pair_weights.resetDimension(PiecePair::DIM);
   for (size_t i = 0; i < piece_pair_weights.dimension(); ++i)
   {
-    int val;
-    is >> val;
+    if (! reader.hasNext())
+      break;
+    int val = reader.read();
     piece_pair_weights.setValue(i, val);
     ++read_count;
   }
   PiecePair::sanitize(piece_pair_weights);
   PiecePair::compile(piece_pair_weights);
 
-  read_count += setUpOne<King25EffectAttack>(is);
-  read_count += setUpOne<King25EffectYAttack>(is);
-  read_count += setUpOne<PiecePairKing>(is);
+  read_count += setUpOne<King25EffectAttack>(reader);
+  read_count += setUpOne<King25EffectYAttack>(reader);
+  read_count += setUpOne<PiecePairKing>(reader);
+  read_count += setUpOne<BishopExchangeSilverKing>(reader);
+  read_count += setUpOne<EnterKingDefense>(reader);
 
   // opening
-  read_count += setUpOne<PieceStand>(is,0);
-  read_count += setUpOne<King25EffectEachBothOpening>(is);
-  read_count += setUpOne<PawnDrop>(is,0);
-  read_count += setUpOne<NoPawnOnStand>(is,0);
-  read_count += setUpOne<GoldRetreat>(is,0);
-  read_count += setUpOne<SilverRetreat>(is,0);
-  read_count += setUpOne<KnightAdvance>(is,0);
-  read_count += setUpOne<AllMajor>(is,0);
-  read_count += setUpOne<KingXBlocked>(is,0);
-  read_count += setUpOne<KingXBlockedY>(is,0);
-  read_count += setUpOne<AllGold>(is,0);
-  read_count += setUpOne<PtypeX>(is,0);
-  read_count += setUpOne<PtypeY>(is,0);
-  read_count += setUpOne<AnagumaEmpty>(is,0);
-  read_count += setUpOne<NonPawnPieceStand>(is,0);
-  read_count += setUpOne<King25EffectDefense>(is,0);
-  read_count += setUpOne<King25EffectYDefense>(is,0);
-  read_count += setUpOne<RookMobility>(is,0);
-  read_count += setUpOne<BishopMobility>(is,0);
-  read_count += setUpOne<LanceMobility>(is,0);
-  read_count += setUpOne<RookEffect>(is,0);
-  read_count += setUpOne<BishopEffect>(is,0);
-  read_count += setUpOne<PawnAdvance>(is,0);
-  read_count += setUpOne<PawnDropY>(is,0);
-  read_count += setUpOne<KnightCheck>(is,0);
+  read_count += setUpOne<PieceStand>(reader,0);
+  read_count += setUpOne<King25EffectEachBothOpening>(reader);
+  read_count += setUpOne<PawnDrop>(reader,0);
+  read_count += setUpOne<NoPawnOnStand>(reader,0);
+  read_count += setUpOne<GoldRetreat>(reader,0);
+  read_count += setUpOne<SilverRetreat>(reader,0);
+  read_count += setUpOne<KnightAdvance>(reader,0);
+  read_count += setUpOne<AllMajor>(reader,0);
+  read_count += setUpOne<KingXBlocked>(reader,0);
+  read_count += setUpOne<KingXBlockedY>(reader,0);
+  read_count += setUpOne<AllGold>(reader,0);
+  read_count += setUpOne<PtypeX>(reader,0);
+  read_count += setUpOne<PtypeY>(reader,0);
+  read_count += setUpOne<AnagumaEmpty>(reader,0);
+  read_count += setUpOne<NonPawnPieceStand>(reader,0);
+  read_count += setUpOne<King25EffectDefense>(reader,0);
+  read_count += setUpOne<King25EffectYDefense>(reader,0);
+  read_count += setUpOne<RookMobility>(reader,0);
+  read_count += setUpOne<BishopMobility>(reader,0);
+  read_count += setUpOne<LanceMobility>(reader,0);
+  read_count += setUpOne<RookEffect>(reader,0);
+  read_count += setUpOne<BishopEffect>(reader,0);
+  read_count += setUpOne<PawnAdvance>(reader,0);
+  read_count += setUpOne<PawnDropY>(reader,0);
+  read_count += setUpOne<KnightCheck>(reader,0);
 
   // midgame
-  read_count += setUpOne<PieceStand>(is,1);
-  read_count += setUpOne<King25EffectEachBothMidgame>(is);
-  read_count += setUpOne<PawnDrop>(is,1);
-  read_count += setUpOne<NoPawnOnStand>(is,1);
-  read_count += setUpOne<GoldRetreat>(is,1);
-  read_count += setUpOne<SilverRetreat>(is,1);
-  read_count += setUpOne<KnightAdvance>(is,1);
-  read_count += setUpOne<AllMajor>(is,1);
-  read_count += setUpOne<KingXBlocked>(is,1);
-  read_count += setUpOne<KingXBlockedY>(is,1);
-  read_count += setUpOne<AllGold>(is,1);
-  read_count += setUpOne<PtypeX>(is,1);
-  read_count += setUpOne<PtypeY>(is,1);
-  read_count += setUpOne<AnagumaEmpty>(is,1);
-  read_count += setUpOne<NonPawnPieceStand>(is,1);
-  read_count += setUpOne<King25EffectDefense>(is,1);
-  read_count += setUpOne<King25EffectYDefense>(is,1);
-  read_count += setUpOne<RookMobility>(is,1);
-  read_count += setUpOne<BishopMobility>(is,1);
-  read_count += setUpOne<LanceMobility>(is,1);
-  read_count += setUpOne<RookEffect>(is,1);
-  read_count += setUpOne<BishopEffect>(is,1);
-  read_count += setUpOne<PawnAdvance>(is,1);
-  read_count += setUpOne<PawnDropY>(is,1);
-  read_count += setUpOne<KnightCheck>(is,1);
+  read_count += setUpOne<PieceStand>(reader,1);
+  read_count += setUpOne<King25EffectEachBothMidgame>(reader);
+  read_count += setUpOne<PawnDrop>(reader,1);
+  read_count += setUpOne<NoPawnOnStand>(reader,1);
+  read_count += setUpOne<GoldRetreat>(reader,1);
+  read_count += setUpOne<SilverRetreat>(reader,1);
+  read_count += setUpOne<KnightAdvance>(reader,1);
+  read_count += setUpOne<AllMajor>(reader,1);
+  read_count += setUpOne<KingXBlocked>(reader,1);
+  read_count += setUpOne<KingXBlockedY>(reader,1);
+  read_count += setUpOne<AllGold>(reader,1);
+  read_count += setUpOne<PtypeX>(reader,1);
+  read_count += setUpOne<PtypeY>(reader,1);
+  read_count += setUpOne<AnagumaEmpty>(reader,1);
+  read_count += setUpOne<NonPawnPieceStand>(reader,1);
+  read_count += setUpOne<King25EffectDefense>(reader,1);
+  read_count += setUpOne<King25EffectYDefense>(reader,1);
+  read_count += setUpOne<RookMobility>(reader,1);
+  read_count += setUpOne<BishopMobility>(reader,1);
+  read_count += setUpOne<LanceMobility>(reader,1);
+  read_count += setUpOne<RookEffect>(reader,1);
+  read_count += setUpOne<BishopEffect>(reader,1);
+  read_count += setUpOne<PawnAdvance>(reader,1);
+  read_count += setUpOne<PawnDropY>(reader,1);
+  read_count += setUpOne<KnightCheck>(reader,1);
 
 #ifdef EVAL_QUAD
   // midgame2
-  read_count += setUpOne<PieceStand>(is,2);
-  read_count += setUpOne<King25EffectEachBothMidgame2>(is);
-  read_count += setUpOne<PawnDrop>(is,2);
-  read_count += setUpOne<NoPawnOnStand>(is,2);
-  read_count += setUpOne<GoldRetreat>(is,2);
-  read_count += setUpOne<SilverRetreat>(is,2);
-  read_count += setUpOne<KnightAdvance>(is,2);
-  read_count += setUpOne<AllMajor>(is,2);
-  read_count += setUpOne<KingXBlocked>(is,2);
-  read_count += setUpOne<KingXBlockedY>(is,2);
-  read_count += setUpOne<AllGold>(is,2);
-  read_count += setUpOne<PtypeX>(is,2);
-  read_count += setUpOne<PtypeY>(is,2);
-  read_count += setUpOne<AnagumaEmpty>(is,2);
-  read_count += setUpOne<NonPawnPieceStand>(is,2);
-  read_count += setUpOne<King25EffectDefense>(is,2);
-  read_count += setUpOne<King25EffectYDefense>(is,2);
-  read_count += setUpOne<RookMobility>(is,2);
-  read_count += setUpOne<BishopMobility>(is,2);
-  read_count += setUpOne<LanceMobility>(is,2);
-  read_count += setUpOne<RookEffect>(is,2);
-  read_count += setUpOne<BishopEffect>(is,2);
-  read_count += setUpOne<PawnAdvance>(is,2);
-  read_count += setUpOne<PawnDropY>(is,2);
-  read_count += setUpOne<KnightCheck>(is,2);
+  read_count += setUpOne<PieceStand>(reader,2);
+  read_count += setUpOne<King25EffectEachBothMidgame2>(reader);
+  read_count += setUpOne<PawnDrop>(reader,2);
+  read_count += setUpOne<NoPawnOnStand>(reader,2);
+  read_count += setUpOne<GoldRetreat>(reader,2);
+  read_count += setUpOne<SilverRetreat>(reader,2);
+  read_count += setUpOne<KnightAdvance>(reader,2);
+  read_count += setUpOne<AllMajor>(reader,2);
+  read_count += setUpOne<KingXBlocked>(reader,2);
+  read_count += setUpOne<KingXBlockedY>(reader,2);
+  read_count += setUpOne<AllGold>(reader,2);
+  read_count += setUpOne<PtypeX>(reader,2);
+  read_count += setUpOne<PtypeY>(reader,2);
+  read_count += setUpOne<AnagumaEmpty>(reader,2);
+  read_count += setUpOne<NonPawnPieceStand>(reader,2);
+  read_count += setUpOne<King25EffectDefense>(reader,2);
+  read_count += setUpOne<King25EffectYDefense>(reader,2);
+  read_count += setUpOne<RookMobility>(reader,2);
+  read_count += setUpOne<BishopMobility>(reader,2);
+  read_count += setUpOne<LanceMobility>(reader,2);
+  read_count += setUpOne<RookEffect>(reader,2);
+  read_count += setUpOne<BishopEffect>(reader,2);
+  read_count += setUpOne<PawnAdvance>(reader,2);
+  read_count += setUpOne<PawnDropY>(reader,2);
+  read_count += setUpOne<KnightCheck>(reader,2);
 #endif
 
   // endgame
-  read_count += setUpOne<PieceStand>(is,EndgameIndex);
-  read_count += setUpOne<King25EffectEachBothEnding>(is);
-  read_count += setUpOne<PawnDrop>(is,EndgameIndex);
-  read_count += setUpOne<NoPawnOnStand>(is,EndgameIndex);
-  read_count += setUpOne<GoldRetreat>(is,EndgameIndex);
-  read_count += setUpOne<SilverRetreat>(is,EndgameIndex);
-  read_count += setUpOne<KnightAdvance>(is,EndgameIndex);
-  read_count += setUpOne<AllMajor>(is,EndgameIndex);
-  read_count += setUpOne<KingXBlocked>(is,EndgameIndex);
-  read_count += setUpOne<KingXBlockedY>(is,EndgameIndex);
-  read_count += setUpOne<AllGold>(is,EndgameIndex);
-  read_count += setUpOne<PtypeX>(is,EndgameIndex);
-  read_count += setUpOne<PtypeY>(is,EndgameIndex);
-  read_count += setUpOne<AnagumaEmpty>(is,EndgameIndex);
-  read_count += setUpOne<NonPawnPieceStand>(is,EndgameIndex);
-  read_count += setUpOne<King25EffectDefense>(is,EndgameIndex);
-  read_count += setUpOne<King25EffectYDefense>(is,EndgameIndex);
-  read_count += setUpOne<RookMobility>(is,EndgameIndex);
-  read_count += setUpOne<BishopMobility>(is,EndgameIndex);
-  read_count += setUpOne<LanceMobility>(is,EndgameIndex);
-  read_count += setUpOne<RookEffect>(is,EndgameIndex);
-  read_count += setUpOne<BishopEffect>(is,EndgameIndex);
-  read_count += setUpOne<PawnAdvance>(is,EndgameIndex);
-  read_count += setUpOne<PawnDropY>(is,EndgameIndex);
-  read_count += setUpOne<KnightCheck>(is,EndgameIndex);
+  read_count += setUpOne<PieceStand>(reader,EndgameIndex);
+  read_count += setUpOne<King25EffectEachBothEnding>(reader);
+  read_count += setUpOne<PawnDrop>(reader,EndgameIndex);
+  read_count += setUpOne<NoPawnOnStand>(reader,EndgameIndex);
+  read_count += setUpOne<GoldRetreat>(reader,EndgameIndex);
+  read_count += setUpOne<SilverRetreat>(reader,EndgameIndex);
+  read_count += setUpOne<KnightAdvance>(reader,EndgameIndex);
+  read_count += setUpOne<AllMajor>(reader,EndgameIndex);
+  read_count += setUpOne<KingXBlocked>(reader,EndgameIndex);
+  read_count += setUpOne<KingXBlockedY>(reader,EndgameIndex);
+  read_count += setUpOne<AllGold>(reader,EndgameIndex);
+  read_count += setUpOne<PtypeX>(reader,EndgameIndex);
+  read_count += setUpOne<PtypeY>(reader,EndgameIndex);
+  read_count += setUpOne<AnagumaEmpty>(reader,EndgameIndex);
+  read_count += setUpOne<NonPawnPieceStand>(reader,EndgameIndex);
+  read_count += setUpOne<King25EffectDefense>(reader,EndgameIndex);
+  read_count += setUpOne<King25EffectYDefense>(reader,EndgameIndex);
+  read_count += setUpOne<RookMobility>(reader,EndgameIndex);
+  read_count += setUpOne<BishopMobility>(reader,EndgameIndex);
+  read_count += setUpOne<LanceMobility>(reader,EndgameIndex);
+  read_count += setUpOne<RookEffect>(reader,EndgameIndex);
+  read_count += setUpOne<BishopEffect>(reader,EndgameIndex);
+  read_count += setUpOne<PawnAdvance>(reader,EndgameIndex);
+  read_count += setUpOne<PawnDropY>(reader,EndgameIndex);
+  read_count += setUpOne<KnightCheck>(reader,EndgameIndex);
 
   // triple
-  read_count += setUpOne<KingPieceRelative>(is,0);
-  read_count += setUpOne<KingPieceRelative>(is,1);
+  read_count += setUpOne<KingPieceRelative>(reader,0);
+  read_count += setUpOne<KingPieceRelative>(reader,1);
 #ifdef EVAL_QUAD
-  read_count += setUpOne<KingPieceRelative>(is,2);
+  read_count += setUpOne<KingPieceRelative>(reader,2);
 #endif
-  read_count += setUpOne<KingPieceRelative>(is,EndgameIndex);
-  read_count += setUpOne<NonPawnPieceStandTurn>(is);
-  read_count += setUpOne<King25EffectEachXY>(is);
-  read_count += setUpOne<RookPawnY>(is);
-  read_count += setUpOne<RookEffectPiece>(is);
-  read_count += setUpOne<BishopEffectPiece>(is);
-  read_count += setUpOne<PieceStandY>(is);
-  read_count += setUpOne<RookEffectPieceKingRelative>(is);
-  read_count += setUpOne<BishopEffectPieceKingRelative>(is);
-  read_count += setUpOne<RookPawnYX>(is);
-  read_count += setUpOne<PawnPtypeOPtypeO>(is);
-  read_count += setUpOne<PromotedMinorPieces>(is);
-  read_count += setUpOne<KingPieceRelativeNoSupport>(is);
-  read_count += setUpOne<NonPawnAttacked>(is);
-  read_count += setUpOne<PtypeYY>(is);
-  read_count += setUpOne<PawnPtypeOPtypeOY>(is);
-  read_count += setUpOne<PawnDropX>(is);
-  read_count += setUpOne<King3Pieces>(is);
-  read_count += setUpOne<King3PiecesXY>(is);
-  read_count += setUpOne<King25EffectEachKXY>(is);
-  read_count += setUpOne<BishopHead>(is);
-  read_count += setUpOne<BishopHeadKingRelative>(is);
-  read_count += setUpOne<KnightCheckY>(is);
-  read_count += setUpOne<KnightHead>(is);
-  read_count += setUpOne<RookPromoteDefense>(is);
-  read_count += setUpOne<PawnDropPawnStand>(is);
-  read_count += setUpOne<PawnDropPawnStandX>(is);
-  read_count += setUpOne<PawnDropPawnStandY>(is);
-  read_count += setUpOne<KnightHeadOppPiecePawnOnStand>(is);
-  read_count += setUpOne<KingXBothBlocked>(is);
-  read_count += setUpOne<KingXBothBlockedY>(is);
-  read_count += setUpOne<KingRookBishop>(is);
-  read_count += setUpOne<PromotedMinorPiecesY>(is);
-  read_count += setUpOne<King25EffectSupported>(is);
-  read_count += setUpOne<King25EffectSupportedY>(is);
-  read_count += setUpOne<NonPawnAttackedKingRelative>(is);
-  read_count += setUpOne<NonPawnAttackedPtype>(is);
-  read_count += setUpOne<PtypeCount>(is);
-  read_count += setUpOne<KingXBlocked3>(is);
-  read_count += setUpOne<KingXBlocked3Y>(is);
-  read_count += setUpOne<PtypeCountXY>(is);
-  read_count += setUpOne<PtypeCountXYAttack>(is);
-  read_count += setUpOne<LanceEffectPieceKingRelative>(is);
-  read_count += setUpOne<KingMobility>(is);
-  read_count += setUpOne<KingMobilitySum>(is);
-  read_count += setUpOne<PtypeYPawnY>(is);
-  read_count += setUpOne<GoldAndSilverNearKing>(is);
-  read_count += setUpOne<PtypeCombination>(is);
-  read_count += setUpOne<PieceStandCombinationBoth>(is);
-  read_count += setUpOne<King25BothSide>(is);
-  read_count += setUpOne<King25BothSideX>(is);
-  read_count += setUpOne<King25BothSideY>(is);
-  read_count += setUpOne<GoldAndSilverNearKingCombination>(is);
-  read_count += setUpOne<KingMobilityWithRook>(is);
-  read_count += setUpOne<KingMobilityWithBishop>(is);
-  read_count += setUpOne<NumPiecesBetweenBishopAndKingSelf>(is);
-  read_count += setUpOne<NumPiecesBetweenBishopAndKingOpp>(is);
-  read_count += setUpOne<NumPiecesBetweenBishopAndKingAll>(is);
-  read_count += setUpOne<King25Effect3>(is);
-  read_count += setUpOne<SilverHeadPawnKingRelative>(is);
-  read_count += setUpOne<GoldKnightKingRelative>(is);
-  read_count += setUpOne<RookMobilitySum>(is);
-  read_count += setUpOne<RookMobilityX>(is);
-  read_count += setUpOne<RookMobilityY>(is);
-  read_count += setUpOne<RookMobilitySumKingX>(is);
-  read_count += setUpOne<RookMobilityXKingX>(is);
-  read_count += setUpOne<PinPtype>(is);
-  read_count += setUpOne<PinPtypeDistance>(is);
-  read_count += setUpOne<BishopMobilityEach>(is);
-  read_count += setUpOne<BishopBishopPiece>(is);
-  read_count += setUpOne<NonPawnPieceStandCombination>(is);
-  read_count += setUpOne<CanCheckNonPawnPieceStandCombination>(is);
-  read_count += setUpOne<King25Effect3Y>(is);
-  read_count += setUpOne<RookRook>(is);
-  read_count += setUpOne<RookRookPiece>(is);
-  read_count += setUpOne<PinPtypePawnAttack>(is);
-  read_count += setUpOne<King25Mobility>(is);
-  read_count += setUpOne<King25MobilityX>(is);
-  read_count += setUpOne<King25MobilityY>(is);
-  read_count += setUpOne<King25EffectCountCombination>(is);
-  read_count += setUpOne<GoldSideMove>(is);
-  read_count += setUpOne<King25EffectCountCombinationY>(is);
-  read_count += setUpOne<RookPromoteDefenseRookH>(is);
-  read_count += setUpOne<BishopHeadX>(is);
-  read_count += setUpOne<PawnDropNonDrop>(is);
-  read_count += setUpOne<PawnStateKingRelative>(is);
-  read_count += setUpOne<SilverFork>(is);
-  read_count += setUpOne<BishopRookFork>(is);
-  read_count += setUpOne<BishopStandFile5>(is);
-  read_count += setUpOne<KnightFork>(is);
-  read_count += setUpOne<NonPawnAttackedPtypePair>(is);
+  read_count += setUpOne<KingPieceRelative>(reader,EndgameIndex);
+  read_count += setUpOne<NonPawnPieceStandTurn>(reader);
+  read_count += setUpOne<King25EffectEachXY>(reader);
+  read_count += setUpOne<RookPawnY>(reader);
+  read_count += setUpOne<RookEffectPiece>(reader);
+  read_count += setUpOne<BishopEffectPiece>(reader);
+  read_count += setUpOne<PieceStandY>(reader);
+  read_count += setUpOne<RookEffectPieceKingRelative>(reader);
+  read_count += setUpOne<BishopEffectPieceKingRelative>(reader);
+  read_count += setUpOne<RookPawnYX>(reader);
+  read_count += setUpOne<PawnPtypeOPtypeO>(reader);
+  read_count += setUpOne<PromotedMinorPieces>(reader);
+  read_count += setUpOne<KingPieceRelativeNoSupport>(reader);
+  read_count += setUpOne<NonPawnAttacked>(reader);
+  read_count += setUpOne<PtypeYY>(reader);
+  read_count += setUpOne<PawnPtypeOPtypeOY>(reader);
+  read_count += setUpOne<PawnDropX>(reader);
+  read_count += setUpOne<King3Pieces>(reader);
+  read_count += setUpOne<King3PiecesXY>(reader);
+  read_count += setUpOne<King25EffectEachKXY>(reader);
+  read_count += setUpOne<BishopHead>(reader);
+  read_count += setUpOne<BishopHeadKingRelative>(reader);
+  read_count += setUpOne<KnightCheckY>(reader);
+  read_count += setUpOne<KnightHead>(reader);
+  read_count += setUpOne<RookPromoteDefense>(reader);
+  read_count += setUpOne<PawnDropPawnStand>(reader);
+  read_count += setUpOne<PawnDropPawnStandX>(reader);
+  read_count += setUpOne<PawnDropPawnStandY>(reader);
+  read_count += setUpOne<KnightHeadOppPiecePawnOnStand>(reader);
+  read_count += setUpOne<KingXBothBlocked>(reader);
+  read_count += setUpOne<KingXBothBlockedY>(reader);
+  read_count += setUpOne<KingRookBishop>(reader);
+  read_count += setUpOne<PromotedMinorPiecesY>(reader);
+  read_count += setUpOne<King25EffectSupported>(reader);
+  read_count += setUpOne<King25EffectSupportedY>(reader);
+  read_count += setUpOne<NonPawnAttackedKingRelative>(reader);
+  read_count += setUpOne<NonPawnAttackedPtype>(reader);
+  read_count += setUpOne<PtypeCount>(reader);
+  read_count += setUpOne<KingXBlocked3>(reader);
+  read_count += setUpOne<KingXBlocked3Y>(reader);
+  read_count += setUpOne<PtypeCountXY>(reader);
+  read_count += setUpOne<PtypeCountXYAttack>(reader);
+  read_count += setUpOne<LanceEffectPieceKingRelative>(reader);
+  read_count += setUpOne<KingMobility>(reader);
+  read_count += setUpOne<KingMobilitySum>(reader);
+  read_count += setUpOne<PtypeYPawnY>(reader);
+  read_count += setUpOne<GoldAndSilverNearKing>(reader);
+  read_count += setUpOne<PtypeCombination>(reader);
+  read_count += setUpOne<PieceStandCombinationBoth>(reader);
+  read_count += setUpOne<King25BothSide>(reader);
+  read_count += setUpOne<King25BothSideX>(reader);
+  read_count += setUpOne<King25BothSideY>(reader);
+  read_count += setUpOne<GoldAndSilverNearKingCombination>(reader);
+  read_count += setUpOne<KingMobilityWithRook>(reader);
+  read_count += setUpOne<KingMobilityWithBishop>(reader);
+  read_count += setUpOne<NumPiecesBetweenBishopAndKingSelf>(reader);
+  read_count += setUpOne<NumPiecesBetweenBishopAndKingOpp>(reader);
+  read_count += setUpOne<NumPiecesBetweenBishopAndKingAll>(reader);
+  read_count += setUpOne<King25Effect3>(reader);
+  read_count += setUpOne<SilverHeadPawnKingRelative>(reader);
+  read_count += setUpOne<GoldKnightKingRelative>(reader);
+  read_count += setUpOne<RookMobilitySum>(reader);
+  read_count += setUpOne<RookMobilityX>(reader);
+  read_count += setUpOne<RookMobilityY>(reader);
+  read_count += setUpOne<RookMobilitySumKingX>(reader);
+  read_count += setUpOne<RookMobilityXKingX>(reader);
+  read_count += setUpOne<PinPtype>(reader);
+  read_count += setUpOne<PinPtypeDistance>(reader);
+  read_count += setUpOne<BishopMobilityEach>(reader);
+  read_count += setUpOne<BishopBishopPiece>(reader);
+  read_count += setUpOne<NonPawnPieceStandCombination>(reader);
+  read_count += setUpOne<CanCheckNonPawnPieceStandCombination>(reader);
+  read_count += setUpOne<King25Effect3Y>(reader);
+  read_count += setUpOne<RookRook>(reader);
+  read_count += setUpOne<RookRookPiece>(reader);
+  read_count += setUpOne<PinPtypePawnAttack>(reader);
+  read_count += setUpOne<King25Mobility>(reader);
+  read_count += setUpOne<King25MobilityX>(reader);
+  read_count += setUpOne<King25MobilityY>(reader);
+  read_count += setUpOne<King25EffectCountCombination>(reader);
+  read_count += setUpOne<GoldSideMove>(reader);
+  read_count += setUpOne<King25EffectCountCombinationY>(reader);
+  read_count += setUpOne<RookPromoteDefenseRookH>(reader);
+  read_count += setUpOne<BishopHeadX>(reader);
+  read_count += setUpOne<PawnDropNonDrop>(reader);
+  read_count += setUpOne<PawnStateKingRelative>(reader);
+  read_count += setUpOne<SilverFork>(reader);
+  read_count += setUpOne<BishopRookFork>(reader);
+  read_count += setUpOne<BishopStandFile5>(reader);
+  read_count += setUpOne<KnightFork>(reader);
+  read_count += setUpOne<NonPawnAttackedPtypePair>(reader);
+  read_count += setUpOne<MajorCheckWithCapture>(reader);
+  read_count += setUpOne<SilverAdvance26>(reader);
 
-  initialized_flag = is ? Loaded : Zero;
+  initialized_flag = reader.failed() ? Zero : Loaded;
   if (initialized_flag != Loaded)
   {
-    std::cerr << "Failed to load OpenMidEndingEval data " << read_count << std::endl;
+    std::cerr << "Failed to load OpenMidEndingEval data "
+	      << ' ' << read_count << std::endl;
   }
   return initialized_flag == Loaded;
 }
@@ -335,47 +359,47 @@ bool osl::eval::ml::OpenMidEndingEval::setUp(const char *filename)
 bool osl::eval::ml::OpenMidEndingEval::setUp()
 {
   std::string filename = OslConfig::home();
-  filename += "/data/eval.txt";
+  filename += "/data/eval.bin";
   return setUp(filename.c_str());  
 }
 
 osl::eval::ml::
 OpenMidEndingEval::OpenMidEndingEval(const NumEffectState &state)
-  : progress(state)
+  : progress(state), use_progress_independent_value_limit(! OslConfig::hasByoyomi())
 {
   assert(initialized_flag != Zero);
   
   pawns.fill(0);
   black_pawn_count = 0;
-  turn = state.getTurn();
+  turn = state.turn();
   for (int i = PtypeTraits<PAWN>::indexMin;
        i < PtypeTraits<PAWN>::indexLimit; ++i)
   {
-    const Piece pawn = state.getPieceOf(i);
+    const Piece pawn = state.pieceOf(i);
     if (pawn.owner() == BLACK)
       ++black_pawn_count;
     if (pawn.isOnBoard() && !pawn.isPromoted())
-      pawns[pawn.owner()][pawn.position().x() - 1] =
-	pawn.position().y();
+      pawns[pawn.owner()][pawn.square().x() - 1] =
+	pawn.square().y();
   }
   black_major_count = 0;
   black_gold_count = 0;
   for (int i = PtypeTraits<ROOK>::indexMin;
        i < PtypeTraits<ROOK>::indexLimit; ++i)
   {
-    if (state.getPieceOf(i).owner() == BLACK)
+    if (state.pieceOf(i).owner() == BLACK)
       ++black_major_count;
   }
   for (int i = PtypeTraits<BISHOP>::indexMin;
        i < PtypeTraits<BISHOP>::indexLimit; ++i)
   {
-    if (state.getPieceOf(i).owner() == BLACK)
+    if (state.pieceOf(i).owner() == BLACK)
       ++black_major_count;
   }
   for (int i = PtypeTraits<GOLD>::indexMin;
        i < PtypeTraits<GOLD>::indexLimit; ++i)
   {
-    if (state.getPieceOf(i).owner() == BLACK)
+    if (state.pieceOf(i).owner() == BLACK)
       ++black_gold_count;
   }
   updateGoldSilverNearKing(state);
@@ -385,7 +409,7 @@ OpenMidEndingEval::OpenMidEndingEval(const NumEffectState &state)
   ptype_board_count.fill(0);
   for (int i = 0; i < Piece::SIZE; ++i)
   {
-    const Piece piece = state.getPieceOf(i);
+    const Piece piece = state.pieceOf(i);
     if (piece.ptype() == KING)
       continue;
     ++ptype_count[piece.owner()][piece.ptype()];
@@ -403,8 +427,6 @@ OpenMidEndingEval::OpenMidEndingEval(const NumEffectState &state)
     non_pawn_stand_count[WHITE] += 
       state.countPiecesOnStand(osl::WHITE, ptype);
   }
-  pin_mask[WHITE] = state.pin(WHITE);
-  pin_mask[BLACK] = state.pin(BLACK);
   progress_independent_value = PieceEval::eval(state);
   piece_stand_value = PieceStand::eval(state);
   piece_pair_value = PiecePair::eval(state, piece_pair_weights);
@@ -434,6 +456,8 @@ OpenMidEndingEval::OpenMidEndingEval(const NumEffectState &state)
 						    black_vertical,
 						    black_king_vertical);
   recalculated_value =
+    BishopExchangeSilverKing::eval(state) + 
+    EnterKingDefense::eval(state) + 
     King25EffectAttack::eval(state,
 			     black_attack_effect,
 			     black_attack_piece,
@@ -455,8 +479,8 @@ OpenMidEndingEval::OpenMidEndingEval(const NumEffectState &state)
 				   white_attack_piece,
 				   black_attack_supported_piece,
 				   white_attack_supported_piece,
-				   state.getKingPosition<BLACK>().y(),
-				   state.getKingPosition<WHITE>().y());
+				   state.kingSquare<BLACK>().y(),
+				   state.kingSquare<WHITE>().y());
 
     recalculated_stage_value = result_supported + result_supported_y;
     king_rook_bishop[BLACK]=KingRookBishop::evalOne<BLACK>(state);
@@ -498,6 +522,8 @@ OpenMidEndingEval::OpenMidEndingEval(const NumEffectState &state)
 						   black_king_vertical,
 						   white_king_vertical);
   recalculated_stage_value += BishopStandFile5::eval(state);
+  recalculated_stage_value += MajorCheckWithCapture::eval(state);
+  recalculated_stage_value += SilverAdvance26::eval(state);
 
   king_table_value = KingPieceRelative::eval(state);
 
@@ -560,7 +586,7 @@ OpenMidEndingEval::OpenMidEndingEval(const NumEffectState &state)
   recalculated_stage_value += NonPawnAttackedPtypePair::eval(state);
   piece_fork_turn = SilverFork::eval(state, silver_drop);
   piece_fork_turn += BishopRookFork::eval(state, bishop_drop, rook_drop);
-  piece_fork_turn += KnightFork::eval(state, knight_drop);
+  piece_fork_turn += KnightFork::eval(state, knight_fork_squares, knight_drop);
   invalidateCache();
 }
 
@@ -590,7 +616,7 @@ void osl::eval::ml::
 OpenMidEndingEval::update(const NumEffectState &new_state, Move last_move)
 {
   turn = alt(turn);
-  assert(new_state.getTurn() == turn);
+  assert(new_state.turn() == turn);
   if (last_move.isPass())
   {
     invalidateCache();
@@ -606,10 +632,10 @@ void osl::eval::ml::
 OpenMidEndingEval::updateSub(const NumEffectState &new_state, Move last_move)
 {
   assert(last_move.player()==P);
-  const Position opp_king =
-    new_state.getKingPosition<PlayerTraits<P>::opponent>();
-  const Position self_king =
-    new_state.getKingPosition<P>();
+  const Square opp_king =
+    new_state.kingSquare<PlayerTraits<P>::opponent>();
+  const Square self_king =
+    new_state.kingSquare<P>();
   Ptype captured = last_move.capturePtype();
   if (captured != PTYPE_EMPTY)
   {
@@ -649,7 +675,7 @@ OpenMidEndingEval::updateSub(const NumEffectState &new_state, Move last_move)
       const int x_diff = std::abs(last_move.to().x() - opp_king.x());
       if (y_diff <= 2 && x_diff <= 3)
       {
-	--gs_near_king_count[PlayerTraits<P>::opponent][std::max(x_diff, y_diff) - 1];
+	--gs_near_king_count[alt(P)][std::max(x_diff, y_diff) - 1];
       }
     }
   }
@@ -684,7 +710,7 @@ OpenMidEndingEval::updateSub(const NumEffectState &new_state, Move last_move)
   {
     --non_pawn_stand_count[P];
   }
-  if (last_move.ptype() == PPAWN && last_move.isPromote())
+  if (last_move.ptype() == PPAWN && last_move.isPromotion())
   {
     pawns[P][last_move.from().x() - 1] = 0;
   }
@@ -692,7 +718,7 @@ OpenMidEndingEval::updateSub(const NumEffectState &new_state, Move last_move)
   {
     pawns[P][last_move.to().x() - 1] = last_move.to().y();
   }
-  const Position kb = new_state.getKingPosition<BLACK>(), kw = new_state.getKingPosition<WHITE>();
+  const Square kb = new_state.kingSquare<BLACK>(), kw = new_state.kingSquare<WHITE>();
   {
     BoardMask mask = new_state.changedEffects();
     mask.set(last_move.from());
@@ -730,8 +756,6 @@ OpenMidEndingEval::updateSub(const NumEffectState &new_state, Move last_move)
   progress.update(new_state, last_move);
 #endif
 
-  pin_mask[WHITE] = new_state.pin(WHITE);
-  pin_mask[BLACK] = new_state.pin(BLACK);
   progress_independent_value =
     PieceEval::evalWithUpdate<P>(new_state, last_move, progress_independent_value);
   piece_stand_value =
@@ -771,6 +795,8 @@ OpenMidEndingEval::updateSub(const NumEffectState &new_state, Move last_move)
 				       king25_effect_each);
   
   recalculated_value =
+    BishopExchangeSilverKing::eval(new_state) + 
+    EnterKingDefense::eval(new_state) + 
     King25EffectAttack::eval(new_state,
 			     black_attack_effect,
 			     black_attack_piece,
@@ -811,8 +837,8 @@ OpenMidEndingEval::updateSub(const NumEffectState &new_state, Move last_move)
 				   white_attack_piece,
 				   black_attack_supported_piece,
 				   white_attack_supported_piece,
-				   new_state.getKingPosition<BLACK>().y(),
-				   new_state.getKingPosition<WHITE>().y());
+				   new_state.kingSquare<BLACK>().y(),
+				   new_state.kingSquare<WHITE>().y());
     recalculated_stage_value += result_supported + result_supported_y;
     if(isMajorNonPieceOK(last_move.ptype()) || 
        isMajorNonPieceOK(last_move.capturePtype())){ // rook or bishop
@@ -917,6 +943,8 @@ OpenMidEndingEval::updateSub(const NumEffectState &new_state, Move last_move)
   recalculated_stage_value += RookRookPiece::eval(new_state);
   recalculated_stage_value += King25EffectCountCombination::eval(new_state, effect25);
   recalculated_stage_value += BishopStandFile5::eval(new_state);
+  recalculated_stage_value += MajorCheckWithCapture::eval(new_state);
+  recalculated_stage_value += SilverAdvance26::eval(new_state);
 #ifdef USE_TEST_PROGRESS
   recalculated_stage_value += progress.rawData().non_pawn_ptype_attacked_pair_eval[BLACK]
     + progress.rawData().non_pawn_ptype_attacked_pair_eval[WHITE];
@@ -925,7 +953,7 @@ OpenMidEndingEval::updateSub(const NumEffectState &new_state, Move last_move)
 #endif
   piece_fork_turn = SilverFork::eval(new_state, silver_drop);
   piece_fork_turn += BishopRookFork::eval(new_state, bishop_drop, rook_drop);
-  piece_fork_turn += KnightFork::eval(new_state, knight_drop);
+  piece_fork_turn += KnightFork::evalWithUpdate<P>(new_state, last_move, knight_fork_squares, knight_drop);
   invalidateCache();
 }
 
@@ -1008,13 +1036,9 @@ osl::eval::ml::OpenMidEndingEval::debugInfo(const NumEffectState &state)
   debug_info.values[OpenMidEndingEvalDebugInfo::KINGX_BLOCKED_ENDING] =
     kingx_blocked[BLACK][EndgameIndex] + kingx_blocked[WHITE][EndgameIndex];
   debug_info.values[OpenMidEndingEvalDebugInfo::PIN_OPENING] =
-    Pin::eval(state,
-	      pin_mask[BLACK],
-	      pin_mask[WHITE])[0];
+    Pin::eval(state, state.pin(BLACK), state.pin(WHITE))[0];
   debug_info.values[OpenMidEndingEvalDebugInfo::PIN_ENDING] =
-    Pin::eval(state,
-	      pin_mask[BLACK],
-	      pin_mask[WHITE])[EndgameIndex];
+    Pin::eval(state, state.pin(BLACK), state.pin(WHITE))[EndgameIndex];
   debug_info.values[OpenMidEndingEvalDebugInfo::ALL_GOLD_OPENING] =
     AllGold::eval(black_gold_count)[0];
   debug_info.values[OpenMidEndingEvalDebugInfo::ALL_GOLD_ENDING] =
@@ -1245,6 +1269,7 @@ setRandom()
   setRandomOne<LanceEffectPieceKingRelative>();
   setRandomOne<KingMobility>();
   setRandomOne<KingMobilitySum>();
+  setRandomOne<MajorCheckWithCapture>();
 
 }
 #endif
