@@ -3,6 +3,10 @@
 #include "osl/eval/progressEval.h"
 #include "osl/container/pieceValues.h"
 #include "osl/state/numEffectState.h"
+#include "osl/oslConfig.h"
+#include <boost/filesystem.hpp>
+#include <iostream>
+#include <cstdio>
 
 BOOST_STATIC_ASSERT((osl::eval::ProgressEval::ROUND_UP 
 		     & (osl::eval::ProgressEval::ROUND_UP-1))
@@ -33,8 +37,32 @@ ProgressEvalGeneral<Opening>::expect(const NumEffectState& state, Move move) con
 
 template <class Opening>
 void osl::eval::
-ProgressEvalGeneral<Opening>::setUpInternal()
+ProgressEvalGeneral<Opening>::setUpInternal(const char *filename_given)
 {
+  // read weights if exists
+  std::string filename;
+  if (filename_given)
+    filename = filename_given;
+  else {
+    filename = OslConfig::home();
+    filename += "/data/progresseval.txt";
+  }
+  if (boost::filesystem::exists(filename.c_str())) {
+    if (OslConfig::verbose())
+      std::cerr << "loading " << filename << "\n";
+    CArray<int, AdjustableDimension> w;
+    FILE *fp = fopen(filename.c_str(), "r");
+    for (size_t i=0; i<w.size(); ++i) {
+      if (fscanf(fp, "%d", &w[i]) != 1) {
+	std::cerr << filename << " read failed " << i << "\n";
+	break;
+      }
+    }
+    fclose(fp);
+    resetWeights(&w[0]);
+  }
+
+  // others
   for (int i=0; i<PTYPEO_SIZE; ++i) {
     // 序盤を使用
     const PtypeO ptypeo = static_cast<PtypeO>(i+PTYPEO_MIN);
@@ -44,6 +72,14 @@ ProgressEvalGeneral<Opening>::setUpInternal()
   }
 }
 
+template <class Opening>
+void osl::eval::
+ProgressEvalGeneral<Opening>::resetWeights(const int *w)
+{
+  opening_eval_t::resetWeights(w);
+  endgame_eval_t::resetWeights(w+PTYPE_SIZE);
+}
+
 template <class Opening>
 osl::eval::
 ProgressEvalGeneral<Opening>::ProgressEvalGeneral(const NumEffectState& state) 
@@ -55,13 +91,13 @@ ProgressEvalGeneral<Opening>::ProgressEvalGeneral(const NumEffectState& state)
   for (int i = PtypeTraits<ROOK>::indexMin;
        i < PtypeTraits<ROOK>::indexLimit; ++i)
   {
-    if (state.getPieceOf(i).owner() == osl::BLACK)
+    if (state.pieceOf(i).owner() == osl::BLACK)
       ++major_pieces;
   }
   for (int i = PtypeTraits<BISHOP>::indexMin;
        i < PtypeTraits<BISHOP>::indexLimit; ++i)
   {
-    if (state.getPieceOf(i).owner() == osl::BLACK)
+    if (state.pieceOf(i).owner() == osl::BLACK)
       ++major_pieces;
   }
   can_check_pieces.fill(0);
@@ -126,7 +162,7 @@ ProgressEvalGeneral<Opening>::initializeCheckPieceDir(
 {
   if (PtypeTraits<PTYPE>::moveMask & (DirectionTraits<Dir>::mask
 				      | DirectionTraits<LongDir>::mask))
-    can_check_pieces[PlayerTraits<P>::index][Dir] = count;
+    can_check_pieces[P][Dir] = count;
 }
 
 template <class Opening>
@@ -137,7 +173,7 @@ ProgressEvalGeneral<Opening>::calculateMobilityBonusRook(const NumEffectState& s
   int val=0;
   for(int i=PtypeTraits<ROOK>::indexMin;
       i<PtypeTraits<ROOK>::indexLimit;++i){
-    Piece p=state.getPieceOf(i);
+    Piece p=state.pieceOf(i);
     if(p.isOnBoardByOwner<BLACK>()){
       int vc= RookMobility::countVerticalAll(BLACK,state,p);
       int hc= RookMobility::countHorizontalAll(BLACK,state,p);
@@ -174,7 +210,7 @@ ProgressEvalGeneral<Opening>::calculateMobilityBonusBishop(const NumEffectState&
   int val=0;
   for(int i=PtypeTraits<BISHOP>::indexMin;
       i<PtypeTraits<BISHOP>::indexLimit;++i){
-    Piece p=state.getPieceOf(i);
+    Piece p=state.pieceOf(i);
     if(p.isOnBoardByOwner<BLACK>()){
       int c= BishopMobility::countAll(BLACK,state,p);
       if(p.isPromoted())
@@ -201,7 +237,7 @@ ProgressEvalGeneral<Opening>::calculateMobilityBonusLance(const NumEffectState&
   int val=0;
   for(int i=PtypeTraits<LANCE>::indexMin;
       i<PtypeTraits<LANCE>::indexLimit;++i){
-    Piece p=state.getPieceOf(i);
+    Piece p=state.pieceOf(i);
     if(p.isOnBoardByOwner<BLACK>() && !p.isPromoted()){
       int c= LanceMobility::countAll(BLACK,state,p);
       val+=MobilityTable::lance[c];
@@ -337,13 +373,13 @@ ProgressEvalGeneral<Opening>::update(
   
   {
     bool capture_or_drop = last_move.isDrop() || last_move.capturePtype() != PTYPE_EMPTY;
-    const Position kb = new_state.getKingPosition<BLACK>(), kw = new_state.getKingPosition<WHITE>();
+    const Square kb = new_state.kingSquare<BLACK>(), kw = new_state.kingSquare<WHITE>();
     BoardMask mask = new_state.changedEffects();
     mask.set(last_move.from()); mask.set(last_move.to());
-    if ((capture_or_drop && new_state.getTurn() == BLACK)
+    if ((capture_or_drop && new_state.turn() == BLACK)
 	|| mask.anyInRange(Board_Mask_Table3x3.mask(kb)))
       attack_bonus[BLACK] = calculateAttackBonusEach<WHITE>(new_state);
-    if ((capture_or_drop && new_state.getTurn() == WHITE)
+    if ((capture_or_drop && new_state.turn() == WHITE)
 	|| mask.anyInRange(Board_Mask_Table3x3.mask(kw)))
       attack_bonus[WHITE] = calculateAttackBonusEach<BLACK>(new_state);
   }
@@ -364,14 +400,14 @@ int osl::eval::
 ProgressEvalGeneral<Opening>::calculatePinBonus(
   const NumEffectState& state) const
 {
-  const Piece black_king = state.getKingPiece<BLACK>();
-  const Piece white_king = state.getKingPiece<WHITE>();
+  const Piece black_king = state.kingPiece<BLACK>();
+  const Piece white_king = state.kingPiece<WHITE>();
   int bonus = 0;
   PieceMask white_mask = pin_mask[WHITE] = state.pin(WHITE);
   PieceMask black_mask = pin_mask[BLACK] = state.pin(BLACK);
   while (white_mask.any())
   {
-    const Piece piece = state.getPieceOf(white_mask.takeOneBit());
+    const Piece piece = state.pieceOf(white_mask.takeOneBit());
     bonus -= endgame_eval.valueOf(
       black_king, white_king,
       piece) / 4;
@@ -379,7 +415,7 @@ ProgressEvalGeneral<Opening>::calculatePinBonus(
 
   while (black_mask.any())
   {
-    const Piece piece = state.getPieceOf(black_mask.takeOneBit());
+    const Piece piece = state.pieceOf(black_mask.takeOneBit());
     bonus -= endgame_eval.valueOf(
       black_king, white_king,
       piece) / 4;
@@ -397,9 +433,9 @@ ProgressEvalGeneral<Opening>::calculateAttackRooks(
   for(int i = PtypeTraits<ROOK>::indexMin;
       i < PtypeTraits<ROOK>::indexLimit; ++i)
   {
-    const Piece rook = state.getPieceOf(i);
-    if (rook.isOnBoard() && rook.position().canPromote(rook.owner()) &&
-	state.getKingPiece(alt(rook.owner())).position().canPromote(rook.owner()))
+    const Piece rook = state.pieceOf(i);
+    if (rook.isOnBoard() && rook.square().canPromote(rook.owner()) &&
+	state.kingPiece(alt(rook.owner())).square().canPromote(rook.owner()))
     {
       if (rook.owner() == BLACK)
 	++rooks;
@@ -431,19 +467,19 @@ ProgressEvalGeneral<Opening>::calculateAttackBonusOne(
   const NumEffectState& state) const
 {
   const Player defense = PlayerTraits<Attack>::opponent;
-  const Position king = state.getKingPosition<defense>();
+  const Square king = state.kingSquare<defense>();
 
-  const Position target = king + DirectionPlayerTraits<Dir, defense>::offset();
+  const Square target = king + DirectionPlayerTraits<Dir, defense>::offset();
   int result = 0;
 
-  const Piece p = state.getPieceAt(target);
+  const Piece p = state.pieceAt(target);
   if (! p.isEdge() && (Dir != UUR || Attack != BLACK || p.isOnBoard()))
   {
     int effect_diff = (state.countEffect(Attack, target) -
 		       state.countEffect(PlayerTraits<Attack>::opponent, target));
     if ((effect_diff >= 0 && p.isEmpty()) ||
 	(effect_diff >= 1 && !p.isEmpty() &&
-	 p.owner() == PlayerTraits<Attack>::opponent))
+	 p.owner() ==alt(Attack)))
     {
       if (Dir == UL || Dir == U || Dir == UR)
 	result = PtypeEvalTraits<PAWN>::val * 3 * 16;
@@ -460,7 +496,7 @@ ProgressEvalGeneral<Opening>::calculateAttackBonusOne(
 	    state.hasEffectByPtype<ROOK>(Attack,target) ||
 	    state.hasEffectByPtype<BISHOP>(Attack,target))) ||
 	  (p.isEmpty() &&
-	   can_check_pieces[PlayerTraits<Attack>::index][Dir] > 0))
+	   can_check_pieces[Attack][Dir] > 0))
 	result += PtypeEvalTraits<PAWN>::val * 16;
     }
   }
@@ -500,22 +536,22 @@ ProgressEvalGeneral<Opening>::calculateSilverPenalty(
   for (int i = PtypeTraits<SILVER>::indexMin;
        i < PtypeTraits<SILVER>::indexLimit; ++i)
   {
-    const Piece silver = state.getPieceOf(i);
+    const Piece silver = state.pieceOf(i);
     if (!silver.isOnBoard() || silver.isPromoted())
       continue;
 
-    if (silver.position().y() >= 4 && silver.position().y() <= 6)
+    if (silver.square().y() >= 4 && silver.square().y() <= 6)
     {
-      Position dl = Board_Table.nextPosition(silver.owner(),
-					     silver.position(), DL);
-      Position dr = Board_Table.nextPosition(silver.owner(),
-					     silver.position(), DR);
+      Square dl = Board_Table.nextSquare(silver.owner(),
+					     silver.square(), DL);
+      Square dr = Board_Table.nextSquare(silver.owner(),
+					     silver.square(), DR);
       if ((!dl.isOnBoard() ||
-	   state.getPieceAt(dl).isOnBoardByOwner(silver.owner()) ||
-	   state.hasEffectBy(alt(silver.owner()), dl)) &&
+	   state.pieceAt(dl).isOnBoardByOwner(silver.owner()) ||
+	   state.hasEffectAt(alt(silver.owner()), dl)) &&
 	  (!dr.isOnBoard() ||
-	   state.getPieceAt(dr).isOnBoardByOwner(silver.owner()) ||
-	   state.hasEffectBy(alt(silver.owner()), dr)))
+	   state.pieceAt(dr).isOnBoardByOwner(silver.owner()) ||
+	   state.hasEffectAt(alt(silver.owner()), dr)))
       {
 	if (silver.owner() == BLACK)
 	  result -= bonus;
@@ -538,16 +574,16 @@ ProgressEvalGeneral<Opening>::calculateGoldPenalty(
   for (int i = PtypeTraits<GOLD>::indexMin;
        i < PtypeTraits<GOLD>::indexLimit; ++i)
   {
-    const Piece gold = state.getPieceOf(i);
+    const Piece gold = state.pieceOf(i);
     if (!gold.isOnBoard())
       continue;
 
-    if (gold.position().y() >= 4 && gold.position().y() <= 6)
+    if (gold.square().y() >= 4 && gold.square().y() <= 6)
     {
-      Position d = Board_Table.nextPosition(gold.owner(),
-					    gold.position(), D);
-      if ((state.getPieceAt(d).isOnBoardByOwner(gold.owner()) ||
-	   state.hasEffectBy(alt(gold.owner()), d)))
+      Square d = Board_Table.nextSquare(gold.owner(),
+					    gold.square(), D);
+      if ((state.pieceAt(d).isOnBoardByOwner(gold.owner()) ||
+	   state.hasEffectAt(alt(gold.owner()), d)))
       {
 	if (gold.owner() == BLACK)
 	  result -= bonus;
@@ -578,17 +614,17 @@ ProgressEvalGeneral<Opening>::calculateKnightCheckEach(
 {
   const int bonus = (P == BLACK ? 1 : -1) *
     (PtypeEvalTraits<PAWN>::val * 3 + PtypeEvalTraits<PAWN>::val / 2) * 16;
-  const Position king = state.getKingPosition<PlayerTraits<P>::opponent>();
-  const Position up = king +
+  const Square king = state.kingSquare<PlayerTraits<P>::opponent>();
+  const Square up = king +
     DirectionPlayerTraits<U,PlayerTraits<P>::opponent>::offset();
-  if (!state.hasEffectBy<PlayerTraits<P>::opponent>(king)
-      && ! state.getPieceAt(up).isEdge())
+  if (!state.hasEffectAt<PlayerTraits<P>::opponent>(king)
+      && ! state.pieceAt(up).isEdge())
   {
-    const Position ur =
+    const Square ur =
       up + DirectionPlayerTraits<UR,PlayerTraits<P>::opponent>::offset();
-    if (! state.getPieceAt(ur).isEdge() && 
-	state.getPieceAt(ur).isEmpty() &&
-	!state.hasEffectBy<PlayerTraits<P>::opponent>(ur) &&
+    if (! state.pieceAt(ur).isEdge() && 
+	state.pieceAt(ur).isEmpty() &&
+	!state.hasEffectAt<PlayerTraits<P>::opponent>(ur) &&
 	(state.hasPieceOnStand<KNIGHT>(P) ||
 	 state.hasEffectByPtype<KNIGHT>(P, ur)))
     {
@@ -598,11 +634,11 @@ ProgressEvalGeneral<Opening>::calculateKnightCheckEach(
 	return bonus / 2;
     }
 
-    const Position ul =
+    const Square ul =
       up + DirectionPlayerTraits<UL,PlayerTraits<P>::opponent>::offset();
-    if (! state.getPieceAt(ul).isEdge() &&
-	state.getPieceAt(ul).isEmpty() &&
-	!state.hasEffectBy<PlayerTraits<P>::opponent>(ul) &&
+    if (! state.pieceAt(ul).isEdge() &&
+	state.pieceAt(ul).isEmpty() &&
+	!state.hasEffectAt<PlayerTraits<P>::opponent>(ul) &&
 	(state.hasPieceOnStand<KNIGHT>(P) ||
 	 state.hasEffectByPtype<KNIGHT>(P, ul)))
     {
@@ -622,7 +658,7 @@ int osl::eval::
 ProgressEvalGeneral<Opening>::calculateEnterKingBonus(
   const NumEffectState& state) const
 {
-  const Position king = state.getKingPosition<P>();
+  const Square king = state.kingSquare<P>();
 
   if ((P == BLACK && king.y() > 4) ||
       (P == WHITE && king.y() < 6))
@@ -640,24 +676,24 @@ ProgressEvalGeneral<Opening>::calculateEnterKingBonus(
     bool found_opening = false;
     for (int x = min_x; x <= max_x; ++x)
     {
-      Position pos(x, y);
-      Piece piece = state.getPieceAt(pos);
+      Square pos(x, y);
+      Piece piece = state.pieceAt(pos);
       if (piece.isEmpty())
       {
-	if (!state.hasEffectBy<PlayerTraits<P>::opponent>(pos))
+	if (!state.hasEffectAt<PlayerTraits<P>::opponent>(pos))
 	  found_opening = true;
 	else if (state.countEffect(P, pos) <=
-		 state.countEffect(PlayerTraits<P>::opponent, pos))
+		 state.countEffect(alt(P), pos))
 	  return 0;
       }
-      else if (piece.owner() == PlayerTraits<P>::opponent)
+      else if (piece.owner() == alt(P))
       {
 	return 0;
       }
       else if (piece.owner() == P)
       {
 	if (state.countEffect(P, pos) <
-	    state.countEffect(PlayerTraits<P>::opponent, pos))
+	    state.countEffect(alt(P), pos))
 	  return 0;
       }
       else
@@ -677,7 +713,7 @@ int osl::eval::
 ProgressEvalGeneral<Opening>::calculateMiddleKingBonus(
   const NumEffectState& state) const
 {
-  const Position king = state.getKingPosition<P>();
+  const Square king = state.kingSquare<P>();
 
   if ((P == BLACK && king.y() >= 6 && major_pieces == 4) ||
       (P == WHITE && king.y() <= 4 && major_pieces == 0))
@@ -696,28 +732,28 @@ ProgressEvalGeneral<Opening>::calculateRookRankBonus(
   for (int i = PtypeTraits<ROOK>::indexMin;
        i < PtypeTraits<ROOK>::indexLimit; ++i)
   {
-    const Piece rook = state.getPieceOf(i);
+    const Piece rook = state.pieceOf(i);
     const Player owner = rook.owner();
     const int target_y = owner == BLACK ? 3 : 7;
     const int inbetween_y = owner == BLACK ? 4 : 6;
-    if (rook.isOnBoard() && !rook.position().canPromote(owner))
+    if (rook.isOnBoard() && !rook.square().canPromote(owner))
     {
-      const Piece rank5 = state.getPieceAt(Position(rook.position().x(), 5));
-      const Piece rank4 = state.getPieceAt(Position(rook.position().x(),
+      const Piece rank5 = state.pieceAt(Square(rook.square().x(), 5));
+      const Piece rank4 = state.pieceAt(Square(rook.square().x(),
 						    inbetween_y));
-      const Position rank3_pos(rook.position().x(), target_y);
+      const Square rank3_pos(rook.square().x(), target_y);
       if (state.hasEffectByPtype<SILVER>(
 	    owner,
-	    Position(rook.position().x(),
+	    Square(rook.square().x(),
 	    inbetween_y)) &&
 	  !rank5.isOnBoardByOwner(alt(owner)) &&
-	  !state.getPieceAt(rank3_pos).isOnBoardByOwner(owner) &&
+	  !state.pieceAt(rank3_pos).isOnBoardByOwner(owner) &&
 	  state.countEffect(alt(owner), 
-			    Position(rook.position().x(), target_y)) <= 1 &&
+			    Square(rook.square().x(), target_y)) <= 1 &&
 	  state.countEffect(owner,
-			    Position(rook.position().x(), inbetween_y)) >=
+			    Square(rook.square().x(), inbetween_y)) >=
 	  state.countEffect(alt(owner),
-			    Position(rook.position().x(), inbetween_y)))
+			    Square(rook.square().x(), inbetween_y)))
       {
 	if (rook.owner() == BLACK)
 	  bonus += PtypeEvalTraits<PAWN>::val * 2 * 16;
@@ -726,14 +762,14 @@ ProgressEvalGeneral<Opening>::calculateRookRankBonus(
       }
       else if (((rank5.isOnBoardByOwner(owner) &&
 		 rank5.ptype() == PAWN &&
-		 state.hasEffectByPiece(rook, rank5.position())) ||
+		 state.hasEffectByPiece(rook, rank5.square())) ||
 		(rank4.isOnBoardByOwner(owner) &&
 		 rank4.ptype() == PAWN &&
-		 state.hasEffectByPiece(rook, rank4.position()))) &&
-	       !state.hasEffectBy(alt(owner),
+		 state.hasEffectByPiece(rook, rank4.square()))) &&
+	       !state.hasEffectAt(alt(owner),
 				      rank3_pos) &&
 		  state.countEffect(alt(owner),
-				    Position(rook.position().x(),
+				    Square(rook.square().x(),
 						  inbetween_y)) <= 1)
       {
 	if (rook.owner() == BLACK)
@@ -742,8 +778,8 @@ ProgressEvalGeneral<Opening>::calculateRookRankBonus(
 	  bonus -= PtypeEvalTraits<PAWN>::val * 2 * 16;
       }
       else if (state.hasEffectByPiece(rook, rank3_pos) &&
-	       !state.hasEffectBy(alt(owner), rank3_pos) &&
-	       !state.isPawnMaskSet(owner, rook.position().x()))
+	       !state.hasEffectAt(alt(owner), rank3_pos) &&
+	       !state.isPawnMaskSet(owner, rook.square().x()))
       {
 	if (rook.owner() == BLACK)
 	  bonus += PtypeEvalTraits<PAWN>::val * 16;
