@@ -8,6 +8,22 @@
 #include <iostream>
 #include <fstream>
 
+bool osl::progress::ml::
+operator==(const NewProgressData& l, const NewProgressData& r)
+{
+  return l.progresses == r.progresses
+    && l.attack5x5_progresses == r.attack5x5_progresses
+    && l.stand_progresses == r.stand_progresses
+    && l.effect_progresses == r.effect_progresses
+    && l.defenses == r.defenses
+    && l.rook == r.rook && l.bishop == r.bishop && l.gold == r.gold
+    && l.silver == r.silver && l.promoted == r.promoted
+    && l.king_relative_attack == r.king_relative_attack
+    && l.king_relative_defense == r.king_relative_defense
+    && l.non_pawn_ptype_attacked_pair == r.non_pawn_ptype_attacked_pair
+    && l.non_pawn_ptype_attacked_pair_eval == r.non_pawn_ptype_attacked_pair_eval;
+}
+
 osl::misc::CArray<int, 81*15*10>
 osl::progress::ml::NewProgress::attack_relative;
 osl::misc::CArray<int, 81*15*10>
@@ -174,9 +190,9 @@ bool osl::progress::ml::NewProgress::setUp(const char *filename)
   }
   for(int king_x=1;king_x<=9;king_x++){
     for(int king_y=1;king_y<=9;king_y++){
-      Position king(king_x,king_y);
+      Square king(king_x,king_y);
       int king_index=(king_x-1)*9+king_y-1;
-      const Position center = Centering5x3::adjustCenter(king);
+      const Square center = Centering5x3::adjustCenter(king);
       const int min_x = center.x() - 2;
       const int min_y = center.y() - 1;
       int i=0;
@@ -184,7 +200,7 @@ bool osl::progress::ml::NewProgress::setUp(const char *filename)
       {
 	for (int dy=0; dy<3; ++dy,++i)
 	{
-	  const Position target(min_x+dx,min_y+dy);
+	  const Square target(min_x+dx,min_y+dy);
 	  int index0=king_index*15+i;
 	  int index_a=index0*10;
 	  int index_d=index0*10;
@@ -242,13 +258,13 @@ template <osl::Player P>
 void osl::progress::ml::NewProgress::progressOne(
   const NumEffectState &state, int &attack, int &defense)
 {
-  const Position king = state.getKingPosition<P>();
-  const Position center = Centering5x3::adjustCenter(king);
+  const Square king = state.kingSquare<P>();
+  const Square center = Centering5x3::adjustCenter(king);
   const int min_x = center.x() - 2;
   const int min_y = center.y() - 1;
 
   attack = defense = 0;
-  Position kingRel=king;
+  Square kingRel=king;
   if(P==WHITE){
     kingRel=kingRel.rotate180();
   }
@@ -258,9 +274,9 @@ void osl::progress::ml::NewProgress::progressOne(
   {
     for (int dy=0; dy<3; ++dy)
     {
-      const Position target(min_x+dx,min_y+dy);
+      const Square target(min_x+dx,min_y+dy);
       const int attack_count =
-	state.countEffect(PlayerTraits<P>::opponent, target);
+	state.countEffect(alt(P), target);
       const int defense_count =
 	state.countEffect(P, target);
       attack += attack_count *attack_relative[index_a]+
@@ -282,28 +298,27 @@ template <osl::Player P>
 void osl::progress::ml::NewProgress::updateAttack5x5PiecesAndState(
   const NumEffectState &state)
 {
-  const Position king = state.getKingPosition<P>();
+  const Square king = state.kingSquare<P>();
   const int min_x = std::max(1, king.x() - 2);
   const int max_x = std::min(9, king.x() + 2);
   const int min_y = std::max(1, king.y() - 2);
   const int max_y = std::min(9, king.y() + 2);
-  effect_progresses[PlayerTraits<P>::index] = 0;
+  effect_progresses[P] = 0;
 
   PieceMask mask;
   for (int y = min_y; y <= max_y; ++y)
   {
     for (int x = min_x; x <= max_x; ++x)
     {
-      const Position target(x, y);
-      const NumBitmapEffect effect = state.getEffect(target);
+      const Square target(x, y);
+      const NumBitmapEffect effect = state.effectSetAt(target);
       const int effect_diff =
-	effect.countEffect(PlayerTraits<P>::opponent) -
-	effect.countEffect(P);
+	effect.countEffect(alt(P)) - effect.countEffect(P);
       const int x_diff = std::abs(x - king.x());
       const int y_diff = (P == WHITE ? king.y() - y : y - king.y());
       int index = std::max(std::min(effect_diff, 2), -2) + 2 + 5 * x_diff +
 	5 * 3 * (y_diff + 2);
-      effect_progresses[PlayerTraits<P>::index] += effectstate_weight[index];
+      effect_progresses[P] += effectstate_weight[index];
       mask |= effect;
     }
   }
@@ -315,7 +330,7 @@ void osl::progress::ml::NewProgress::updateAttack5x5Pieces(
   PieceMask mask, const NumEffectState& state)
 {
   const Player attack = PlayerTraits<P>::opponent;
-  mask &= state.getOnBoardMask(attack);
+  mask &= state.piecesOnBoard(attack);
 
   rook[attack] = mask.selectBit<ROOK>().countBit();
   bishop[attack] = mask.selectBit<BISHOP>().countBit();
@@ -334,11 +349,11 @@ int osl::progress::ml::NewProgress::attack5x5Value(
   const NumEffectState &state) const
 {
   const Player attack = PlayerTraits<P>::opponent;
-  int king_x = state.getKingPosition<P>().x();
+  int king_x = state.kingSquare<P>().x();
   if (king_x > 5)
     king_x = 10 - king_x;
-  const int king_y = (P == BLACK ? state.getKingPosition<P>().y() :
-		      10 - state.getKingPosition<P>().y());
+  const int king_y = (P == BLACK ? state.kingSquare<P>().y() :
+		      10 - state.kingSquare<P>().y());
   return (attack5x5_x_weight[index5x5x(
 	      rook[attack] + state.countPiecesOnStand<ROOK>(attack),
 	      bishop[attack] + state.countPiecesOnStand<BISHOP>(attack),
@@ -357,15 +372,15 @@ void
 osl::progress::ml::NewProgress::updatePieceKingRelativeBonus(
   const NumEffectState &state)
 {
-  const CArray<Position,2> kings = {{ 
-      state.getKingPosition(BLACK),
-      state.getKingPosition(WHITE),
+  const CArray<Square,2> kings = {{ 
+      state.kingSquare(BLACK),
+      state.kingSquare(WHITE),
     }};
   king_relative_attack.fill(0);
   king_relative_defense.fill(0);
   for (int i = 0; i < Piece::SIZE; ++i)
   {
-    const Piece piece = state.getPieceOf(i);
+    const Piece piece = state.pieceOf(i);
     if (piece.ptype() == osl::KING || !piece.isOnBoard())
       continue;
     Player pl = piece.owner();
@@ -382,15 +397,15 @@ template <osl::Player Owner>
 void osl::progress::ml::NewProgress::
 updateNonPawnAttackedPtypePairOne(const NumEffectState& state)
 {
-  PieceMask attacked = state.effectedMask(alt(Owner)) & state.getOnBoardMask(Owner);
-  attacked.reset(state.getKingPiece<Owner>().number());
+  PieceMask attacked = state.effectedMask(alt(Owner)) & state.piecesOnBoard(Owner);
+  attacked.reset(state.kingPiece<Owner>().number());
   mask_t ppawn = state.promotedPieces().getMask<PAWN>() & attacked.selectBit<PAWN>();
   attacked.clearBit<PAWN>();
   attacked.orMask(PtypeFuns<PAWN>::indexNum, ppawn);
   PieceVector pieces;
   while (attacked.any())
   {
-    const Piece piece = state.getPieceOf(attacked.takeOneBit());
+    const Piece piece = state.pieceOf(attacked.takeOneBit());
     pieces.push_back(piece);
   }
   typedef eval::ml::NonPawnAttackedPtypePair feature_t;
@@ -424,16 +439,16 @@ osl::progress::ml::NewProgress::NewProgress(
   assert(initialized_flag);
   
   progressOne<BLACK>(state,
-		     progresses[PlayerTraits<BLACK>::index],
-		     defenses[PlayerTraits<WHITE>::index]);
+		     progresses[BLACK],
+		     defenses[WHITE]);
   progressOne<WHITE>(state,
-		     progresses[PlayerTraits<WHITE>::index],
-		     defenses[PlayerTraits<BLACK>::index]);
+		     progresses[WHITE],
+		     defenses[BLACK]);
   updateAttack5x5PiecesAndState<BLACK>(state);
   updateAttack5x5PiecesAndState<WHITE>(state);
-  attack5x5_progresses[PlayerTraits<BLACK>::index] =
+  attack5x5_progresses[BLACK] =
     attack5x5Value<BLACK>(state);
-  attack5x5_progresses[PlayerTraits<WHITE>::index] =
+  attack5x5_progresses[WHITE] =
     attack5x5Value<WHITE>(state);
   stand_progresses.fill(0);
   BOOST_FOREACH(Ptype ptype, PieceStand::order)
@@ -444,12 +459,12 @@ osl::progress::ml::NewProgress::NewProgress(
       state.countPiecesOnStand(WHITE, ptype);
     for (int j = 0; j < black_count; ++j)
     {
-      stand_progresses[PlayerTraits<WHITE>::index] +=
+      stand_progresses[WHITE] +=
 	stand_weight[Ptype_Table.getIndexMin(ptype) + j];
     }
     for (int j = 0; j < white_count; ++j)
     {
-      stand_progresses[PlayerTraits<BLACK>::index] +=
+      stand_progresses[BLACK] +=
 	stand_weight[Ptype_Table.getIndexMin(ptype) + j];
     }
   }
@@ -464,22 +479,18 @@ void osl::progress::ml::NewProgress::updateMain(
   Move last_move)
 {
   const Player altP=PlayerTraits<P>::opponent;
-  assert(new_state.getTurn()==altP);
+  assert(new_state.turn()==altP);
   assert(last_move.player()==P);
-  const Position kb = new_state.getKingPosition<BLACK>(), kw = new_state.getKingPosition<WHITE>();
+  const Square kb = new_state.kingSquare<BLACK>(), kw = new_state.kingSquare<WHITE>();
   const BoardMask mb = new_state.changedEffects(BLACK), mw = new_state.changedEffects(WHITE);
   const bool king_move = last_move.ptype() == KING;
   if ((king_move && altP == BLACK) || mb.anyInRange(Board_Mask_Table5x3_Center.mask(kw)) || mw.anyInRange(Board_Mask_Table5x3_Center.mask(kw)))
   {
-    progressOne<WHITE>(new_state,
-		       progresses[PlayerTraits<WHITE>::index],
-		       defenses[PlayerTraits<BLACK>::index]);
+    progressOne<WHITE>(new_state,progresses[WHITE],defenses[BLACK]);
   }
   if ((king_move && altP == WHITE) || mw.anyInRange(Board_Mask_Table5x3_Center.mask(kb)) || mb.anyInRange(Board_Mask_Table5x3_Center.mask(kb)))
   {
-    progressOne<BLACK>(new_state,
-		       progresses[PlayerTraits<BLACK>::index],
-		       defenses[PlayerTraits<WHITE>::index]);
+    progressOne<BLACK>(new_state,progresses[BLACK],defenses[WHITE]);
   }
 
   const Ptype captured = last_move.capturePtype();
@@ -507,9 +518,9 @@ void osl::progress::ml::NewProgress::updateMain(
   }
   else
   {
-    const CArray<Position,2> kings = {{ 
-	new_state.getKingPosition(BLACK),
-	new_state.getKingPosition(WHITE),
+    const CArray<Square,2> kings = {{ 
+	new_state.kingSquare(BLACK),
+	new_state.kingSquare(WHITE),
       }};
     if (!last_move.isDrop())
     {
@@ -559,10 +570,10 @@ void osl::progress::ml::NewProgress::updateSub(
   Move last_move)
 {
   const Player altP=PlayerTraits<P>::opponent;
-  assert(new_state.getTurn()==altP);
+  assert(new_state.turn()==altP);
   if (last_move.isPass())
     return;
-  const Position kb = new_state.getKingPosition<BLACK>(), kw = new_state.getKingPosition<WHITE>();
+  const Square kb = new_state.kingSquare<BLACK>(), kw = new_state.kingSquare<WHITE>();
   const BoardMask mb = new_state.changedEffects(BLACK), mw = new_state.changedEffects(WHITE);
   const bool king_move = last_move.ptype() == KING;
   const Ptype captured = last_move.capturePtype();
@@ -572,12 +583,12 @@ void osl::progress::ml::NewProgress::updateSub(
       mw.anyInRange(Board_Mask_Table5x5.mask(kw)))
   {
     updateAttack5x5PiecesAndState<WHITE>(new_state);
-    attack5x5_progresses[PlayerTraits<WHITE>::index] =
+    attack5x5_progresses[WHITE] =
       attack5x5Value<WHITE>(new_state);
   }
   else if (altP == WHITE &&(last_move.isDrop() || captured != PTYPE_EMPTY))
   {
-    attack5x5_progresses[PlayerTraits<WHITE>::index] =
+    attack5x5_progresses[WHITE] =
       attack5x5Value<WHITE>(new_state);
   }
   if ((king_move && altP == WHITE) ||
@@ -585,12 +596,12 @@ void osl::progress::ml::NewProgress::updateSub(
       mb.anyInRange(Board_Mask_Table5x5.mask(kb)))
   {
     updateAttack5x5PiecesAndState<BLACK>(new_state);
-    attack5x5_progresses[PlayerTraits<BLACK>::index] =
+    attack5x5_progresses[BLACK] =
       attack5x5Value<BLACK>(new_state);
   }
   else if (altP == BLACK && (last_move.isDrop() || captured != PTYPE_EMPTY))
   {
-    attack5x5_progresses[PlayerTraits<BLACK>::index] =
+    attack5x5_progresses[BLACK] =
       attack5x5Value<BLACK>(new_state);
   }
   updateMain<P>(new_state, last_move);
