@@ -5,7 +5,7 @@
 #include "osl/rating/group.h"
 #include "osl/rating/group/captureGroup.h"
 #include "osl/rating/group/escape.h"
-#include "osl/rating/group/positionGroup.h"
+#include "osl/rating/group/squareGroup.h"
 #include "osl/rating/group/patternGroup.h"
 #include "osl/rating/group/bigramGroup.h"
 #include "osl/rating/group/king8Group.h"
@@ -202,8 +202,6 @@ void osl::rating::
 FeatureSet::generateRating(const NumEffectState& state, const RatingEnv& env,
 			   int limit, RatedMoveVector& out, bool in_pv_or_all) const
 {
-  static bool use_log_linear_probability = OslConfig::useLogLinearProbability();
-  in_pv_or_all |= use_log_linear_probability;
 #if (defined RATING_STAT) || (defined RATING_STAT2)
   in_pv_or_all = true;
 #endif
@@ -213,11 +211,11 @@ FeatureSet::generateRating(const NumEffectState& state, const RatingEnv& env,
   if (in_check)
     GenerateEscapeKing::generate(state, moves);
   else
-    GenerateAllMoves::generate(state.getTurn(), state, moves);
+    GenerateAllMoves::generate(state.turn(), state, moves);
 
   for (size_t i=0; i<moves.size(); ++i) {
     if (moves[i].ptype() == KING) {
-      if (state.hasEffectBy(alt(state.getTurn()), moves[i].to()))
+      if (state.hasEffectAt(alt(state.turn()), moves[i].to()))
 	continue;
     } else {
       if (! in_check && env.my_pin.any() && ! moves[i].isDrop()
@@ -389,55 +387,15 @@ namespace osl
 }
 #endif
 
-void osl::rating::
-FeatureSet::normalize(const RatedMoveVector& score,
-		      MoveLogProbVector& out)
-{
-  static const double scale = 100.0 / log(0.5);
-  static const double scale_exp = 1.0/400.0*log(10.0);
-  double sum = 0.0;
-  FixedCapacityVector<double, Move::MaxUniqMoves> expscores;
-  BOOST_FOREACH(RatedMove m, score) {
-    double e = exp(m.rating()*scale_exp);
-    expscores.push_back(e);
-    sum += e;
-  }
-  for (size_t i=0; i<score.size(); ++i) {
-    double p = expscores[i]/sum;
-    if (std::isnan(p) || p <= 1.0/(1<<12)) p = 1.0/(1<<12);
-    int log_prob = std::max(10,(int)floor(log(p)*scale));
-    assert(log_prob >= 0);
-    out.push_back(MoveLogProb(score[i].move(), log_prob));
-  }
-}
-
-void osl::rating::
-FeatureSet::generateLogProbNormalized(const NumEffectState& state, const RatingEnv& env,
-				      int limit, MoveLogProbVector& out, bool in_pv_or_all) const
-{
-  RatedMoveVector score;
-  generateRating(state, env, limit, score, in_pv_or_all);
-  if (score.empty())
-    return;
-  normalize(score, out);
-}
-
 void osl::rating::
 FeatureSet::generateLogProb(const NumEffectState& state, const RatingEnv& env,
 			    int limit, MoveLogProbVector& out, bool in_pv_or_all) const
 {
-  static bool use_log_linear_probability = OslConfig::useLogLinearProbability();
-
   RatedMoveVector score;
   generateRating(state, env, limit, score, in_pv_or_all);
   if (score.empty())
     return;
 
-  if (use_log_linear_probability) {
-    normalize(score, out);
-    return;
-  }
-
   const int highest = score[0].rating();
   const int progress8 = env.progress.value()/2;
   for (size_t i=0; i<score.size(); ++i) {
@@ -736,9 +694,9 @@ osl::rating::
 StandardFeatureSet::StandardFeatureSet(bool allow_load_failure)
 {
   add(new CaptureGroup());
-  add(new PositionYGroup());
+  add(new SquareYGroup());
   add(new RelativeKingXGroup(true));
-  add(new PositionXGroup());
+  add(new SquareXGroup());
   add(new TakeBackGroup());
   add(new RelativeKingYGroup(true));
   add(new FromEffectGroup());
