@@ -551,7 +551,7 @@ AlphaBeta2Parallel<EvalT>::nextMove(int tree_id)
       ? tree[parent]->template nextMove<BLACK>() 
       : tree[parent]->template nextMove<WHITE>();
     if (m.validMove()) {
-      assert(m.getMove().player() == info->turn);
+      assert(m.player() == info->turn);
       ++(info->move_index);
     }
     return std::make_pair(m, old_index);
@@ -596,7 +596,7 @@ AlphaBeta2Tree<EvalT>::testMoveRoot(int tree_id, const MoveLogProb& m)
     w = parent->window;
     assert(w.isConsistent());
   }
-  assert(P == m.getMove().player());
+  assert(P == m.player());
 #ifndef GPSONE
   if (this->multi_pv) {
     int width = this->multi_pv*this->eval.captureValue(newPtypeO(P, PAWN))/200;
@@ -608,7 +608,7 @@ AlphaBeta2Tree<EvalT>::testMoveRoot(int tree_id, const MoveLogProb& m)
   const int result = alphaBetaSearch<P>(m, w, false);
 
   if (eval::betterThan(P, result, parent->search_value)) {
-    makePV(m.getMove());
+    makePV(m.move());
     if (eval::betterThan(P, result, w.beta(P))) {
       {
 	boost::mutex::scoped_lock lk_smp(shared->lock_smp);
@@ -631,7 +631,7 @@ AlphaBeta2Tree<EvalT>::testMoveRoot(int tree_id, const MoveLogProb& m)
       parent->window.alpha(P) = result + EvalTraits<P>::delta;
       parent->best_move = m;
       parent->search_value = result;
-      updateRootPV(P, std::cerr, result, m.getMove());
+      updateRootPV(P, std::cerr, result, m.move());
       assert(parent->window.isConsistent());
       shared->tree[shared->parentID(tree_id)]->pv[0] = pv[0];
     }
@@ -639,7 +639,7 @@ AlphaBeta2Tree<EvalT>::testMoveRoot(int tree_id, const MoveLogProb& m)
 #ifndef GPSONE
   else if (this->multi_pv && !stopping() 
 	   && eval::betterThan(P, result, w.alpha(P)))
-    addMultiPV(P, result, m.getMove());
+    addMultiPV(P, result, m.move());
 #endif
 }
 
@@ -687,7 +687,7 @@ examineMovesRootPar(int tree_id)
 #ifndef GPSONE
     if (OslConfig::usiMode()) {
       boost::mutex::scoped_lock lk(OslConfig::lock_io);      
-      UsiReporter::rootMove(m.getMove());
+      UsiReporter::rootMove(m.move());
     }
 #endif
     try {
@@ -695,6 +695,9 @@ examineMovesRootPar(int tree_id)
 	testMoveRoot<BLACK>(tree_id, m);
       else
 	testMoveRoot<WHITE>(tree_id, m);
+      if (this->root_limit >= 1600)
+	this->checkmate_searcher->runGC(this->table->isVerbose(),
+					lastMemoryUseRatio1000());
     }
     catch (BetaCut& e) {
       std::cerr << "caught BetaCut at root " << info->thread_id << "\n";
@@ -744,7 +747,7 @@ AlphaBeta2Tree<EvalT>::testMoveOther(int tree_id, const MoveLogProb& m, size_t i
   }
   if (stopping())
     return false;
-  assert(P == m.getMove().player());
+  assert(P == m.player());
   const int result = alphaBetaSearch<P>(m, w, in_pv);
   if (stopping())
     return false;
@@ -758,7 +761,7 @@ AlphaBeta2Tree<EvalT>::testMoveOther(int tree_id, const MoveLogProb& m, size_t i
     parent_search_value = parent->search_value;
   }
   if (eval::betterThan(P, result, parent_search_value)) {
-    makePV(m.getMove());
+    makePV(m.move());
     if (eval::betterThan(P, result, w.beta(P))) {
       cut = true;
       {
@@ -861,7 +864,7 @@ examineMovesOther(int tree_id)
     if (in_pv) {
       in_pv = ! parent->best_move.validMove();
     }
-    assert(parent->turn == m.first.getMove().player());
+    assert(parent->turn == m.first.player());
     try {
       const bool cut_by_move =
 	(parent->turn == BLACK)
@@ -920,7 +923,7 @@ namespace osl
   namespace search
   {
 #ifndef MINIMAL
-    template class AlphaBeta2Parallel<eval::ProgressEval>;
+    template struct AlphaBeta2Parallel<eval::ProgressEval>;
 
     template 
     bool AlphaBeta2Tree<eval::ProgressEval>::examineMovesOther<BLACK>(Window&, MoveLogProb&, int&, int&, int&, int&);
@@ -932,7 +935,7 @@ namespace osl
     template
     void AlphaBeta2Tree<eval::ProgressEval>::examineMovesRootPar<WHITE>(const MoveLogProbVector&, size_t, Window, MoveLogProb&, int&);
 #endif
-    template class AlphaBeta2Parallel<eval::ml::OpenMidEndingEval>;
+    template struct AlphaBeta2Parallel<eval::ml::OpenMidEndingEval>;
 
     template 
     bool AlphaBeta2Tree<eval::ml::OpenMidEndingEval>::examineMovesOther<BLACK>(Window&, MoveLogProb&, int&, int&, int&, int&);
