@@ -3,7 +3,7 @@
 #include "osl/checkmate/dualDfpn.h"
 #include "osl/checkmate/dfpn.h"
 #include "osl/record/csaString.h"
-#include "osl/apply_move/applyMove.h"
+#include "osl/oslConfig.h"
 
 #include <cppunit/TestCase.h>
 #include <cppunit/extensions/HelperMacros.h>
@@ -32,17 +32,15 @@ public:
 };
 CPPUNIT_TEST_SUITE_REGISTRATION(OracleProverTest);
 
-extern bool isShortTest;
-
 const size_t limit = 400000;
 static bool isCheckmate(const char *org, const char *similar)
 {
   NumEffectState state(CsaString(org).getInitialState());
 
   DualDfpn checkmate_search(limit);
-  checkmate_search.setVerbose(! isShortTest);
+  checkmate_search.setVerbose(OslConfig::verbose());
   Move check_move;
-  PathEncoding path(state.getTurn());
+  PathEncoding path(state.turn());
   const bool win = checkmate_search.isWinningState(limit, state, HashKey(state),
 					  path, check_move);
   CPPUNIT_ASSERT(win);
@@ -63,7 +61,7 @@ static bool isCheckmateRaw(const char *org, const char *similar)
   Dfpn dfpn;
   dfpn.setTable(&table);
   Move check_move;
-  PathEncoding path(state.getTurn());
+  PathEncoding path(state.turn());
   const ProofDisproof win = dfpn.hasCheckmateMove
     (state, HashKey(state), path, limit, check_move);
   CPPUNIT_ASSERT(win.isCheckmateSuccess());
@@ -234,7 +232,7 @@ void OracleProverTest::testCheckLong()
 void OracleProverTest::testAdjustMove()
 {
   const Move m1 = 
-    Move(Position(8,9),Position(7,9),KING,PTYPE_EMPTY,false,BLACK);
+    Move(Square(8,9),Square(7,9),KING,PTYPE_EMPTY,false,BLACK);
   Move m2 = m1.newCapture(PAWN);
   CPPUNIT_ASSERT(m1 != m2);
   CPPUNIT_ASSERT_EQUAL(m1.from(), m2.from());
