@@ -65,7 +65,6 @@ CPPUNIT_TEST_SUITE_REGISTRATION(NtesukiSearcherTest);
 
 using namespace osl;
 using namespace osl::ntesuki;
-extern bool isShortTest;
 const size_t TABLE_LIMIT = 5000000;
 const size_t READ_LIMIT = 10000000;
 
@@ -94,7 +93,7 @@ void NtesukiSearcherTest::test0tesuki0()
   NtesukiSearcher searcher(nState, gam.get(),  TABLE_LIMIT,
 			   &stop_flag, !isShortTest, 1);
 
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
+  const int ntesuki_num = searcher.searchSlow(nState.turn(), READ_LIMIT);
   CPPUNIT_ASSERT_EQUAL(0, ntesuki_num);
 }
 
@@ -122,9 +121,9 @@ void NtesukiSearcherTest::test0tesuki0op()
     gam(new movegen_t());
   NumEffectState nState(state);
   NtesukiSearcher searcher(nState, gam.get(),  TABLE_LIMIT,
-			   &stop_flag, !isShortTest, 1);
+			   &stop_flag, OslConfig::verbose(), 1);
 
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
+  const int ntesuki_num = searcher.searchSlow(nState.turn(), READ_LIMIT);
   CPPUNIT_ASSERT_EQUAL(0, ntesuki_num);
 }
 
@@ -146,9 +145,9 @@ void NtesukiSearcherTest::test0tesuki1()
     gam(new movegen_t());
   NumEffectState nState(state);
   NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT,
-			   &stop_flag,!isShortTest, 1);
+			   &stop_flag,OslConfig::verbose(), 1);
 
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn());
+  const int ntesuki_num = searcher.searchSlow(nState.turn());
   CPPUNIT_ASSERT_EQUAL(0, ntesuki_num);
 }
 
@@ -199,15 +198,15 @@ void NtesukiSearcherTest::test0tesuki2()
     boost::scoped_ptr<osl::ntesuki::NtesukiMoveGenerator>
       gam(new movegen_t());
     NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT,
-			     &stop_flag, !isShortTest, 1);
+			     &stop_flag, OslConfig::verbose(), 1);
     
-    if (!isShortTest)
+    if (OslConfig::verbose())
     {
       std::cerr << "trying" << i << "\n"
 		<< state;
     }
     const int ntesuki_num = searcher.searchSlow(osl::WHITE, READ_LIMIT);
-    if (!isShortTest)
+    if (OslConfig::verbose())
     {
       std::cerr << "\tresult\t" << ntesuki_num
 		<< std::endl;
@@ -235,9 +234,9 @@ void NtesukiSearcherTest::test1tesuki0()
   boost::scoped_ptr<osl::ntesuki::NtesukiMoveGenerator>
     gam(new movegen_t());
   NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT,
-			   &stop_flag, !isShortTest);
+			   &stop_flag, OslConfig::verbose());
 
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
+  const int ntesuki_num = searcher.searchSlow(nState.turn(), READ_LIMIT);
   CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
 }
 
@@ -260,9 +259,9 @@ void NtesukiSearcherTest::test2tesuki0()
     gam(new movegen_t());
   NumEffectState nState(state);
   NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT,
-			   &stop_flag, !isShortTest);
+			   &stop_flag, OslConfig::verbose());
 
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn());
+  const int ntesuki_num = searcher.searchSlow(nState.turn());
   CPPUNIT_ASSERT_EQUAL(2, ntesuki_num);
 }
 
@@ -284,9 +283,9 @@ void NtesukiSearcherTest::test2tesuki1()
     gam(new movegen_t());
   NumEffectState nState(state);
   NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT,
-			   &stop_flag, !isShortTest);
+			   &stop_flag, OslConfig::verbose());
 
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn());
+  const int ntesuki_num = searcher.searchSlow(nState.turn());
   CPPUNIT_ASSERT_EQUAL(2, ntesuki_num);
 }
 
@@ -309,13 +308,13 @@ void NtesukiSearcherTest::testRecognizeAttackBack0()
     gam(new movegen_t());
   NumEffectState nState(state);
   NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag,
-			   !isShortTest, 2,
+			   OslConfig::verbose(), 2,
 			   osl::ntesuki::NtesukiRecord::pn_iw,
 			   osl::ntesuki::NtesukiRecord::pn_ps,
 			   osl::ntesuki::NtesukiRecord::tonshi_is,
 			   0);
 
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn());
+  const int ntesuki_num = searcher.searchSlow(nState.turn());
   CPPUNIT_ASSERT_EQUAL(-1, ntesuki_num);
 }
 
@@ -329,7 +328,7 @@ void NtesukiSearcherTest::testRecognizeAttackBack1()
   boost::scoped_ptr<osl::ntesuki::NtesukiMoveGenerator>
     gam(new movegen_t());
   NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT,
-			   &stop_flag, !isShortTest, 2,
+			   &stop_flag, OslConfig::verbose(), 2,
 			   osl::ntesuki::NtesukiRecord::pn_iw,
 			   osl::ntesuki::NtesukiRecord::pn_ps,
 			   osl::ntesuki::NtesukiRecord::tonshi_is,
@@ -344,7 +343,7 @@ void NtesukiSearcherTest::testRecognizeAttackBack1()
   NtesukiRecord::use_dominance = true;
   NtesukiRecord::fixed_search_depth = 4;
 
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
+  const int ntesuki_num = searcher.searchSlow(nState.turn(), READ_LIMIT);
   if (!searcher.exceedReadNodeLimit())
   {
     CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
@@ -352,7 +351,7 @@ void NtesukiSearcherTest::testRecognizeAttackBack1()
 
   const NtesukiTable& table = searcher.getTable();
   const HashKey key = HashKey(state).
-    newHashWithMove(Move(Position(6,2), BISHOP, BLACK));
+    newHashWithMove(Move(Square(6,2), BISHOP, BLACK));
   const NtesukiRecord *record = table.find(key);
   if (record)
   {
@@ -371,7 +370,7 @@ test_state(const char *csa_filename)
 {
   CsaFile file(csa_filename);
   SimpleState init_state = file.getInitialState ();
-  Player winner = init_state.getTurn();
+  Player winner = init_state.turn();
   const osl::vector<Move> moves=file.getRecord().getMoves();
   
   std::cerr << "for " << csa_filename << "\n";
@@ -387,7 +386,7 @@ test_state(const char *csa_filename)
       gam(new movegen_t());
     NumEffectState nState(state);
     NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT,
-			     &stop_flag, !isShortTest, 2,
+			     &stop_flag, OslConfig::verbose(), 2,
 			     NtesukiRecord::pn_iw,
 			     NtesukiRecord::no_ps,
 			     NtesukiRecord::no_is,
@@ -427,8 +426,8 @@ test1tesukiTest22()
     gam(new movegen_t());
   NumEffectState nState(state);
 
-  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
+  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, OslConfig::verbose());
+  const int ntesuki_num = searcher.searchSlow(nState.turn(), READ_LIMIT);
   CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
 }
 
@@ -443,8 +442,8 @@ test1tesukiTest13()
     gam(new movegen_t());
   NumEffectState nState(state);
 
-  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
+  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, OslConfig::verbose());
+  const int ntesuki_num = searcher.searchSlow(nState.turn(), READ_LIMIT);
   CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
 }
 
@@ -459,8 +458,8 @@ test1tesukiTest25()
     gam(new movegen_t());
   NumEffectState nState(state);
 
-  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
+  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, OslConfig::verbose());
+  const int ntesuki_num = searcher.searchSlow(nState.turn(), READ_LIMIT);
   CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
 }
 
@@ -475,8 +474,8 @@ test1tesukiTest29()
     gam(new movegen_t());
   NumEffectState nState(state);
 
-  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
+  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, OslConfig::verbose());
+  const int ntesuki_num = searcher.searchSlow(nState.turn(), READ_LIMIT);
   CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
 }
 
@@ -491,8 +490,8 @@ test1tesukiTest56()
     gam(new movegen_t());
   NumEffectState nState(state);
 
-  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
+  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, OslConfig::verbose());
+  const int ntesuki_num = searcher.searchSlow(nState.turn(), READ_LIMIT);
   CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
 }
 
@@ -507,8 +506,8 @@ test1tesukiTest76()
     gam(new movegen_t());
   NumEffectState nState(state);
 
-  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
+  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, OslConfig::verbose());
+  const int ntesuki_num = searcher.searchSlow(nState.turn(), READ_LIMIT);
   CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
 }
 
@@ -523,8 +522,8 @@ test1tesukiTest94()
     gam(new movegen_t());
   NumEffectState nState(state);
 
-  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, !isShortTest);
-  const int ntesuki_num = searcher.searchSlow(nState.getTurn(), READ_LIMIT);
+  NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT, &stop_flag, OslConfig::verbose());
+  const int ntesuki_num = searcher.searchSlow(nState.turn(), READ_LIMIT);
   CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
 }
 
@@ -550,10 +549,10 @@ void NtesukiSearcherTest::testKatagyoku0()
     gam(new movegen_t());
   NumEffectState nState(state);
   NtesukiSearcher searcher(nState, gam.get(), TABLE_LIMIT,
-			   &stop_flag, !isShortTest);
+			   &stop_flag, OslConfig::verbose());
 
   const int ntesuki_num =
-    searcher.searchSlow(nState.getTurn());
+    searcher.searchSlow(nState.turn());
 
   CPPUNIT_ASSERT_EQUAL(1, ntesuki_num);
 }
